#############################################################################
# srptinfo.tcl Copyright 2004, 2005, 2006 r0lZ
# show/edit VMG_TT_SRPT table.
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

namespace eval ::srptinfo {

namespace export srptinfo

proc srptinfo {} {
	##### get the table of titles from the VMG_TT_SRPT
	if {(! [info exists ::dvddir]) || ($::raw_sectors(0,VMG_TT_SRPT) == "")} {
		bell
		return
	}

	set w ".srptinfo"
	if {![winfo exists $w]} {
		toplevel $w -class Dialog
		wm withdraw $w
		wm protocol $w WM_DELETE_WINDOW {set ::srptinfo::done 0}
		bind $w <Escape> "set ::srptinfo::done 0"
		bind $w <Return> "set ::srptinfo::done 1"
		if {$::config(tk) != "aqua"} {wm transient $w .}
		if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
		wm title $w "Title Play Map table"

		frame  $w.mf -borderwidth 2 -relief groove
		pack   $w.mf -padx 4 -pady 4 -expand 1 -fill both
		canvas $w.mf.c -yscrollcommand "$w.mf.vs set" -height 10
		pack   $w.mf.c -side left -expand 1 -fill both
		scrollbar $w.mf.vs -command "$w.mf.c yview" -orient vert
		pack   $w.mf.vs -side left -fill y
		frame  $w.mf.c.f1	;# destroyed later

		# buttons
		frame  $w.fb
		pack   $w.fb -padx 2 -pady 4 -fill x
		button $w.fb.cl -text "Clear Prohibited User Operations" \
				-command {::srptinfo::clear_puos}
		pack   $w.fb.cl -side left -padx 2
		frame  $w.fb.pad
		pack   $w.fb.pad -expand 1 -fill x -side left
		button $w.fb.ok -text "OK" -width 10 -command {set ::srptinfo::done 1}
		pack   $w.fb.ok -side left -padx 3
		button $w.fb.cancel -text "Cancel" -width 10 -command {set ::srptinfo::done 0}
		pack   $w.fb.cancel -side left -padx 2
		if {$::config(platform) == "mac"} {
			pack [frame $w.fb.macpad] -side left -padx 6
		}

		set win_created true
	} else {
		set win_created false
	}

	destroy $w.mf.c.f1
	frame $w.mf.c.f1
	pack  $w.mf.c.f1 -anchor nw -fill both -expand 1
	$w.mf.c create window 0 0 -window $w.mf.c.f1 -anchor nw

	label $w.mf.c.f1.w_0_0 -width 8 -text "Title"
	label $w.mf.c.f1.w_0_1 -width 8 -text "VTS"
	label $w.mf.c.f1.w_0_2 -width 8 -text "TTN"
	label $w.mf.c.f1.w_0_3 -width 8 -text "Chapters\n(PTTs)"
	label $w.mf.c.f1.w_0_4 -width 8 -text "Angles"
	label $w.mf.c.f1.w_0_5 -width 8 -text "Parental\nmanag.\nmask"
	label $w.mf.c.f1.w_0_6 -width 8 -text "Sequential\nTitle"
	label $w.mf.c.f1.w_0_7 -width 8 -text "PUO:\nTime play\nor search"
	label $w.mf.c.f1.w_0_8 -width 8 -text "PUO:\nPTT play\nor search"

	set data $::raw_sectors(0,VMG_TT_SRPT)
	set ::srptinfo::numtitles [get_int data 0 2]
	for {set t 1} {$t <= $::srptinfo::numtitles} {incr t} {
		set t_idx [expr {8+(($t-1)*12)}]

		# Title type flags
		set type [get_int data $t_idx 1]
		set ::srptinfo::title($t,uop0) [expr { $type & 1}]
		set ::srptinfo::title($t,uop1) [expr {($type & 2) / 2}]
		if {[expr {($type & 0X40) / 0X40}]} {
			set sequential "no"
		} else {
			set sequential "yes"
		}

		label $w.mf.c.f1.w_${t}_0 -text $t
		label $w.mf.c.f1.w_${t}_1 -text [get_int data [expr {$t_idx+6}] 1]
		label $w.mf.c.f1.w_${t}_2 -text [get_int data [expr {$t_idx+7}] 1]
		label $w.mf.c.f1.w_${t}_3 -text [get_int data [expr {$t_idx+2}] 2]
		label $w.mf.c.f1.w_${t}_4 -text [get_int data [expr {$t_idx+1}] 1]
		label $w.mf.c.f1.w_${t}_5 -text [get_int data [expr {$t_idx+4}] 2]
		label $w.mf.c.f1.w_${t}_6 -text $sequential
		checkbutton $w.mf.c.f1.w_${t}_7 -variable ::srptinfo::title($t,uop0)
		checkbutton $w.mf.c.f1.w_${t}_8 -variable ::srptinfo::title($t,uop1)
	}
	for {set t 0} {$t <= $::srptinfo::numtitles} {incr t} {
		for {set f 0} {$f <= 8} {incr f} {
			grid $w.mf.c.f1.w_${t}_$f -row $t -column $f
		}
	}

	# inform scrollbar of the height of the canvas's internal window
	update
	$w.mf.c configure -scrollregion "0 0 0 [winfo height $w.mf.c.f1]" \
			-width [winfo width $w.mf.c.f1]

	# window geometry
	if {$win_created} {
		set maxheight [winfo height $w.mf.c.f1]
		if {$maxheight > 304} {set maxheight 304}
		$w.mf.c configure -height $maxheight
		update
		set ww [winfo reqwidth $w]
		set wh [winfo reqheight $w]
		set wx [expr {[winfo rootx .]+(([winfo width  .]-$ww)/2)}]
		set wy [expr {[winfo rooty .]+(([winfo height .]-$wh)/2)-25}]
		if {$wx < 10} {set wx 10}
		if {$wy < 22} {set wy 22}
		$w.mf.c configure -height 1
		wm geometry $w ${ww}x$wh+$wx+$wy
		wm resizable $w 0 1
	}

	wm deiconify $w
	focus $w
	grab set $w

	set ::srptinfo::done -1
	tkwait variable ::srptinfo::done
	grab release $w
	destroy $w

	if {$::srptinfo::done} {
		::main::dvd_modified true false
		for {set t 1} {$t <= $::srptinfo::numtitles} {incr t} {
			set t_idx [expr {8+(($t-1)*12)}]

			# Title type flags
			set type [get_int data $t_idx 1]
			if {$::srptinfo::title($t,uop0)} {
				set type [expr {$type | 1}]
			} else {
				set type [expr {$type & 0XFE}]
			}
			if {$::srptinfo::title($t,uop1)} {
				set type [expr {$type | 2}]
			} else {
				set type [expr {$type & 0XFD}]
			}
			set_int ::raw_sectors(0,VMG_TT_SRPT) $t_idx $type 1
		}
	}
	return
}

proc clear_puos {} {
	for {set t 0} {$t <= $::srptinfo::numtitles} {incr t} {
		set ::srptinfo::title($t,uop0) 0
		set ::srptinfo::title($t,uop1) 0
	}
}

proc remap_titles_numbers {} {
	set w ".srptremap"

	set data $::raw_sectors(0,VMG_TT_SRPT)
	set numtitles [get_int data 0 2]
	if {$numtitles < 2} {
		bell
		return
	}

	set remap_titles {}
	for {set t 1} {$t <= $numtitles} {incr t} {
		set t_idx [expr {8+(($t-1)*12)}]
		lappend remap_titles \
				[format "%3d : old Title %3d , VTS %2d, TTN %2d (%d chapters)" \
				$t $t [get_int data [expr {$t_idx+6}] 1] [get_int data [expr {$t_idx+7}] 1] \
				[get_int data [expr {$t_idx+2}] 2]]
	}

	set searchstring "sprm(4:Title number in volume)"


	set refs 0
	set progress_max [expr ($::pgcs(numvts)*2)+2]
	set found false
	for {set vts 0} {$vts <= $::pgcs(numvts)} {incr vts} {
		show_msg "Searching for \"$searchstring\" in VTSM $vts" [expr ($vts*2)+1] $progress_max
		for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
				if {[::main::find_in_cmds $searchstring $::menus($vts,$lu,$pgc) \
						-forwards false false] >= 0} {
					set found true
					break
				}
			}
			if {$found} break
		}
		if {$found} break
		if {$vts > 0} {
			show_msg "Searching for \"$searchstring\" in VTST $vts" [expr ($vts*2)+2] $progress_max
			for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
				if {[::main::find_in_cmds $searchstring $::pgcs($vts,$pgc) \
						-forwards false false] >= 0} {
					set found true
					break
				}
			}
		}
		if {$found} break
	}
	show_msg ""
	if {$found} {
		set txt "Warning: SPRM 4 (Title number in volume) is used\n"
		set txt "${txt}in the VM commands of this DVD.\n"
		set txt "${txt}Remapping the title numbers can be dangerous!\n"
		set txt "${txt}(If you continue, you will have to fix the commands\n"
		set txt "${txt}manually.  Search for \"sprm(4:\" in the DVD!)\n"
		set txt "${txt}\nOK to continue anyway?"
		set r [tk_messageBox -title "Remap Title numbers" -icon warning \
			-message $txt -parent . -type yesno -default no]
		if {$r == "no"} return
	}


	set remap_titles [::utils::remap_gui $w \
			"Remap Title numbers" "This function renumbers the Titles safely!\nThe targets of the Jump_TT commands will be renumbered automatically in the VMG,\nincluding, if needed, in the menu buttons, when you hit OK." \
			$remap_titles]
	if {$remap_titles != ""} {
		set data [string range $::raw_sectors(0,VMG_TT_SRPT) 0 7]
		for {set t 0} {$t < $numtitles} {incr t} {
			set orig [lindex [lindex $remap_titles $t] 4]
			set t_idx [expr {8+(($orig-1)*12)}]
			set data "$data[string range $::raw_sectors(0,VMG_TT_SRPT) $t_idx [expr $t_idx+11]]"
			set remap($orig) [expr $t+1]
		}
		set ::raw_sectors(0,VMG_TT_SRPT) $data
		# fix Jump_TT commands in VMG

		foreach i [lsearch -all -regexp $::pgcs(0,0) "30 .2 .. .. .. .. .. .."] {
			set cmd [lindex $::pgcs(0,0) $i]
			set origtarget [x2d [string range $cmd 15 16]]
			if {[info exists remap($origtarget)]} {
				set cmd [string range $cmd 0 14][format %02X $remap($origtarget)][string range $cmd 17 end]
				lset ::pgcs(0,0) $i $cmd
			}
		}
		for {set lu 1} {$lu <= $::menus(0,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus(0,$lu,numpgcs)} {incr pgc} {
				foreach i [lsearch -all -regexp $::menus(0,$lu,$pgc) "30 .2 .. .. .. .. .. .."] {
					set cmd [lindex $::menus(0,$lu,$pgc) $i]
					set origtarget [x2d [string range $cmd 15 16]]
					if {[info exists remap($origtarget)]} {
						set cmd [string range $cmd 0 14][format %02X $remap($origtarget)][string range $cmd 17 end]
						lset ::menus(0,$lu,$pgc) $i $cmd
					}
				}
				set numcells $::menus(0,$lu,$pgc,numcells)
				if {$numcells > 0} {
					set cellpostable $::menus(0,$lu,$pgc,cellpostable)
					for {set cell 1} {$cell <= $numcells} {incr cell} {
						set idx [expr {($cell-1) * 4}]
						set vobid [get_int cellpostable $idx 2]
						incr idx 3
						set cellid [get_int cellpostable $idx 1]
						if {[info exists ::buttons::butsinfo(menu,0,$vobid,$cellid,numbuts)]} {
							set numbuts $::buttons::butsinfo(menu,0,$vobid,$cellid,numbuts)
							if {$numbuts > 0} {
								set numgroups $::buttons::butsinfo(menu,0,$vobid,$cellid,numgroups)
								if {$numgroups > 0} {
									for {set butnum 1} {$butnum <= $numbuts} {incr butnum} {
										for {set group 1} {$group <= $numgroups} {incr group} {
											set cmd [string toupper $::buttons::butsinfo(menu,0,$vobid,$cellid,$group,$butnum,command)]
											if {[lsearch -regexp [list $cmd] "30 .2 .. .. .. .. .. .."] > -1} {
												set origtarget [x2d [string range $cmd 15 16]]
												if {[info exists remap($origtarget)]} {
													set cmd [string range $cmd 0 14][format %02X $remap($origtarget)][string range $cmd 17 end]
													set ::buttons::butsinfo(menu,0,$vobid,$cellid,$group,$butnum,command) $cmd
													set ::buttons::butsinfo(menu,edited) 1
													set ::buttons::butsinfo(menu,0,edited) 1
													set ::buttons::butsinfo(menu,0,$vobid,edited) 1
													set ::buttons::butsinfo(menu,0,$vobid,$cellid,edited) 1													
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		for {set vts 1} {$vts <= $::pgcs(numvts)} {incr vts} {
			for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
				if {[info exists ::pgcs($vts,$pgc,Title)]} {
					if {[info exists remap($::pgcs($vts,$pgc,Title))]} {
						set ::pgcs($vts,$pgc,Title) $remap($::pgcs($vts,$pgc,Title))
					}
				}
			}
		}
		for {set t 1} {$t <= $::pgcs(Title,NumTitles)} {incr t} {
			catch {set tmp($remap($t)) $::pgcs(Title,$t)}
		}
		for {set t 1} {$t <= $::pgcs(Title,NumTitles)} {incr t} {
			catch {set ::pgcs(Title,$t) $tmp($t)}
		}
		::main::init_tree_menu_no_move
		::main::dvd_modified true false
	}
}

proc srptinfo_dump {} {
	##### get the table of titles from the VMG_TT_SRPT
	if {(! [info exists ::dvddir]) || ($::raw_sectors(0,VMG_TT_SRPT) == "")} {
		bell
		return
	}

	set txt "Title Play Map table\n"
	set txt "${txt}DVD-TEXT General Name: \"[::utils::get_General_Name]\"\n"
	set txt "${txt}Provider ID: \"[::utils::get_provider_ID]\"\n"
	set txt "${txt}Number of VTS: $::pgcs(numvts)\n\n"

	set txt "${txt}Title  VTS   TTN  Chapters Angles Parental   Sequential    PUO:        PUO:\n"
	set txt "${txt}                   (PTTs)         Managment    Title     Time Play   PTT Play\n"
	set txt "${txt}                                  Mask (hex)             or search   or search\n\n"

	set data $::raw_sectors(0,VMG_TT_SRPT)
	set numtitles [get_int data 0 2]
	for {set t 1} {$t <= $numtitles} {incr t} {
		set t_idx [expr {8+(($t-1)*12)}]

		# Title type flags
		set type [get_int data $t_idx 1]
		set uop0 "allowed"
		set uop1 "allowed"
		if {[expr { $type & 1}] > 0} {set uop0 "prohibited"}
		if {[expr { $type & 2}] > 0} {set uop1 "prohibited"}
		set title($t,uop1) [expr {($type & 2) / 2}]
		if {[expr {($type & 0X40) / 0X40}]} {
			set sequential "no"
		} else {
			set sequential "yes"
		}
		set txt $txt[format "%3d    %2d    %2d     %3d    %3d     0x%04X       %-3s      %-10s  %-10s" \
				$t [get_int data [expr {$t_idx+6}] 1] [get_int data [expr {$t_idx+7}] 1] \
				[get_int data [expr {$t_idx+2}] 2] [get_int data [expr {$t_idx+1}] 1] \
				[get_int data [expr {$t_idx+4}] 2] $sequential $uop0 $uop1 ]\n
	}
	::main::showtext .srptdump "Title Play Map Table" $txt
	return
}

} ;# end namespace srptinfo
