#############################################################################
# utils.tcl Copyright 2004, 2005, 2006 r0lZ
# Procedures related to the Utilities menu.
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

namespace eval ::utils {
namespace export utils_menu
namespace import ::readdvd::*



#############################################################################
## utility procedures

#############################################################################
## Change the DVD Region Code
proc set_region_code {} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}

	set regions {
		{USA, Canada, U.S. Territories}
		{Europe, Japan, South Africa, Middle East}
		{Southeast Asia and East Asia (including Hong Kong)}
		{Australia, New Zealand, Latin America, Pacific Islands}
		{Eastern Europe, Russia, India, Africa, North Korea, Mongolia}
		{China}
		{reserved}
		{Special international venues (airplanes, cruise ships...)}
	}

	set code [expr {[get_int ::raw_sectors(0,VMGM_MAT) 0X23 1] ^ 0XFF}]

	# init Search flag
	if {![info exists ::utils::regcode_search]} {
		set ::utils::regcode_search 1
	}

	if {[winfo exists .regioncode]} {destroy .regioncode}
	set w .regioncode
	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w "Region Code"
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm protocol $w WM_DELETE_WINDOW {set ::utils::regioncode_done 0}
	bind $w <Escape> "set ::utils::regioncode_done 0"
	bind $w <Return> "set ::utils::regioncode_done 1"

	frame $w.f -bd 2 -relief groove
	pack  $w.f -padx 4 -pady 3 -fill x
	label $w.f.lab -text "Allowed regions:"
	pack  $w.f.lab -anchor w
	for {set i 0} {$i < 8} {incr i} {
		set bitmask [expr {round(pow(2,$i))}]
		set ::utils::regcode_$i [expr {$code & $bitmask}]
		checkbutton $w.f.c$i -text "[expr {$i+1}] (val:$bitmask) [lindex $regions $i]" \
				-variable ::utils::regcode_$i -onvalue [expr {$bitmask}] -bd $::config(min_bd) -anchor w
		pack $w.f.c$i -fill x -anchor w
	}
	button $w.f.cl -text "  Region Free  " \
		-command {for {set i 0} {$i < 8} {incr i} {set ::utils::regcode_$i [expr {round(pow(2,$i))}]}}
	pack   $w.f.cl -padx 2 -pady 4 -anchor w

	frame $w.f2 -bd 2 -relief groove
	pack  $w.f2 -padx 4 -pady 3 -fill x

	set txt "RCE (Region Code Enhanced) protection:"
	set txt "$txt\nThe region code of your player may be tested by programmation."
	set txt "$txt\nIf it doesn't match the DVD zone, a jump to a PGC that will inhibit the"
	set txt "$txt\nplayback or return to the menu is usually done."
	set txt "$txt\nYou may try to replace the sprm(20) reference(s) by the constant value"
	set txt "$txt\n(between parenthesis above) matching the original DVD region code,"
	set txt "$txt\nor replace the command by NOP, or modify the goto or the jump target."
	label $w.f2.lab -text $txt -anchor w -justify left
	pack  $w.f2.lab -pady 2 -fill x -anchor w

	set txt "Search for \"sprm(20:Player region code)\" in the DVD commands?"
	checkbutton $w.f2.s -text $txt -variable ::utils::regcode_search -anchor w -bd $::config(min_bd)
	pack  $w.f2.s -pady 2 -fill x

	# buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -anchor e
	button $w.fb.ok -text "OK" -width 10 -command {set ::utils::regioncode_done 1}
	pack   $w.fb.ok -side left -padx 2
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::utils::regioncode_done 0}
	pack   $w.fb.cancel -side left -padx 2

	update
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm maxsize $w [winfo reqwidth $w] [winfo reqheight $w]
	set wx [expr {[winfo rootx .]+(([winfo width  .]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm deiconify $w
	focus $w
	grab set $w

	tkwait variable ::utils::regioncode_done

	if {$::utils::regioncode_done} {
		set code 0
		for {set i 0} {$i < 8} {incr i} {
			set bitmask [expr {round(pow(2,$i))}]
			set code [expr {$code | [subst \$::utils::regcode_$i]}]
		}
		set_int ::raw_sectors(0,VMGM_MAT) 0X23 [expr {$code ^ 0XFF}] 1
		::main::dvd_modified true false
	}

	grab release $w
	destroy $w

	if {$::utils::regioncode_done && $::utils::regcode_search} {
		find_results {sprm(20:Player region code)} false false
	}
	return
}

#############################################################################
## Filter prohibited user operations
proc filter_puos {} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	set utiltitle "Filter Prohibited User Operations"

	if {!$::config(expert_mode)} {
		set txt "This utility will selectively clear the PUOs"
		set txt "$txt\nin all PGCs of the VMGM Menus,"
		set txt "$txt\nin all PGCs of the VTSM Menus,"
		set txt "$txt\nin all PGCs of the VTST Titles,"
		set txt "$txt\nand in the VMG Title Play Map Table."
		set txt "$txt\n"
		set txt "$txt\nContinue ?"
		set r [tk_messageBox -title $utiltitle -icon question -message $txt \
				-parent . -type yesno -default no]
		if {$r == "no"} return
	}

	# VMG Menus PUOs
	set puos 0
	for {set lu 1} {$lu <= $::menus(0,numlus)} {incr lu} {
		for {set pgc 1} {$pgc <= $::menus(0,$lu,numpgcs)} {incr pgc} {
			set puos [expr {$puos | [get_int ::menus(0,$lu,$pgc,header) 8 4]}]
		}
	}
	if {$puos == 0} {
		show_warning -title $utiltitle -icon info -parent . \
				-message "No Prohibited User Operations found in the VMGM Menus."
	} else {
		set ::pgced::puos $puos
		set filterpuos [::pgced::puos_editor "Filter PUOs for all VMGM Menus :\nClear the PUOs you want to clear,\nSet the PUOs you don't want to change." .]
		if {$filterpuos != $puos} {
			::main::dvd_modified true false
			for {set lu 1} {$lu <= $::menus(0,numlus)} {incr lu} {
				for {set pgc 1} {$pgc <= $::menus(0,$lu,numpgcs)} {incr pgc} {
					set_int ::menus(0,$lu,$pgc,header) 8 \
							[expr {$filterpuos & [get_int ::menus(0,$lu,$pgc,header) 8 4]}] 4
				}
			}
		}
	}

	# VTS Menus PUOs
	set puos 0
	for {set vts 1} {$vts <= $::menus(numvts)} {incr vts} {
		for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
				set puos [expr {$puos | [get_int ::menus($vts,$lu,$pgc,header) 8 4]}]
			}
		}
	}
	if {$puos == 0} {
		show_warning -title $utiltitle -icon info -parent . \
				-message "No Prohibited User Operations found in the VTSM Menus."
	} else {
		set ::pgced::puos $puos
		set filterpuos [::pgced::puos_editor "Filter PUOs for all VTSM Menus :\nClear the PUOs you want to clear,\nSet the PUOs you don't want to change." .]
		if {$filterpuos != $puos} {
			::main::dvd_modified true false
			for {set vts 1} {$vts <= $::menus(numvts)} {incr vts} {
				for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
					for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
						set_int ::menus($vts,$lu,$pgc,header) 8 \
								[expr {$filterpuos & [get_int ::menus($vts,$lu,$pgc,header) 8 4]}] 4
					}
				}
			}
		}
	}

	# Titles PUOs
	set puos 0
	for {set vts 1} {$vts <= $::pgcs(numvts)} {incr vts} {
		for {set ttn 1} {$ttn <= $::pgcs($vts,numpgcs)} {incr ttn} {
			set puos [expr {$puos | [get_int ::pgcs($vts,$ttn,header) 8 4]}]
		}
	}
	if {$puos == 0} {
		show_warning -title $utiltitle -icon info -parent . \
				-message "No Prohibited User Operations found in the VTST Title's PGCs."
	} else {
		set ::pgced::puos $puos
		set filterpuos [::pgced::puos_editor "Filter PUOs for all VTST Titles :\nClear the PUOs you want to clear,\nSet the PUOs you don't want to change." .]
		if {$filterpuos != $puos} {
			::main::dvd_modified true false
			for {set vts 1} {$vts <= $::pgcs(numvts)} {incr vts} {
				for {set ttn 1} {$ttn <= $::pgcs($vts,numpgcs)} {incr ttn} {
					set_int ::pgcs($vts,$ttn,header) 8 \
							[expr {$filterpuos & [get_int ::pgcs($vts,$ttn,header) 8 4]}] 4
				}
			}
		}
	}

	# VMG_TT_SRPT PUOs
	set numtitles [get_int ::raw_sectors(0,VMG_TT_SRPT) 0 2]
	set puos 0
	for {set t 1} {$t <= $numtitles} {incr t} {
		set t_idx [expr {8+(($t-1)*12)}]
		set puos [expr {$puos | ([get_int ::raw_sectors(0,VMG_TT_SRPT) $t_idx 1] & 3)}]
	}
	if {$puos == 0} {
		show_warning -title $utiltitle -icon info -parent . \
				-message "No Prohibited User Operations found in the Title Play Map Table."
	} else {
		set filterpuos $puos
		if {[expr {$puos & 1}]} {
			set txt "At least one Title has the \"Time Play or Search\" prohibited"
			set txt "$txt\nin the VMG Title Play Map Table."
			set txt "$txt\n"
			set txt "$txt\nClear this prohibition for all Titles ?"
			set r [tk_messageBox -title $utiltitle -icon question -message $txt \
					-parent . -type yesno -default no]
			if {$r == "yes"} {set filterpuos [expr {$filterpuos & 2}]}
		}
		if {[expr {$puos & 2}]} {
			set txt "At least one Title has the \"PTT Play or Search\" prohibited"
			set txt "$txt\nin the VMG Title Play Map Table."
			set txt "$txt\n"
			set txt "$txt\nClear this prohibition for all Titles ?"
			set r [tk_messageBox -title $utiltitle -icon question -message $txt \
					-parent . -type yesno -default no]
			if {$r == "yes"} {set filterpuos [expr {$filterpuos & 1}]}
		}
		if {$puos != $filterpuos} {
			::main::dvd_modified true false
			set mask [expr {$filterpuos | 0XFC}]
			for {set t 1} {$t <= $numtitles} {incr t} {
				set t_idx [expr {8+(($t-1)*12)}]
				set byte [expr {[get_int ::raw_sectors(0,VMG_TT_SRPT) $t_idx 1] & $mask}]
				set_int ::raw_sectors(0,VMG_TT_SRPT) $t_idx $byte 1
			}
		}
	}
}

#############################################################################
## Import VTS Titles
proc import_VTST {} {
	set utiltitle "Import VTST"
	if {! [info exists ::dvddir]} {
		bell
		return 0
	}
	if {$::pgcs(numvts) >= 99} {
		show_warning -title $utiltitle -icon error -parent . \
				-message "Maximum number of VTS (99) reached!"
		return 0
	}

	set sourceifofile [tk_getOpenFile -initialdir $::config(importmenuinitialdir) \
		-filetypes {{"VTS IFO files" {*_0.IFO}}} \
		-title "Select the VTS IFO file with the title(s) to import"]
	if {$sourceifofile == ""} {return 0}
	if {[file tail $sourceifofile] == "VIDEO_TS.IFO"} {
		show_warning -title $utiltitle -icon error -parent . \
				-message "You must select a VTS IFO (VTS_**_*.IFO)"
		return 0
	}
	set sourceifofile [file nativename $sourceifofile]
	set ::config(importmenuinitialdir) [file dirname $sourceifofile]

	# read the VTS
	set sourcevtsnum [readdvdfile $sourceifofile ::imported_sectors ::imported_pgcs ::imported_menus ::imported_ttns]
	if {$sourcevtsnum == -1} {return 0}
	if {$sourcevtsnum == 0} {
		show_warning -title $utiltitle -icon error -parent . \
				-message "You must select a VTS IFO (VTS_**_*.IFO)"
		# cleanup
		unset ::imported_sectors ::imported_pgcs ::imported_menus
		return 0
	}

	# How many TTNs are in the imported VTS?
	set num_new_ttns [get_int ::imported_sectors($sourcevtsnum,VTS_PTT_SRPT) 0 2]
	if {$num_new_ttns == 0} {
		show_warning -title $utiltitle -icon error -parent . \
				-message "Error: there are no titles defined in VTS file\n\"$sourceifofile\""
		# cleanup
		unset ::imported_sectors ::imported_pgcs ::imported_menus
		return 0
	}

	# read the source VIDEO_TS.IFO to get the VMG_TT_SRPT table.
	set vmgfile [file join [file dirname $sourceifofile] "VIDEO_TS.IFO"]
	set src_vmg_tt_srpt ""
	if {[file exists $vmgfile]} {
		if {[readdvdfile $vmgfile ::ivmg_sectors ::ivmg_pgcs ::ivmg_menus ::ivmg_ttns] == 0} {
			set src_vmg_tt_srpt $::ivmg_sectors(0,VMG_TT_SRPT)
			unset ::ivmg_sectors ::ivmg_pgcs ::ivmg_menus
		}
	}
	if {$src_vmg_tt_srpt == ""} {
		set txt "WARNING:\nCan't find or load the source VMG (VIDEO_TS.IFO)!\n"
		set txt "$txt\nThe VMG Table of Titles will be"
		set txt "$txt\ninitialized with default values."
		set r [tk_messageBox -title $utiltitle -icon warning -parent . -type okcancel \
				-default cancel -message $txt]
		if {$r == "cancel"} {
			# cleanup
			unset ::imported_sectors ::imported_pgcs ::imported_menus
			return 0
		}
	}

	### Go modify the DVD!
	# init the new VTS sectors
	set newvtsnum [expr {$::pgcs(numvts) + 1}]
	::readdvd::parse_VTS_PTT_SRPT ::imported_sectors($sourcevtsnum,VTS_PTT_SRPT) $newvtsnum ::imported_pgcs ::imported_ttns
	set ::raw_sectors($newvtsnum,VTSI_MAT) \
			$::imported_sectors($sourcevtsnum,VTSI_MAT)
	set ::raw_sectors($newvtsnum,VTS_PTT_SRPT) \
			$::imported_sectors($sourcevtsnum,VTS_PTT_SRPT)
	set ::raw_sectors($newvtsnum,VTS_PGCI) \
			$::imported_sectors($sourcevtsnum,VTS_PGCI)
	set ::raw_sectors($newvtsnum,VTS_TMAPTI) \
			$::imported_sectors($sourcevtsnum,VTS_TMAPTI)
	set ::raw_sectors($newvtsnum,VTS_C_ADT) \
			$::imported_sectors($sourcevtsnum,VTS_C_ADT)
	set ::raw_sectors($newvtsnum,VTS_VOBU_ADMAP) \
			$::imported_sectors($sourcevtsnum,VTS_VOBU_ADMAP)

	# clear the menu sectors
	set ::raw_sectors($newvtsnum,VTSM_PGCI_UT) ""
	set ::raw_sectors($newvtsnum,VTSM_C_ADT) ""
	set ::raw_sectors($newvtsnum,VTSM_VOBU_ADMAP) ""

	# cleanup
	unset ::imported_sectors ::imported_pgcs ::imported_menus

	# Init variables and parse the new TTNs
	set ::pgcs(numvts)  $newvtsnum
	set ::menus(numvts) $newvtsnum
	set_int ::raw_sectors(0,VMGM_MAT) 62 $newvtsnum 2
	set ::menus($newvtsnum,numlus) 0
	::readdvd::parse_VTS_PGCI ::raw_sectors($newvtsnum,VTS_PGCI) $newvtsnum ::pgcs
	::readdvd::parse_VTS_PTT_SRPT ::raw_sectors($newvtsnum,VTS_PTT_SRPT) $newvtsnum ::pgcs ::ttns

	# Add new Titles to the TT_SRPT Title Table.
	# If the original table was read, copy the relevent entries,
	# otherwise, set default values (except for PTTs value)
	set old_num_titles [get_int ::raw_sectors(0,VMG_TT_SRPT) 0 2]
	set data [string range $::raw_sectors(0,VMG_TT_SRPT) 0 [expr 8+($old_num_titles*12)-1]]
	if {$src_vmg_tt_srpt != ""} {
		set src_num_titles [get_int src_vmg_tt_srpt 0 2]
		set num_new_entries 0
		for {set t 1} {$t <= $src_num_titles} {incr t} {
			set startidx [expr {($t * 12) - 4}]
			set endidx   [expr {$startidx + 11}]
			if {$endidx > [string length $src_vmg_tt_srpt]} break
			set entry [string range $src_vmg_tt_srpt $startidx $endidx]
			set_int entry 4 0 2	;# clear the parental ID mask
			set vtsn [get_int entry 6 1]
			if {$vtsn == $sourcevtsnum} {
				set_int entry 6 $newvtsnum 1
				set data "$data$entry"
				incr num_new_entries
			}
		}
		set new_num_titles [expr {$old_num_titles + $num_new_entries}]
		set_int data 0 $new_num_titles 2
		set_int data 4 [expr {[string length $data] -1}] 4
		set ::raw_sectors(0,VMG_TT_SRPT) $data
	} else {
		set new_num_titles [expr {$old_num_titles + $num_new_ttns}]
		set_int data 0 $new_num_titles 2
		set ::pgcs(Title,NumTitles) $new_num_titles
		set vtssrpt $::raw_sectors($newvtsnum,VTS_PTT_SRPT)
		for {set t 1} {$t <= $num_new_ttns} {incr t} {
			# TODO: Set real values instead of Type=Sequential, Angles=1, parental management=0
			# get number of PTTs from the VTS_PTT_SRPT table.
			set entry "\074\001\000\000\000\000\000\000\000\000\000\000"
			# set number of chapters from the VTS_PTT_SRPT
			if {$vtssrpt != ""} {
				set numttus [get_int vtssrpt 0 2]
				set idx [get_int vtssrpt [expr {4 + ($t * 4)}] 4]
				set pgcn [get_int vtssrpt $idx 2]
				set num_ptts 0
				while {$pgcn == $t} {
					incr num_ptts
					incr idx 4
					if {$idx >= [string length $vtssrpt]} break
					set pgcn [get_int vtssrpt $idx 2]
				}
			}
			set_int entry 2 $num_ptts 2
			set_int entry 6 $newvtsnum 1
			set_int entry 7 $t 1
			set data "$data$entry"
		}
		set_int data 4 [expr {[string length $data] -1}] 4
		set ::raw_sectors(0,VMG_TT_SRPT) $data
	}

	# get the table of titles from the VMG_TT_SRPT again
	# (to be able to display the title number in the PGC selectors)
	set ::pgcs(Title,NumTitles) $new_num_titles
	for {set t 1} {$t <= $new_num_titles} {incr t} {
		set t_idx [expr {14+(($t-1)*12)}]
		set ::pgcs([get_int data $t_idx 1],$::ttns([get_int data $t_idx 1],[get_int data [expr {$t_idx+1}] 1]),Title) $t
		set ::pgcs(Title,$t) [list [get_int data $t_idx 1] $::ttns([get_int data $t_idx 1],[get_int data [expr {$t_idx+1}] 1])]
	}

	# Reinit GUI
	::main::init_tree_menu
	::main::display_pgc pgc $newvtsnum 1
	::main::dvd_modified true false

	# copy or move the TTN VOBs, if user wants so.
	set destvobbasename [file join $::dvddir "VTS_[format %02i $newvtsnum]_"]
	set sourcevobbasename [string trimright [file rootname $sourceifofile] "0"]
	if {$destvobbasename != $sourcevobbasename} {
		set numvobs 0
		set vobsize 0
		for {set i 1} {$i < 10} {incr i} {
			if {! [file exists "${sourcevobbasename}$i.VOB"]} break
			set numvobs $i
			set vobsize [expr {$vobsize+([file size "${sourcevobbasename}$i.VOB"]/1024.0)}]
		}
		set vobkbsize [expr {round($vobsize*100)/100.0}]
		set vobmbsize [expr {round(($vobkbsize/1024.0)*100)/100.0}]
		set vobgbsize [expr {round(($vobmbsize/1024.0)*100)/100.0}]
		if {$vobmbsize > 1} {
			if {$vobgbsize > 1} {
				set sizemsg "($vobgbsize GB.)"
			} else {
				set sizemsg "($vobmbsize MB.)"
			}
		} else {
			set sizemsg "($vobkbsize KB.)"
		}
		if {$numvobs > 0} {
			if {$numvobs > 1} {
				set pluriel "s"
				set vobfilesmsg "$numvobs TTN VOB files"
			} else {
				set pluriel ""
				set vobfilesmsg "TTN VOB file"
			}
			set txt "New VTS $newvtsnum ($num_new_ttns title"
			if {$num_new_ttns > 1} {set txt "${txt}s"}
			set txt "$txt) imported OK.\n"
			set txt "$txt\nImport the $vobfilesmsg $sizemsg ?"
			if {[file exists ${destvobbasename}1.VOB] || [file exists ${destvobbasename}0.IFO]} {
				set txt "$txt\n\nWARNING: There are already some VTS $newvtsnum"
				set txt "$txt\nIFO/VOB files in your DVD folder."
				set txt "$txt\nIF YOU CONTINUE, THEY WILL BE DELETED FIRST!"
			}
			set r [tk_dialog .confirm "PgcEdit" $txt "question" 0 \
					"  Copy VOB$pluriel  " "  Move VOB$pluriel  " "Don't import VOB$pluriel"]
			if {$r != 2} {
				for {set i 1} {$i < 10} {incr i} {
					if {[file exists "${destvobbasename}$i.VOB"]} {
						deletefile "${destvobbasename}$i.VOB" "PgcEdit: Import Title" .
					}
				}
			}
			update idletasks
			switch $r {
				0 {	# copy VOBs
					for {set i 1} {$i <= $numvobs} {incr i} {
						update idletasks
						show_msg "Copying VOB $i/$numvobs.  Please wait." $i $numvobs
						after 100 {set ::pause false}
						tkwait variable ::pause
						copyfile "${sourcevobbasename}$i.VOB" "${destvobbasename}$i.VOB" -force "PgcEdit: Import Titles" .
					}
				}
				1 {	# move VOBs
					show_msg "Moving VOBs."
					for {set i 1} {$i <= $numvobs} {incr i} {
						movefile ${sourcevobbasename}$i.VOB ${destvobbasename}$i.VOB false $utiltitle .
					}
				}
			}
		}
	}
	if {$::raw_sectors(0,VMG_PTL_MAIT) != ""} {
		set txt "Warning:\nThe original DVD contains Parental Managment.\n"
		set txt "$txt\nIt is probably better to remove the parental managment now, but"
		set txt "$txt\nchildrens will be able to watch the DVD without restrictions!"
		set txt "$txt\nIf you don't remove it, the new title will inherit the parental"
		set txt "$txt\nmanagment level of the VMGM (Title menu) of the current DVD."
		set txt "$txt\n\nOK to remove the parental managment (recommended)?"
		set r [tk_messageBox -title $utiltitle -icon question -parent . -type yesno \
				-default yes -message $txt]
		if {$r == "yes"} {
			::macros::remove_parental_managment
		} else {
			# adds new entries in the VMG_PTL_MAIT table
			fix_vts_parental_management
		}
	}
	show_msg ""
	return [expr {$old_num_titles+1}]
}

#############################################################################
## Import VTS Titles and replace current domain
proc replace_VTST {} {
	if {! [info exists ::dvddir]} {
		bell
		return 0
	}
	set utiltitle "Replace VTST"
	set origvtsnum [lindex [split [lindex $::editor_cmds_from 1] ,] 0]

	if {[lindex $::editor_cmds_from 0] != "pgc" || $origvtsnum == 0} {
		show_warning -title $utiltitle -icon error -parent . \
				-message "You must select the title domain to replace!"
		bell
		return 0
	}

	show_msg "$utiltitle"
	if {!$::config(expert_mode)} {
		set txt "This function will replace the current VTST domain\n"
		set txt "${txt}by an imported one.\n"
		set txt "${txt}\nYou have to select the VTST domain to replace\n"
		set txt "${txt}before calling this function.\n"
		set txt "${txt}\nYou may not import a VTST domain with less titles\n"
		set txt "${txt}than in the original domain.\n"
		set txt "${txt}\nIf your imported VTST domain has more titles than\n"
		set txt "${txt}in the original VTST, the new titles will be created,\n"
		set txt "${txt}and the global table of titles expanded accordingly.\n"
		set txt "${txt}However, in this case, you will need to modify the\n"
		set txt "${txt}navigation commands to access them, or to remove\n"
		set txt "${txt}them with the Remove Last PGC in Domain function\n"
		set txt "${txt}(followed by a processing by VobBlanker to remove\n"
		set txt "${txt}the unused cells).\n"
		set txt "${txt}\nYou may freely import a domain with a different\n"
		set txt "${txt}number of chapters or programs per title, but you\n"
		set txt "${txt}must understand what you are doing.\n"
		set txt "${txt}\nThe menu of the current domain is not affected by\n"
		set txt "${txt}this function.\n"
		set txt "${txt}\nSince the VOB files will be modified, don't forget to\n"
		set txt "${txt}save the DVD after using this function!\n"
		set txt "${txt}\nContinue?"
		set r [tk_messageBox -title $utiltitle -icon question -parent . -type okcancel \
				-default ok -message $txt]
		if {$r == "cancel"} {
			show_msg ""
			return 0
		}
	}

	set sourceifofile [tk_getOpenFile -initialdir $::config(importmenuinitialdir) \
		-filetypes {{"VTS IFO files" {*_0.IFO}}} \
		-title "Select the VTS IFO file with the title(s) to import"]
	if {$sourceifofile == ""} {return 0}
	if {[file tail $sourceifofile] == "VIDEO_TS.IFO"} {
		show_warning -title $utiltitle -icon error -parent . \
				-message "You must select a VTS IFO (VTS_**_*.IFO)"
		return 0
	}
	set ::config(importmenuinitialdir) [file dirname $sourceifofile]
	set sourceifofile [file normalize [file nativename $sourceifofile]]
	if {[file dirname $sourceifofile] == [file normalize [file nativename $::dvddir]]} {
		set sourcevtsnum [string trimleft [string range [file tail $sourceifofile] 4 5] 0]
		if {$sourcevtsnum == $origvtsnum} {
			show_warning -title $utiltitle -icon error -parent . \
					-message "Cannot import VTS $sourcevtsnum over itself!\nSelect another source DVD or another titleset."
			return 0
		}
	}

	# read the VTS
	set sourcevtsnum [::readdvd::readdvdfile $sourceifofile ::imported_sectors ::imported_pgcs ::imported_menus ::imported_ttns]
	if {$sourcevtsnum == -1} {return 0}
	if {$sourcevtsnum == 0} {
		show_warning -title $utiltitle -icon error -parent . \
				-message "You must select a VTS IFO (VTS_**_*.IFO)"
		# cleanup
		unset ::imported_sectors ::imported_pgcs ::imported_menus
		return 0
	}

	# How many TTNs are in the imported VTS?
	set num_new_ttns [get_int ::imported_sectors($sourcevtsnum,VTS_PTT_SRPT) 0 2]
	if {$num_new_ttns == 0} {
		show_warning -title $utiltitle -icon error -parent . \
				-message "Error: there are no titles defined in VTS file\n\"$sourceifofile\""
		# cleanup
		unset ::imported_sectors ::imported_pgcs ::imported_menus
		return 0
	}

	# Panic if the imported domain has less TTNs than the current one.
	set num_orig_ttns [get_int ::raw_sectors($origvtsnum,VTS_PTT_SRPT) 0 2]
	if {$num_new_ttns != $num_orig_ttns} {
		set txt "There are $num_orig_ttns TTNs in the original title domain"
		set txt "$txt\nand $num_new_ttns TTNs in the imported title domain."
		if {$num_new_ttns > $num_orig_ttns} {
			set txt "$txt\n\nIf you continue, [expr $num_new_ttns - $num_orig_ttns] new Titles will be created."
			set txt "$txt\n\nContinue anyway?"
			set r [tk_messageBox -title $utiltitle -icon warning -parent . -type yesno \
					-default no -message "WARNING:\n$txt"]
			if {$r == "no"} {
				# cleanup
				unset ::imported_sectors ::imported_pgcs ::imported_menus
				return 0
			}
		} else {
			set txt "$txt\n\nThe imported domain must have $num_orig_ttns TTNs, or more."
			set txt "$txt\nSorry."
			tk_messageBox -title $utiltitle -icon error -parent . -message "ERROR:\n$txt"
			unset ::imported_sectors ::imported_pgcs ::imported_menus
			return 0
		}
	}

	# read the source VIDEO_TS.IFO to get the VMG_TT_SRPT table.
	set vmgfile [file join [file dirname $sourceifofile] "VIDEO_TS.IFO"]
	set src_vmg_tt_srpt ""
	if {[file exists $vmgfile]} {
		if {[::readdvd::readdvdfile $vmgfile ::ivmg_sectors ::ivmg_pgcs ::ivmg_menus ::ivmg_ttns] == 0} {
			set src_vmg_tt_srpt $::ivmg_sectors(0,VMG_TT_SRPT)
			unset ::ivmg_sectors ::ivmg_pgcs ::ivmg_menus
		}
	}
	if {$src_vmg_tt_srpt == "" && $num_new_ttns > $num_orig_ttns} {
		set txt "WARNING:\nCan't find or load the source VMG (VIDEO_TS.IFO)!\n"
		set txt "$txt\nThe new titles entries in the VMG Table of Titles"
		set txt "$txt\n will be initialized with default values."
		set r [tk_messageBox -title $utiltitle -icon warning -parent . -type okcancel \
				-default cancel -message $txt]
		if {$r == "cancel"} {
			# cleanup
			unset ::imported_sectors ::imported_pgcs ::imported_menus
			show_msg ""
			return 0
		}
	}

	set orig_numpgcs $::pgcs($origvtsnum,numpgcs)

	# ask if the user wants to keep the PGC commands, links and CLUT
	set txt "Do you want to keep, from the original PGCs,\n"
	set txt "${txt}  - the VM commands (pre, post and cell commands),\n"
	set txt "${txt}  - the Playback mode (sequential, random or shuffle),\n"
	set txt "${txt}  - the PUOs (Prohibited User Operations),\n"
	set txt "${txt}  - the PrevPGC, NextPGC and GoUpPGC links,\n"
	set txt "${txt}  - the Streams assignments (audio and subpic),\n"
	set txt "${txt}  - the CLUT (Color Lookup Table),\n"
	set txt "${txt}and, when the number of cells of the new PGC\n"
	set txt "${txt}matches the number of cells of the original PGC,\n"
	set txt "${txt}  - the Program Table (Program numbers),\n"
	set txt "${txt}  - the Chapter Table (Chapter numbers),\n"
	set txt "${txt}  - the cell's VOBU Still flags,\n"
	set txt "${txt}  - the Cell Still Times,\n"
	set txt "${txt}  - the Cell Command numbers ?\n\n"
	set txt "${txt}(Use this option to replace a title by a reencoded one.)"
	set r [tk_messageBox -title $utiltitle -icon question -parent . -type yesnocancel \
			-default no -message $txt]
	if {$r == "cancel"} {
		# cleanup
		unset ::imported_sectors ::imported_pgcs ::imported_menus
		show_msg ""
		return 0
	}
	if {$r == "yes"} {
		set keep_original_datas true
		for {set pgc 1} {$pgc <= $::pgcs($origvtsnum,numpgcs)} {incr pgc} {
			set orig_cmds($pgc) $::pgcs($origvtsnum,$pgc)
			set orig_header($pgc) $::pgcs($origvtsnum,$pgc,header)
			set orig_programmap($pgc) $::pgcs($origvtsnum,$pgc,programmap)
			set orig_cellpbtable($pgc) $::pgcs($origvtsnum,$pgc,cellpbtable)
			set orig_pttlist($pgc) $::pgcs($origvtsnum,$pgc,pttlist)
		}
	} else {
		set keep_original_datas false
	}

	# backup VOBs
	set destvobtailname "VTS_[format %02i $origvtsnum]_"
	if {[file exists [file join $::dvddir ${destvobtailname}1.VOB]]} {
		set txt "Move original VOBs to backup?"
		set default "yes"
		if {[file exists [file join $::backupdir ${destvobtailname}1.VOB]]} {
			set txt "${txt}\n\nWARNING: ${destvobtailname}1.VOB\nalready exists in backup folder!"
			set default "no"
		}
		set r [tk_messageBox -title $utiltitle -icon question -parent . -type yesnocancel \
				-default $default -message $txt]
		if {$r == "cancel"} {
			unset -nocomplain ::imported_sectors ::imported_pgcs ::imported_menus
			show_msg ""
			return 0
		}
		if {$r == "yes"} {
			for {set i 1} {$i < 10} {incr i} {
				if {[file exists [file join $::backupdir "${destvobtailname}$i.VOB"]]} {
					deletefile [file join $::backupdir "${destvobtailname}$i.VOB"] $utiltitle .
				}
				if {[file exists [file join $::dvddir "${destvobtailname}$i.VOB"]]} {
					movefile [file join $::dvddir "${destvobtailname}$i.VOB"] $::backupdir -force $utiltitle .
				}
			}
		} else {
			for {set i 1} {$i < 10} {incr i} {
				if {[file exists [file join $::dvddir "${destvobtailname}$i.VOB"]]} {
					deletefile [file join $::dvddir "${destvobtailname}$i.VOB"] $utiltitle .
				}
			}
		}
	}

	# move or copy the TTN VOBs
	set destvobbasename [file join $::dvddir "VTS_[format %02i $origvtsnum]_"]
	set sourcevobbasename [string trimright [file rootname $sourceifofile] "0"]
	if {$destvobbasename != $sourcevobbasename} {
		set numvobs 0
		set vobsize 0
		for {set i 1} {$i < 10} {incr i} {
			if {! [file exists "${sourcevobbasename}$i.VOB"]} break
			set numvobs $i
			set vobsize [expr {$vobsize+([file size "${sourcevobbasename}$i.VOB"]/1024.0)}]
		}
		set vobkbsize [expr {round($vobsize*100)/100.0}]
		set vobmbsize [expr {round(($vobkbsize/1024.0)*100)/100.0}]
		set vobgbsize [expr {round(($vobmbsize/1024.0)*100)/100.0}]
		if {$vobmbsize > 1} {
			if {$vobgbsize > 1} {
				set sizemsg "($vobgbsize GB.)"
			} else {
				set sizemsg "($vobmbsize MB.)"
			}
		} else {
			set sizemsg "($vobkbsize KB.)"
		}
		if {$numvobs > 0} {
			if {$numvobs > 1} {
				set pluriel "s"
				set vobfilesmsg "$numvobs TTN VOB files"
			} else {
				set pluriel ""
				set vobfilesmsg "TTN VOB file"
			}
			set txt "New VTS $origvtsnum ($num_new_ttns title"
			if {$num_new_ttns > 1} {set txt "${txt}s"}
			set txt "$txt) imported OK.\n"
			set txt "$txt\nImport the $vobfilesmsg $sizemsg ?"
			set r [tk_dialog .confirm $utiltitle $txt "question" 0 \
					"  Copy VOB$pluriel  " "  Move VOB$pluriel  " "Don't import VOB$pluriel"]
			update idletasks
			switch $r {
				0 {	# copy VOBs
					for {set i 1} {$i <= $numvobs} {incr i} {
						update idletasks
						show_msg "Copying VOB $i/$numvobs.  Please wait." $i $numvobs
						after 100 {set ::pause false}
						tkwait variable ::pause
						copyfile "${sourcevobbasename}$i.VOB" "${destvobbasename}$i.VOB" -force "PgcEdit: Import Titles" .
					}
				}
				1 {	# move VOBs
					show_msg "Moving VOBs."
					for {set i 1} {$i <= $numvobs} {incr i} {
						movefile ${sourcevobbasename}$i.VOB ${destvobbasename}$i.VOB false $utiltitle .
					}
				}
			}
		}
	}


	### Go modify the DVD!
	# init the new VTS sectors
	::readdvd::parse_VTS_PTT_SRPT ::imported_sectors($sourcevtsnum,VTS_PTT_SRPT) $origvtsnum ::imported_pgcs ::imported_ttns
	set ::raw_sectors($origvtsnum,VTSI_MAT) \
			[string range $::raw_sectors($origvtsnum,VTSI_MAT) 0 511][string range $::imported_sectors($sourcevtsnum,VTSI_MAT) 512 end]
	set ::raw_sectors($origvtsnum,VTS_PTT_SRPT) \
			$::imported_sectors($sourcevtsnum,VTS_PTT_SRPT)
	set ::raw_sectors($origvtsnum,VTS_PGCI) \
			$::imported_sectors($sourcevtsnum,VTS_PGCI)
	set ::raw_sectors($origvtsnum,VTS_TMAPTI) \
			$::imported_sectors($sourcevtsnum,VTS_TMAPTI)
	set ::raw_sectors($origvtsnum,VTS_C_ADT) \
			$::imported_sectors($sourcevtsnum,VTS_C_ADT)
	set ::raw_sectors($origvtsnum,VTS_VOBU_ADMAP) \
			$::imported_sectors($sourcevtsnum,VTS_VOBU_ADMAP)

	# cleanup
	unset ::imported_sectors ::imported_pgcs ::imported_menus

	# Init variables and parse the new TTNs
	::readdvd::parse_VTS_PGCI ::raw_sectors($origvtsnum,VTS_PGCI) $origvtsnum ::pgcs
	::readdvd::parse_VTS_PTT_SRPT ::raw_sectors($origvtsnum,VTS_PTT_SRPT) $origvtsnum ::pgcs ::ttns

	# Add new Titles to the TT_SRPT Title Table.
	# If the original table was read, copy the relevent entries,
	# otherwise, set default values (except for PTTs value)
	if {$num_new_ttns > $num_orig_ttns} {
		set total_old_num_titles [get_int ::raw_sectors(0,VMG_TT_SRPT) 0 2]
		set data [string range $::raw_sectors(0,VMG_TT_SRPT) 0 [expr 8+($total_old_num_titles*12)-1]]

		set vts_old_num_titles 0
		for {set t 1} {$t <= $total_old_num_titles} {incr t} {
			set startidx [expr {($t * 12) - 4}]
			set endidx   [expr {$startidx + 11}]
			if {$endidx > [string length $::raw_sectors(0,VMG_TT_SRPT)]} break
			set entry [string range $::raw_sectors(0,VMG_TT_SRPT) $startidx $endidx]
			set vtsn [get_int entry 6 1]
			if {$vtsn == $origvtsnum} {
				incr vts_old_num_titles
			}
		}

		if {$src_vmg_tt_srpt != ""} {
			set src_num_titles [get_int src_vmg_tt_srpt 0 2]
			set num_new_entries 0
			set current_ttn 0
			for {set t 1} {$t <= $src_num_titles} {incr t} {
				set startidx [expr {($t * 12) - 4}]
				set endidx   [expr {$startidx + 11}]
				if {$endidx > [string length $src_vmg_tt_srpt]} break
				set entry [string range $src_vmg_tt_srpt $startidx $endidx]
				set vtsn [get_int entry 6 1]
				if {$vtsn == $sourcevtsnum} {
					incr current_ttn
					set_int entry 6 $origvtsnum 1
					if {$current_ttn > $vts_old_num_titles} {
						set data "$data$entry"
						incr num_new_entries
					} else {
						set this_ttn 0
						for {set ot 1} {$ot <= $total_old_num_titles} {incr ot} {
							set startidx [expr {($ot * 12) - 4}]
							set endidx   [expr {$startidx + 11}]
							if {$endidx > [string length $::raw_sectors(0,VMG_TT_SRPT)]} break
							set tmpentry [string range $::raw_sectors(0,VMG_TT_SRPT) $startidx $endidx]
							set vtsn [get_int tmpentry 6 1]
							if {$vtsn == $origvtsnum} {
								incr this_ttn
								if {$this_ttn == $current_ttn} {
									incr startidx -1
									incr endidx
									set part1 [string range $::raw_sectors(0,VMG_TT_SRPT) 0 $startidx]
									if {$endidx < [string length $::raw_sectors(0,VMG_TT_SRPT)]} {
										set part3 [string range $::raw_sectors(0,VMG_TT_SRPT) $endidx end]
									} else {
										set part3 ""
									}
									set data $part1$entry$part3
								}
							}
						}
					}
				}
			}
			set new_num_titles [expr {$total_old_num_titles + $num_new_entries}]
			set_int data 0 $new_num_titles 2
			set_int data 4 [expr {[string length $data] -1}] 4
			set ::raw_sectors(0,VMG_TT_SRPT) $data
		} else {
			set vtssrpt $::raw_sectors($origvtsnum,VTS_PTT_SRPT)
			set src_num_titles $num_new_ttns
			set num_new_entries 0
			set current_ttn 0
			for {set t 1} {$t <= $num_new_ttns} {incr t} {
				set entry "\074\001\000\000\000\000\000\000\000\000\000\000"
				incr current_ttn
				set_int entry 6 $origvtsnum 1
				set num_ptts 1
				if {$vtssrpt != ""} {
					set numttus [get_int vtssrpt 0 2]
					set idx [get_int vtssrpt [expr {4 + ($t * 4)}] 4]
					set pgcn [get_int vtssrpt $idx 2]
					set num_ptts 0
					while {$pgcn == $t} {
						incr num_ptts
						incr idx 4
						if {$idx >= [string length $vtssrpt]} break
						set pgcn [get_int vtssrpt $idx 2]
					}
				}
				set_int entry 2 $num_ptts 2
				set_int entry 7 $current_ttn 1
				if {$current_ttn > $vts_old_num_titles} {
					set data "$data$entry"
					incr num_new_entries
				} else {
					set this_ttn 0
					for {set ot 1} {$ot <= $total_old_num_titles} {incr ot} {
						set startidx [expr {($ot * 12) - 4}]
						set endidx   [expr {$startidx + 11}]
						if {$endidx > [string length $::raw_sectors(0,VMG_TT_SRPT)]} break
						set tmpentry [string range $::raw_sectors(0,VMG_TT_SRPT) $startidx $endidx]
						set vtsn [get_int tmpentry 6 1]
						if {$vtsn == $origvtsnum} {
							incr this_ttn
							if {$this_ttn == $t} {
								incr startidx -1
								incr endidx
								set part1 [string range $::raw_sectors(0,VMG_TT_SRPT) 0 $startidx]
								if {$endidx < [string length $::raw_sectors(0,VMG_TT_SRPT)]} {
									set part3 [string range $::raw_sectors(0,VMG_TT_SRPT) $endidx end]
								} else {
									set part3 ""
								}
								set data $part1$entry$part3
							}
						}
					}
				}
			}
			set new_num_titles [expr {$total_old_num_titles + $num_new_entries}]
			set_int data 0 $new_num_titles 2
			set_int data 4 [expr {[string length $data] -1}] 4
			set ::raw_sectors(0,VMG_TT_SRPT) $data
		}
	
		# get the table of titles from the VMG_TT_SRPT again
		# (to be able to display the title number in the PGC selectors)
		set ::pgcs(Title,NumTitles) $new_num_titles
		for {set t 1} {$t <= $new_num_titles} {incr t} {
			set t_idx [expr {14+(($t-1)*12)}]
			set ::pgcs([get_int data $t_idx 1],$::ttns([get_int data $t_idx 1],[get_int data [expr {$t_idx+1}] 1]),Title) $t
			set ::pgcs(Title,$t) [list [get_int data $t_idx 1] $::ttns([get_int data $t_idx 1],[get_int data [expr {$t_idx+1}] 1])]
		}
	}

	# Erase BOVs informations
	set ::buttons::butsinfo(pgc,$origvtsnum,scanned) 0
	set ::buttons::butsinfo(pgc,$origvtsnum,errorcode) -1
	set ::buttons::butsinfo(pgc,$origvtsnum,errormsg) "Domain not yet scanned for BOVs"
	set ::buttons::butsinfo(pgc,$origvtsnum,numvobids) 0
	for {set pgc 1} {$pgc <= $orig_numpgcs} {incr pgc} {
		unset -nocomplain ::pgcs($origvtsnum,$pgc,numbuttons)
	}

	# restore original datas if the user has requested so
	if {$keep_original_datas} {
		for {set pgc 1} {$pgc <= $orig_numpgcs} {incr pgc} {
			# process PGC header
			# restore commands
			set ::pgcs($origvtsnum,$pgc) $orig_cmds($pgc)
			# restore some parts of the header
			set newheader $::pgcs($origvtsnum,$pgc,header)
			set oldheader $orig_header($pgc)
			set_int newheader 0x08 [get_int oldheader 0x08 4] 4	;# PUOs
			# Audio streams assignments
			for {set i 0} {$i < 8} {incr i} {
				set offset [expr 0x0C + ($i * 2)]
				set_int newheader $offset [get_int oldheader $offset 2] 2
			}
			# Subpic streams assignments
			for {set i 0} {$i < 32} {incr i} {
				set offset [expr 0x1C + ($i * 4)]
				set_int newheader $offset [get_int oldheader $offset 4] 4
			}
			set_int newheader 0x9C [get_int oldheader 0x9C 2] 2	;# NextPGCN
			set_int newheader 0x9E [get_int oldheader 0x9E 2] 2	;# PrevPGCN
			set_int newheader 0xA0 [get_int oldheader 0xA0 2] 2	;# GoUpPGCN
			set_int newheader 0xA2 [get_int oldheader 0xA2 1] 1	;# PG Playback Mode
			set_int newheader 0xA3 [get_int oldheader 0xA3 1] 1	;# PGC Still Time
			# CLUT
			for {set i 0} {$i < 16} {incr i} {
				set offset [expr 0xA4 + ($i * 4)]
				set_int newheader $offset [get_int oldheader $offset 4] 4
			}
			set ::pgcs($origvtsnum,$pgc,header) $newheader

			# if the number of cells is identical, process also the program map,
			# PTT list and cell playback table 
			set numcells	[get_int oldheader 0x03 1]
			set newnumcells	[get_int newheader 0x03 1]
			if {$numcells == $newnumcells} {
				set ::pgcs($origvtsnum,$pgc,programmap) $orig_programmap($pgc)
				set ::pgcs($origvtsnum,$pgc,pttlist) $orig_pttlist($pgc)
				set newcellpbtable $::pgcs($origvtsnum,$pgc,cellpbtable)
				set oldcellpbtable $orig_cellpbtable($pgc)
				for {set c 0} {$c < $numcells} {incr c} {
					set offset [expr $c * 0x18]
					incr offset
					set_int newcellpbtable $offset [get_int oldcellpbtable $offset 1] 1	;# VOBU Still
					incr offset
					set_int newcellpbtable $offset [get_int oldcellpbtable $offset 1] 1	;# Cell still time
					incr offset
					set_int newcellpbtable $offset [get_int oldcellpbtable $offset 1] 1	;# Cell command #
				}
				set ::pgcs($origvtsnum,$pgc,cellpbtable) $newcellpbtable
			} else {
				set txt "Warning:\nThere are $numcells cells in the original PGC $pgc.\n"
				set txt "${txt}There are $newnumcells cells in the imported PGC $pgc.\n\n"
				set txt "${txt}The Programs, cell's VOBU Still flag, Cell Still Time,\n"
				set txt "${txt}Cell Command numbers and the Chapter numbers\n"
				set txt "${txt}will NOT be kept from the original PGC!"
				tk_messageBox -title $utiltitle -icon warning -parent . -message $txt
			}
		}
	}

	# Reinit GUI
	::main::init_tree_menu
	::main::display_pgc pgc $origvtsnum 1
	::main::dvd_modified true false

	show_msg "$utiltitle: Done.  Please save the DVD now!"
	return $num_new_ttns
}

#############################################################################
## Remap GUI
## Open a window with a listbox where the user can rearrange the order of the items.
## Args: Window name, window title, message to display on top ot the window (or empty string),
## list to be displayed in the listbox.
## The first element in the strings of the list MUST be the number of the element, starting
## at 1.  It is retrieved using [lindex [lindex $contents $n] 0], and it MUST be a positive
## integer, sorted!  Example: {{1 : orig=first} {2 : orig=second} {3 : orig=third}}
proc remap_gui {w title msg contents} {
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog
	wm withdraw $w
	wm protocol $w WM_DELETE_WINDOW {set ::utils::remap_done 0}
	bind $w <Escape> "set ::utils::remap_done 0"
	bind $w <Return> "set ::utils::remap_done 1"
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm title $w $title

	if {$msg != ""} {
		label  $w.lab -justify left -text $msg
		pack   $w.lab -anchor w -padx 2
	}
	frame  $w.mf -borderwidth 2 -relief groove
	pack   $w.mf -padx 4 -expand 1 -fill both
	listbox $w.mf.lb -yscrollcommand "$w.mf.vs set" -height [llength $contents] -width 52 \
			-listvariable ::utils::remap_list -font $::config(fixed_width_font) \
			-selectmode extended
	pack   $w.mf.lb -side left -expand 1 -fill both
	scrollbar $w.mf.vs -command "$w.mf.lb yview" -orient vert
	pack   $w.mf.vs -side left -fill y


	# buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -fill x
	frame  $w.fb.f1
	pack   $w.fb.f1 -side left -padx 2
	button $w.fb.f1.up -text "Move up"    -width 9 -command "::utils::move_remap_item $w up"
	pack   $w.fb.f1.up -pady 1
	button $w.fb.f1.dn -text "Move down"  -width 9 -command "::utils::move_remap_item $w down"
	pack   $w.fb.f1.dn -pady 1
	frame  $w.fb.f2
	pack   $w.fb.f2 -side left -padx 2
	button $w.fb.f2.top -text "To top"    -width 9 -command "::utils::move_remap_item $w top"
	pack   $w.fb.f2.top -pady 1
	button $w.fb.f2.bot -text "To bottom" -width 9 -command "::utils::move_remap_item $w bottom"
	pack   $w.fb.f2.bot -pady 1
	button $w.fb.inv -text "Invert selection" -command {
			set lb .vtsremap.mf.lb
			set s [$lb curselection]
			$lb selection set 0 end
			foreach i $s {
				$lb selection clear $i
			}
		}
	pack   $w.fb.inv -side left -pady 1 -padx 4 -anchor n
	frame  $w.fb.pad
	pack   $w.fb.pad -expand 1 -fill x -side left
	frame  $w.fb.b
	pack   $w.fb.b -side left -anchor s
	button $w.fb.b.ok -text "OK" -width 10 -command {set ::utils::remap_done 1}
	pack   $w.fb.b.ok -side left -padx 3
	button $w.fb.b.cancel -text "Cancel" -width 10 -command {set ::utils::remap_done 0}
	pack   $w.fb.b.cancel -side left -padx 2
	if {$::config(platform) == "mac"} {
		pack [frame $w.fb.macpad] -side left -padx 6
	}

	# window geometry
	if {[llength $contents] < 3} {
		$w.mf.lb configure -height 3
	}
	if {[llength $contents] > 20} {
		$w.mf.lb configure -height 20
	}
	update
	set ww [winfo reqwidth $w]
	set wh [winfo reqheight $w]
	set wx [expr {[winfo rootx .]+(([winfo width  .]-$ww)/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-$wh)/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	$w.mf.lb configure -height 3
	update
	wm minsize $w 100 [winfo reqheight $w]
	wm geometry $w ${ww}x$wh+$wx+$wy
	wm resizable $w 0 1

	set ::utils::remap_list $contents
	$w.mf.lb activate 0
	$w.mf.lb selection set 0
	
	wm deiconify $w
	grab set $w

	set ::utils::remap_done -1
	tkwait variable ::utils::remap_done
	destroy $w

	if {$::utils::remap_done} {
		set rc $::utils::remap_list
		unset ::utils::remap_list
		return $rc
	} else {
		unset ::utils::remap_list
		return ""
	}
}
proc move_remap_item {w direction} {
	set lb $w.mf.lb
	set currents [$lb curselection]
	if {[llength $currents] == 0 || [llength $currents] == [llength $::utils::remap_list]} {
		bell
		return
	}
	switch $direction {
		down {
			set target [expr [lindex $currents end] - [llength $currents] +2]
		}
		up {
			set target [expr [lindex $currents 0] - 1]
		}
		bottom {
			set target [expr [llength $::utils::remap_list] - [llength $currents]]
		}
		top {
			set target 0
		}
	}

	set moved {}
	set orig {}
	for {set i 0} {$i < [llength $::utils::remap_list]} {incr i} {
		if {[lsearch $currents $i] > -1} {
			lappend moved [lindex $::utils::remap_list $i]
		} else {
			lappend orig  [lindex $::utils::remap_list $i]
		}
	}
	if {$target < 0} {set target 0}
	if {$target > [llength $orig]} {set target [llength $orig]}

	if {$target == 0} {
		set ::utils::remap_list [concat $moved $orig]
	} elseif {$target >= [llength $orig]} {
		set ::utils::remap_list [concat $orig $moved]
	} else {
		set ::utils::remap_list [concat [lrange $orig 0 [expr $target-1]] $moved [lrange $orig $target end]]
	}
	for {set i 0} {$i < [llength $::utils::remap_list]} {incr i} {
		lset ::utils::remap_list $i \
				[format "%3d%s" [expr $i+1] [string range [lindex $::utils::remap_list $i] 3 end]]
	}
	$lb activate $target
	$lb selection clear 0 end
	$lb selection set $target [expr $target+[llength $moved]-1]
	$lb see [expr $target+[llength $moved]-1]
	$lb see $target
	switch $direction {
		down {
			if {[expr $target+[llength $moved]-1] < [llength $::utils::remap_list]} {
				$lb see [expr $target+[llength $moved]]
			}
		}
		up {
			if {$target > 0} {
				$lb see [expr $target -1]
			}
		}
	}
}

#############################################################################
## Remap Titlesets
proc remap_titlesets {} {
	set w ".vtsremap"

	set numtitlesets $::pgcs(numvts)
	if {$numtitlesets < 2} {
		bell
		return
	}

	if {$::dvd_modified} {
		set r [tk_messageBox -title "Remap Titlesets plugin" -icon warning -type okcancel -default ok \
				-message "WARNING!\nThis function requires to save the DVD.\nThe DVD has been modified.\nIf you continue, it will be saved\nautomatically during the operation!"]
		if {$r == "cancel"} {
			return
		}
	}

	# Build list to display in dialog
	for {set vts 1} {$vts <= $numtitlesets} {incr vts} {
		set titles_in_titleset($vts) {}
	}
	for {set t 1} {$t <= [get_int ::raw_sectors(0,VMG_TT_SRPT) 0 2]} {incr t} {
		set vts [get_int ::raw_sectors(0,VMG_TT_SRPT) [expr {($t*12)+2}] 1]
		set titles_in_titleset($vts) "$titles_in_titleset($vts), $t"
	}
	set remap_titlesets {}
	for {set vts 1} {$vts <= $numtitlesets} {incr vts} {
		set t_idx [expr {8+(($vts-1)*12)}]
		lappend remap_titlesets [format "%3d : old VTS %3d  (with Title[string trim $titles_in_titleset($vts) ,])" $vts $vts]
	}

	set remap_titlesets [remap_gui $w "Remap Titlesets" \
			"This function moves the Titlesets safely!\nThe targets of the JumpSS_VTSM commands will be renumbered automatically in the VMG,\nincluding, if needed, in the menu buttons, when you hit OK." \
			$remap_titlesets] 

	if {$remap_titlesets != ""} {
		# remember user's PGC comments
		foreach i [array names ::pgcs_comments] {
			set old_pgcs_comments($i) $::pgcs_comments($i)
		}
		foreach i [array names ::menus_comments] {
			set old_menus_comments($i) $::menus_comments($i)
		}

		# Save DVD
		if {$::dvd_modified} {
			::main::Write_DVD false
		}

		# build simple remapping table
		set remapping(0) 0
		for {set i 0} {$i < $numtitlesets} {incr i} {
			set remapping([lindex [lindex $remap_titlesets $i] 4]) [expr $i+1]
		}

		# rename files on HDD
		set rc 0
		for {set vts 1} {$vts <= $numtitlesets} {incr vts} {
			set filenames [list "VTS_[format %02d $vts]_0.IFO" "VTS_[format %02d $vts]_0.BUP"]
			for {set i 0} {$i <= 9} {incr i} {
				lappend filenames "VTS_[format %02d $vts]_$i.VOB"
			}
			foreach i $filenames {
				if {[file exists [file join $::dvddir $i]]} {
					set rc [catch {file rename [file join $::dvddir $i] [file join $::dvddir _orig_$i]} errstr]
					if {$rc} {
						tk_messageBox -title "Remap Titlesets plugin" -icon info \
								-message "Error: Cannot rename file $i\n$errstr\n\nAborting."
						break
					}
				}
			}
			if {$rc} {
				break
			}
		}
		if {! $rc} {
			for {set vts 1} {$vts <= $numtitlesets} {incr vts} {
				foreach i {IFO BUP} {
					catch {file rename [file join $::dvddir "_orig_VTS_[format %02d $vts]_0.$i"] \
							[file join $::dvddir "VTS_[format %02d $remapping($vts)]_0.$i"]}
				}
				for {set i 0} {$i <= 9} {incr i} {
					catch {file rename	[file join $::dvddir "_orig_VTS_[format %02d $vts]_$i.VOB"] \
							[file join $::dvddir "VTS_[format %02d $remapping($vts)]_$i.VOB"]}
				}
			}
		} else {
			# If error when renaming files, restore original filenames and exit
			for {set vts 1} {$vts <= $numtitlesets} {incr vts} {
				set filenames [list "VTS_[format %02d $vts]_0.IFO" "VTS_[format %02d $vts]_0.BUP"]
				for {set i 0} {$i <= 9} {incr i} {
					lappend filenames "VTS_[format %02d $vts]_$i.VOB"
				}
				foreach i $filenames {
					if {[file exists [file join $::dvddir _orig_$i]]} {
						catch {file rename [file join $::dvddir _orig_$i] [file join $::dvddir $i]}
					}
				}
			}
			return
		}

		# Change Titleset numbers in VMG_TT_SRPT
		for {set t 1} {$t <= [get_int ::raw_sectors(0,VMG_TT_SRPT) 0 2]} {incr t} {
			set vts_idx [expr {($t*12)+2}]
			set_int ::raw_sectors(0,VMG_TT_SRPT) $vts_idx \
					$remapping([get_int ::raw_sectors(0,VMG_TT_SRPT) $vts_idx 1]) 1
		}

		# fix JumpSS_VTSM commands in VMG
		foreach i [lsearch -all -regexp $::pgcs(0,0) "30 .6 .. .. .. 8. .. .."] {
			set cmd [lindex $::pgcs(0,0) $i]
			set origtarget [x2d [string range $cmd 12 13]]
			if {$origtarget <= $numtitlesets} {
				set cmd [string range $cmd 0 11][format %02X $remapping($origtarget)][string range $cmd 14 end]
				lset ::pgcs(0,0) $i $cmd
			}
		}
		for {set lu 1} {$lu <= $::menus(0,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus(0,$lu,numpgcs)} {incr pgc} {
				foreach i [lsearch -all -regexp $::menus(0,$lu,$pgc) "30 .6 .. .. .. 8. .. .."] {
					set cmd [lindex $::menus(0,$lu,$pgc) $i]
					set origtarget [x2d [string range $cmd 12 13]]
					if {$origtarget <= $numtitlesets} {
						set cmd [string range $cmd 0 11][format %02X $remapping($origtarget)][string range $cmd 14 end]
						lset ::menus(0,$lu,$pgc) $i $cmd
					}
				}
				set numcells $::menus(0,$lu,$pgc,numcells)
				if {$numcells > 0} {
					set cellpostable $::menus(0,$lu,$pgc,cellpostable)
					for {set cell 1} {$cell <= $numcells} {incr cell} {
						set idx [expr {($cell-1) * 4}]
						set vobid [get_int cellpostable $idx 2]
						incr idx 3
						set cellid [get_int cellpostable $idx 1]
						if {[info exists ::buttons::butsinfo(menu,0,$vobid,$cellid,numbuts)]} {
							set numbuts $::buttons::butsinfo(menu,0,$vobid,$cellid,numbuts)
							if {$numbuts > 0} {
								set numgroups $::buttons::butsinfo(menu,0,$vobid,$cellid,numgroups)
								if {$numgroups > 0} {
									for {set butnum 1} {$butnum <= $numbuts} {incr butnum} {
										for {set group 1} {$group <= $numgroups} {incr group} {
											set cmd [string toupper $::buttons::butsinfo(menu,0,$vobid,$cellid,$group,$butnum,command)]
											if {[lsearch -regexp [list $cmd] "30 .6 .. .. .. 8. .. .."] > -1} {
												set origtarget [x2d [string range $cmd 12 13]]
												if {$origtarget <= $numtitlesets} {
													set cmd [string range $cmd 0 11][format %02X $remapping($origtarget)][string range $cmd 14 end]
													set ::buttons::butsinfo(menu,0,$vobid,$cellid,$group,$butnum,command) $cmd
													set ::buttons::butsinfo(menu,edited) 1
													set ::buttons::butsinfo(menu,0,edited) 1
													set ::buttons::butsinfo(menu,0,$vobid,edited) 1
													set ::buttons::butsinfo(menu,0,$vobid,$cellid,edited) 1													
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}

		# Save VMGM
		::writedvd::write_VMG_IFO $::dvddir

		# save modified buttons in VMGM as well
		::menued::write_buttons_menus $::dvddir 0		

		# reopen DVD
		::main::ReRead_DVD false true

		# Save VMGM again to rebuild the VMG_VTS_ATRT table
		::writedvd::write_VMG_IFO $::dvddir

		# restore user's PGC comments
		clear_all_pgc_comments
		set ::pgc_comments_modified false
		foreach i [array names old_pgcs_comments] {
			set ::pgc_comments_modified true
			set indexes [split $i ,]
			set vts [lindex $indexes 0]
			set pgc [lindex $indexes 1]
			catch {set ::pgcs_comments($remapping($vts),$pgc) $old_pgcs_comments($i)}
		}
		foreach i [array names old_menus_comments] {
			set ::pgc_comments_modified true
			set indexes [split $i ,]
			set vts [lindex $indexes 0]
			set lu  [lindex $indexes 1]
			set pgc [lindex $indexes 2]
			catch {set ::menus_comments($remapping($vts),$lu,$pgc) $old_menus_comments($i)}
		}
		if {$::pgc_comments_modified} {
			::main::init_tree_menu
		}

		# Do an incremental backup now!
		set r [tk_messageBox -title "Remap Titlesets plugin" -icon question -type yesno -default yes \
				-message "Titlesets remapped successfully.\n\nAs the titlesets are now in a different order,\nthe previous backup is no more suitable.\n\nDo you want to make an incremental backup now?\n(HIGHLY RECOMMENDED!)"]
		if {$r == "yes"} {
			::main::Incremental_Backup
		}		
	}
}

#############################################################################
## Blank out all VTST Titles: GUI
proc blank_out_PGCS_GUI {} {
	set utiltitle "Blank out all PGCs of current domain"
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	set type   [lindex $::editor_cmds_from 0]
	if {$type == "menu"} {
		set type "Menus"
		set vtsi_mat_idx 0x100
	} else {
		set type "Titles"
		set vtsi_mat_idx 0x200
	}
	set vtsnum [lindex [split [lindex $::editor_cmds_from 1] ","] 0]
	if {$vtsnum == 0} {
		set vtsi_table_name "VMGM_MAT"
	} else {
		set vtsi_table_name "VTSI_MAT"
	}
	if {$vtsnum == 0 && $type == "Titles"} {bell; return}
	if {[expr {[get_int ::raw_sectors($vtsnum,$vtsi_table_name) $vtsi_mat_idx 2] & 0x3000}] == 0} {
		set standard "ntsc"
	} else {
		set standard "pal"
	}

	# check if the menu has video.  If not, abort!
	if {[lindex $::editor_cmds_from 0] == "menu"} {
		set hasvideo false
		for {set lu 1} {$lu <= $::menus($vtsnum,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vtsnum,$lu,numpgcs)} {incr pgc} {
				if {$::menus($vtsnum,$lu,$pgc,numcells)>0} {
					set hasvideo true
					break
				}
			}
		}
		if {!$hasvideo} {
			set r [tk_messageBox -title $utiltitle -icon question -parent . -type yesno -default yes \
					-message "This menu has no video.\n\nDo you want to remove the VOB file\n(and all IFO's references to it)?"]
			if {$r == "yes"} {
				::preview::close_preview
				set ::utils::blankout_movevob true
				blank_out_PGCS_remove_menu $vtsnum
			}
			return
		}
	}

	set current_aspect [expr {[get_int ::raw_sectors($vtsnum,$vtsi_table_name) $vtsi_mat_idx 2] & 0x0C00}]
	set restxt "Current video standard:  [string toupper $standard]"
	if {$current_aspect == [expr {(0x0C00)}]} {
		set restxt "$restxt  16:9"
	} else {
		set restxt "$restxt  4:3"
	}

	if {![info exists ::utils::blankout_type]} {
		set ::utils::blankout_type "black"
		set ::utils::blankout_killpb 1
		set ::utils::blankout_clip "pause"
		set ::utils::blankout_clip_celltime 5
		set ::utils::blankout_usemenu false
		set ::utils::blankout_movevob true
	}

	set w .blankout
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w $utiltitle
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm protocol $w WM_DELETE_WINDOW {set ::utils::blankout_done 0}
	bind $w <Escape> "set ::utils::blankout_done 0"
	bind $w <Return> "set ::utils::blankout_done 1"

	if {$vtsnum == 0} {set t "VMGM"} else {set t "VTS $vtsnum"}
	label $w.lab -text "Blank out ALL $type of $t"
	pack  $w.lab -padx 6 -pady 4
	if {$type == "Menus"} {
		frame $w.labw -bd 2 -relief ridge
		pack  $w.labw -fill x -padx 4 -pady 1
		label $w.labw.l -text "WARNING: Blanking menus with buttons may lead to infinite loops!" \
				-background pink -height 2
		pack  $w.labw.l -fill both
	}

	# current video resolution
	frame $w.f0 -bd 2 -relief groove
	pack  $w.f0 -fill x -padx 4 -pady 4
	label $w.f0.l -text $restxt -anchor w -height 2
	pack  $w.f0.l -fill x -padx 8

	# main Blank frame
	frame $w.fm -bd 2 -relief groove
	pack  $w.fm -padx 4 -pady 4 -fill x
	label $w.fm.lab -text "Blank out method:" -anchor w
	pack  $w.fm.lab -fill x -anchor w -padx 8

	# Blank out and remove VOB
	if {$type == "Menus"} {
		frame $w.fm.fr -bd 2 -relief groove
		pack  $w.fm.fr -padx 4 -pady 4 -fill x
		radiobutton $w.fm.fr.b -text "Completely remove the menu VOB file." \
				-variable ::utils::blankout_type -value "rem" -anchor w \
				-command "::utils::blank_out_PGCS_validate $w"
		pack  $w.fm.fr.b -fill x -padx 4 -pady 1
		::balloon::set_balloon $w.fm.fr.b "Remove the VOB and all references in the IFOs.\nThis will convert all PGCs to dummies."
	} elseif {$::utils::blankout_type == "rem"} {
		set ::utils::blankout_type "black"
	}

	# Blank out with black frame
	frame $w.fm.f1 -bd 2 -relief groove
	pack  $w.fm.f1 -padx 4 -fill x
	radiobutton $w.fm.f1.b -text "Replace all $type by a black frame. (VOB size: 10KB)" \
			-variable ::utils::blankout_type -value "black" -anchor w \
			-command "::utils::blank_out_PGCS_validate $w"
	pack  $w.fm.f1.b -fill x -padx 4 -pady 1
	::balloon::set_balloon $w.fm.f1.b "Replace the original video by a short black clip."
	radiobutton $w.fm.f1.r1 -text "Kill playback when it's safe." \
			-variable ::utils::blankout_killpb -value 1 -pady 0
	pack  $w.fm.f1.r1 -anchor w -padx 24 -pady 0
	::balloon::set_balloon $w.fm.f1.r1 "Convert Post to Pre commands only if there are no cell commands in the PGC\nor when the cell commands are compatible with the kill."
	radiobutton $w.fm.f1.r2 -text "Kill playback unconditionally." \
			-variable ::utils::blankout_killpb -value 2 -pady 0
	pack  $w.fm.f1.r2 -anchor w -padx 24 -pady 0
	::balloon::set_balloon $w.fm.f1.r2 "Kill playback even for PGCs with Cell commands.  May be unsafe!"
	radiobutton $w.fm.f1.r3 -text "Don't kill playback." \
			-variable ::utils::blankout_killpb -value 0 -pady 0
	pack  $w.fm.f1.r3 -anchor w -padx 24 -pady 0
	::balloon::set_balloon $w.fm.f1.r3 "Never convert Post to Pre commands.  Some black frames will be played."

	# Replace with clip frame
	frame $w.fm.f2 -bd 2 -relief groove
	pack  $w.fm.f2 -padx 4 -pady 4 -fill x
	set txt "Replace all $type by an imported clip (like a 'feature removed' still frame)."
	set txt "$txt\nThe imported clip MUST have only ONE VOB/Cell ID!"
	radiobutton $w.fm.f2.b -text $txt -justify left \
			-variable ::utils::blankout_type -value "clip" -anchor w \
			-command "::utils::blank_out_PGCS_validate $w"
	pack  $w.fm.f2.b -fill x -padx 4
	::balloon::set_balloon $w.fm.f2.b "Replace all videos by a still frame or an animated clip."
	frame $w.fm.f2.f
	pack  $w.fm.f2.f -fill x -padx 4 -fill x
	frame $w.fm.f2.f.f
	pack  $w.fm.f2.f.f -fill x -padx 20
	radiobutton $w.fm.f2.f.f.b1 -text "Display still frame" \
			-variable ::utils::blankout_clip -value "pause" -anchor w \
			-command "::utils::blank_out_PGCS_validate $w"
	pack  $w.fm.f2.f.f.b1 -fill x -side left
	::balloon::set_balloon $w.fm.f2.f.f.b1 "Pause N seconds after the clip has finished playing."
	entry $w.fm.f2.f.f.e -textvariable ::utils::blankout_clip_celltime -width 4 \
			-validate all -validatecommand "::ced::value_entry_validate 0 254 {%d} {%P}"
	pack  $w.fm.f2.f.f.e -side left
	label $w.fm.f2.f.f.l -text "seconds. (max 254)"
	pack  $w.fm.f2.f.f.l -side left
	radiobutton $w.fm.f2.f.b2 -text "Don't pause: it's an animated clip." \
			-variable ::utils::blankout_clip -value "anim" -anchor w \
			-command "::utils::blank_out_PGCS_validate $w"
	pack  $w.fm.f2.f.b2 -fill x -padx 20
	::balloon::set_balloon $w.fm.f2.f.b2 "Continue immediately when the clip has finished playing."
	radiobutton $w.fm.f2.f.b3 -text "BOV: use the Menu from source file. (For expert users!)\nThe menu MUST be in the FIRST PGC with a video cell,\nand have a 'Continue' button, with LinkTailPGC." \
			-variable ::utils::blankout_clip -value "bov" -anchor w -justify left \
			-command "::utils::blank_out_PGCS_validate $w"
	pack  $w.fm.f2.f.b3 -fill x -padx 20
	::balloon::set_balloon $w.fm.f2.f.b3 "Replace the video by a menu with a 'continue' button."

	# What to do with VOBs? frame
	frame $w.f3 -bd 2 -relief groove
	pack  $w.f3 -padx 4 -fill x
	radiobutton $w.f3.b1 -text "Move original VOB file(s) to backup." \
			-variable ::utils::blankout_movevob -value true -anchor w
	pack  $w.f3.b1 -fill x -padx 4
	::balloon::set_balloon $w.f3.b1 "Keep a backup of the original VOB."
	radiobutton $w.f3.b2 -text "Delete original VOB file(s)." \
			-variable ::utils::blankout_movevob -value false -anchor w
	pack  $w.f3.b2 -fill x -padx 4
	::balloon::set_balloon $w.f3.b2 "Overwrite the original VOB."

	# buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -anchor e
	button $w.fb.ok -text "OK" -width 10 -command {set ::utils::blankout_done 1}
	focus  $w.fb.ok
	pack   $w.fb.ok -side left -padx 2
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::utils::blankout_done 0}
	pack   $w.fb.cancel -side left -padx 2

	blank_out_PGCS_validate $w

	update
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm maxsize $w [winfo reqwidth $w] [winfo reqheight $w]
	set wx [expr {[winfo rootx .]+(([winfo width  .]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm deiconify $w
	focus $w
	grab set $w

	tkwait variable ::utils::blankout_done
	grab release $w
	destroy $w

	if {$::utils::blankout_done} {
		# do the job

		::preview::close_preview

		switch $::utils::blankout_type {
			"rem" {
				blank_out_PGCS_remove_menu $vtsnum
			}
			"black" {
				blank_out_PGCS_blank $vtsnum $type $::utils::blankout_killpb
			}
			"clip" {
				if {$::utils::blankout_clip_celltime == ""} {
					set ::utils::blankout_clip_celltime 5
				}
				switch $::utils::blankout_clip {
					"pause" {blank_out_PGCS_replace $vtsnum $type $::utils::blankout_clip_celltime}
					"anim"  {blank_out_PGCS_replace $vtsnum $type 0}
					"bov"   {blank_out_PGCS_replace $vtsnum $type 255}
				}
			}
		}
		# rebuild the time maps of all titles of the VTS
		if {$type == "Titles"} {
			::tmap::need_new_tmap $vtsnum
		}
	}
	return
}
proc blank_out_PGCS_validate {w} {
	switch $::utils::blankout_type {
		"rem" {
			$w.fm.f2.f.f.b1 configure -state disabled
			$w.fm.f2.f.f.e  configure -state disabled
			$w.fm.f2.f.f.l  configure -state disabled
			$w.fm.f2.f.b2   configure -state disabled
			$w.fm.f2.f.b3   configure -state disabled
			$w.fm.f1.r1     configure -state disabled
			$w.fm.f1.r2     configure -state disabled
			$w.fm.f1.r3     configure -state disabled
		}
		"black" {
			$w.fm.f2.f.f.b1 configure -state disabled
			$w.fm.f2.f.f.e  configure -state disabled
			$w.fm.f2.f.f.l  configure -state disabled
			$w.fm.f2.f.b2   configure -state disabled
			$w.fm.f2.f.b3   configure -state disabled
			$w.fm.f1.r1     configure -state normal
			$w.fm.f1.r2     configure -state normal
			$w.fm.f1.r3     configure -state normal
		}
		"clip" {
			$w.fm.f2.f.f.b1 configure -state normal
			if {$::utils::blankout_clip == "pause"} {
				$w.fm.f2.f.f.e  configure -state normal
				$w.fm.f2.f.f.l  configure -state normal
			} else {
				$w.fm.f2.f.f.e  configure -state disabled
				$w.fm.f2.f.f.l  configure -state disabled
			}
			$w.fm.f2.f.b2   configure -state normal
			$w.fm.f2.f.b3   configure -state normal
			$w.fm.f1.r1     configure -state disabled
			$w.fm.f1.r2     configure -state disabled
			$w.fm.f1.r3     configure -state disabled
		}
	}
	if {$::utils::blankout_clip_celltime == ""} {
		set ::utils::blankout_clip_celltime 5
	}
}
proc blank_out_remove_vobs {vtsnum type} {
	# remove (or move to backup) the original VTST VOBs.
	if {$type == "Titles"} {
		set destvobbasename [file join $::dvddir "VTS_[format %02i $vtsnum]_"]
		if {$::utils::blankout_movevob} {
			if {! [file exists $::backupdir]} {makedir $::backupdir "PgcEdit: Create Backup"}
			set back true
			if {[file exists [file join $::backupdir "VTS_[format %02i $vtsnum]_1.VOB"]]} {
				set r [tk_messageBox -title "PgcEdit: Backup" -icon warning \
					-message "Backup of VOB files already exists!\nOverwrite?" -parent . \
					-type yesno -default no]
				if {$r == "no"} {set back false}
			}
			if {$back} {
				for {set i 1} {$i < 10} {incr i} {
					deletefile [file join $::backupdir VTS_[format %02i $vtsnum]_$i.VOB] "PgcEdit: Blank Out VOB" .
				if {[file exists "${destvobbasename}$i.VOB"]} {
						movefile ${destvobbasename}$i.VOB $::backupdir -force "PgcEdit: Blank Out VOB" .
					}
				}
			} else {
				for {set i 1} {$i < 10} {incr i} {
					deletefile "${destvobbasename}$i.VOB" "PgcEdit: Blank Out VOB" .
				}
			}
		} else {
			for {set i 1} {$i < 10} {incr i} {
				deletefile "${destvobbasename}$i.VOB" "PgcEdit: Blank Out VOB" .
			}
		}
	} else {
		if {$vtsnum == 0} {
			set name "VIDEO_TS.VOB"
		} else {
			set name "VTS_[format %02i $vtsnum]_0.VOB"
		}
		if {$::utils::blankout_movevob} {
			if {! [file exists $::backupdir]} {makedir $::backupdir "PgcEdit: Create Backup"}
			set back true
			if {[file exists [file join $::backupdir $name]]} {
				set r [tk_messageBox -title "PgcEdit: Backup" -icon warning \
					-message "Backup of VOB file already exists!\nOverwrite?" -parent . \
					-type yesno -default no]
				if {$r == "no"} {set back false}
			}
			if {$back} {
				if {[file exists [file join $::dvddir $name]]} {
					deletefile [file join $::backupdir $name] "PgcEdit: Blank Out VOB" .
					movefile [file join $::dvddir $name] $::backupdir -force "PgcEdit: Blank Out VOB" .
				}
			} else {
				deletefile [file join $::dvddir $name] "PgcEdit: Blank Out VOB" .
			}
		} else {
			deletefile [file join $::dvddir $name] "PgcEdit: Blank Out VOB" .
		}
	}
}

#############################################################################
## Remove Menu VOB
proc blank_out_PGCS_remove_menu {vtsnum} {
	if {$vtsnum == 0} {
		set vtsi_table_name "VMGM_MAT"
		set tablesubname "VMGM"
	} else {
		set vtsi_table_name "VTSI_MAT"
		set tablesubname "VTSM"
	}

	set totalpbtime "\000\000\000\000"
	set vattr [get_int ::raw_sectors($vtsnum,$vtsi_table_name) 0x100 2]
	set ::raw_sectors($vtsnum,$vtsi_table_name) \
		[string range $::raw_sectors($vtsnum,$vtsi_table_name) 0 [expr {(0xFF)}]][string repeat \000 [expr {(0x100)}]][string range $::raw_sectors($vtsnum,$vtsi_table_name) [expr {(0x200)}] end]
	if {[expr {$vattr & 0x3000}] == 0} {
		set_int totalpbtime 0 192 4
	} else {
		set_int totalpbtime 0 64 4
		set_int ::raw_sectors($vtsnum,$vtsi_table_name) 0x100 0x1000 2
	}

	# Fix all cells in all PGCS of current VTS
	for {set lu 1} {$lu <= $::menus($vtsnum,numlus)} {incr lu} {
		for {set pgc 1} {$pgc <= $::menus($vtsnum,$lu,numpgcs)} {incr pgc} {
			# Kill PGC Playback
			::macros::kill_pgc_playback ::menus($vtsnum,$lu,$pgc) always never

			set ::menus($vtsnum,$lu,$pgc,numcells) 0
			set ::menus($vtsnum,$lu,$pgc,numprograms) 0
			set ::menus($vtsnum,$lu,$pgc,programmap) ""
			set ::menus($vtsnum,$lu,$pgc,cellpbtable) ""
			set ::menus($vtsnum,$lu,$pgc,cellpostable) ""
			# fix PGC total time
			set ::menus($vtsnum,$lu,$pgc,header) \
					"[string range $::menus($vtsnum,$lu,$pgc,header) 0 3]$totalpbtime[string range $::menus($vtsnum,$lu,$pgc,header) 8 end]"
			# reset PGC Still Time to 0
			set_int ::menus($vtsnum,$lu,$pgc,header) 0xA3 0 1
		}
	}
	# remove old files
	blank_out_remove_vobs $vtsnum "Menus"

	# remove the VTS_C_ADT and VTS_VOBU_ADMAP tables
	set ::raw_sectors($vtsnum,${tablesubname}_C_ADT) ""
	set ::raw_sectors($vtsnum,${tablesubname}_VOBU_ADMAP) ""

	foreach a [array names ::buttons::butsinfo menu,$vtsnum,*] {
		unset ::buttons::butsinfo($a)
	}
	set ::buttons::butsinfo(menu,$vtsnum,errorcode) 1
	set ::buttons::butsinfo(menu,$vtsnum,errormsg) "Can't open menu VOB file."
	set ::buttons::butsinfo(menu,$vtsnum,numvobids) 0

	# Reinit GUI
	set lunum  [lindex [split [lindex $::editor_cmds_from 1] ","] 1]
	set pgcnum [lindex [split [lindex $::editor_cmds_from 1] ","] 2]
	::main::init_tree_menu_no_move
	::main::display_pgc menu $vtsnum $lunum $pgcnum
	::main::dvd_modified true false
	show_msg "VTS $vtsnum Menu VOB removed OK."
	return
}

#############################################################################
#############################################################################
## Blank out all VTST Titles and replace them with black frame
proc blank_out_PGCS_blank {vtsnum type killpb} {
	if {$type == "Menus"} {
		set vtsi_mat_idx 0x100
		if {$vtsnum == 0} {
			set destvobname [file join $::dvddir "VIDEO_TS.VOB"]
			set tablesubname "VMGM"
		} else {
			set destvobname [file join $::dvddir "VTS_[format %02i $vtsnum]_0.VOB"]
			set tablesubname "VTSM"
		}
	} else {
		set vtsi_mat_idx 0x200
		set destvobname [file join $::dvddir "VTS_[format %02i $vtsnum]_1.VOB"]
		set tablesubname "VTS"
	}
	if {$vtsnum == 0} {
		set vtsi_table_name "VMGM_MAT"
	} else {
		set vtsi_table_name "VTSI_MAT"
	}
	set totalpbtime    "\000\000\000\000"
	set standardpbtime "\000\000\000\000"
	set vattr [get_int ::raw_sectors($vtsnum,$vtsi_table_name) $vtsi_mat_idx 2]
	if {[expr {$vattr & 0x3000}] == 0} {
		set standard "ntsc"
		set_int standardpbtime 0 210 4
	} else {
		set standard "pal"
		set_int standardpbtime 0 82 4
	}
	# Fix all cells in all PGCS of current VTS
	if {$type == "Menus"} {
		for {set lu 1} {$lu <= $::menus($vtsnum,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vtsnum,$lu,numpgcs)} {incr pgc} {
				# if there are no cell commands, or if force kill is on, Kill PGC Playback
				if {$killpb == 1} {
					::macros::kill_pgc_playback ::menus($vtsnum,$lu,$pgc) safely never
				} elseif {$killpb == 2} {
					::macros::kill_pgc_playback ::menus($vtsnum,$lu,$pgc) always never
				}
				set ::menus($vtsnum,$lu,$pgc,numbuttons) 0

				if {$standard == "ntsc"} {
					set_int totalpbtime    0 192 4
				} else {
					set_int totalpbtime    0 64 4
				}
				for {set cell 1} {$cell <= $::menus($vtsnum,$lu,$pgc,numcells)} {incr cell} {
					set cellidx [expr {($cell-1)*24}]
					;# set cell_type's STC Discontinuity, and remove Seamless Joint
					set typeflags [get_int ::menus($vtsnum,$lu,$pgc,cellpbtable) $cellidx 1]
					set_int ::menus($vtsnum,$lu,$pgc,cellpbtable) $cellidx [expr {($typeflags & 0xF7) | 2}] 1
					;# clear restricted flag
					set_int ::menus($vtsnum,$lu,$pgc,cellpbtable) [expr {$cellidx+1}] 0 1
					# still time = 0
					set_int ::menus($vtsnum,$lu,$pgc,cellpbtable) [expr {$cellidx+2}] 0 1
					# Set PB time = still PB time in Cell Playback table
					set ::menus($vtsnum,$lu,$pgc,cellpbtable) \
							"[string range $::menus($vtsnum,$lu,$pgc,cellpbtable) 0 [expr {$cellidx+3}] \
							]$standardpbtime\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\004[ \
							string range $::menus($vtsnum,$lu,$pgc,cellpbtable) [expr {$cellidx+24}] end]"

					# Use VOB ID 1, Cell ID 1 in Cell Position table
					set idx [expr {($cell-1)*4}]
					set_int ::menus($vtsnum,$lu,$pgc,cellpostable) $idx 1 2
					set_int ::menus($vtsnum,$lu,$pgc,cellpostable) [expr {$idx+3}] 1 1
					# Add cell time to total PGC time, but only if angle 1.
					if {[expr {$typeflags & 0x10}] == 0 || [expr {$typeflags & 0xC0}] == 0x40} {
						set totalpbtime [::pgced::AddBCDtime $totalpbtime $standardpbtime]
					}
				}
				# fix PGC total time
				set ::menus($vtsnum,$lu,$pgc,header) \
						"[string range $::menus($vtsnum,$lu,$pgc,header) 0 3]$totalpbtime[string range $::menus($vtsnum,$lu,$pgc,header) 8 end]"
				# reset PGC Still Time to 0
				set_int ::menus($vtsnum,$lu,$pgc,header) 0xA3 0 1
			}
		}
		foreach a [array names ::buttons::butsinfo menu,$vtsnum,*] {
			unset ::buttons::butsinfo($a)
		}
		set ::buttons::butsinfo(menu,$vtsnum,errorcode) 0
		set ::buttons::butsinfo(menu,$vtsnum,errormsg) "OK"
		set ::buttons::butsinfo(menu,$vtsnum,numvobids) 1
		set ::buttons::butsinfo(menu,$vtsnum,1,numcellids) 1
		set ::buttons::butsinfo(menu,$vtsnum,1,1,navpack) 0
		set ::buttons::butsinfo(menu,$vtsnum,1,1,numbuts) 0
	} else {
		for {set pgc 1} {$pgc <= $::pgcs($vtsnum,numpgcs)} {incr pgc} {
			# if there are no cell commands, or if force kill is on, Kill PGC Playback
			if {$killpb == 1} {
				::macros::kill_pgc_playback ::pgcs($vtsnum,$pgc) safely never
			} elseif {$killpb == 2} {
				::macros::kill_pgc_playback ::pgcs($vtsnum,$pgc) always never
			}
			set ::pgcs($vtsnum,$pgc,numbuttons) 0
			if {$standard == "ntsc"} {
				set_int totalpbtime    0 192 4
			} else {
				set_int totalpbtime    0 64 4
			}
			set cellpbtable $::pgcs($vtsnum,$pgc,cellpbtable)
			for {set cell 1} {$cell <= $::pgcs($vtsnum,$pgc,numcells)} {incr cell} {
				set cellidx [expr {($cell-1)*24}]
				;# set cell_type's STC Discontinuity and remove Seamless Joint
				set typeflags [get_int cellpbtable $cellidx 1]

				set_int cellpbtable $cellidx [expr {($typeflags & 0xF7) | 2}] 1
				;# clear restricted flag
				set_int cellpbtable [expr {$cellidx+1}] 0 1
				# still time = 0
				set_int cellpbtable [expr {$cellidx+2}] 0 1
				# Set PB time = still PB time in Cell Playback table
				set cellpbtable \
					"[string range $cellpbtable 0 [expr {$cellidx+3}] \
					]$standardpbtime\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\004[ \
					string range $cellpbtable [expr {$cellidx+24}] end]"

				# Use VOB ID 1, Cell ID 1 in Cell Position table
				set idx [expr {($cell-1)*4}]
				set_int ::pgcs($vtsnum,$pgc,cellpostable) $idx 1 2
				set_int ::pgcs($vtsnum,$pgc,cellpostable) [expr {$idx+3}] 1 1
				# Add cell time to total PGC time, but only if angle 1.
				if {[expr {$typeflags & 0x10}] == 0 || [expr {$typeflags & 0xC0}] == 0x40} {
					set totalpbtime [::pgced::AddBCDtime $totalpbtime $standardpbtime]
				}
			}
			set ::pgcs($vtsnum,$pgc,cellpbtable) $cellpbtable
			# fix PGC total time
			set ::pgcs($vtsnum,$pgc,header) \
					"[string range $::pgcs($vtsnum,$pgc,header) 0 3]$totalpbtime[string range $::pgcs($vtsnum,$pgc,header) 8 end]"
			# reset PGC Still Time to 0
			set_int ::pgcs($vtsnum,$pgc,header) 0xA3 0 1
		}
		foreach a [array names ::buttons::butsinfo pgc,$vtsnum,*] {
			unset ::buttons::butsinfo($a)
		}
		set ::buttons::butsinfo(pgc,$vtsnum,errorcode) 0
		set ::buttons::butsinfo(pgc,$vtsnum,errormsg) "OK"
		set ::buttons::butsinfo(pgc,$vtsnum,numvobids) 1
		set ::buttons::butsinfo(pgc,$vtsnum,1,numcellids) 1
		set ::buttons::butsinfo(pgc,$vtsnum,1,1,navpack) 0
		set ::buttons::butsinfo(pgc,$vtsnum,1,1,numbuts) 0
	}

	# remove old files
	blank_out_remove_vobs $vtsnum $type

	# patch the current video attributes in IFOs
	set vattr [expr {($vattr & 0x3F0E) | 0x4000}]	;# MPEG 2, VBR, Camera, both NTSC CC off.
	set_int ::raw_sectors($vtsnum,$vtsi_table_name) $vtsi_mat_idx $vattr 2

	# set number of audio streams and number of subpics to 0
	set_int ::raw_sectors($vtsnum,$vtsi_table_name) [expr {$vtsi_mat_idx + 2}] 0 2
	set_int ::raw_sectors($vtsnum,$vtsi_table_name) [expr {$vtsi_mat_idx + 0x54}] 0 2

	# copy the new blank VOB
	saveFile $destvobname binary [::blankcell::blankCell $vattr 1 1 0]

	# replace the VTS_C_ADT and VTS_VOBU_ADMAP tables with blank 1 cell VOB tables
	set ::raw_sectors($vtsnum,${tablesubname}_C_ADT) \
		"\000\001\000\000\000\000\000\023\000\001\001\000\000\000\000\000\000\000\000\004"
	set ::raw_sectors($vtsnum,${tablesubname}_VOBU_ADMAP) \
		"\000\000\000\007\000\000\000\000"

	# Reinit GUI
	#refresh_current_pgc
	::main::init_tree_menu_no_move
	::main::dvd_modified true false
	show_msg "VTS $vtsnum $type blanked OK."
	return
}


#############################################################################
## Replace all VTST Titles by an imported title
proc blank_out_PGCS_replace {vtsnum type cell_still_time} {

	# set destination defaults
	if {$type == "Menus"} {
		set vtsi_mat_idx 0x100
		if {$vtsnum == 0} {
			set destvobname [file join $::dvddir "VIDEO_TS.VOB"]
			set tablesubname "VMGM"
		} else {
			set destvobname [file join $::dvddir "VTS_[format %02i $vtsnum]_0.VOB"]
			set tablesubname "VTSM"
		}
	} else {
		set vtsi_mat_idx 0x200
		set tablesubname "VTS"
	}
	if {$vtsnum == 0} {
		set vtsi_table_name "VMGM_MAT"
	} else {
		set vtsi_table_name "VTSI_MAT"
	}

	# set source defaults
	if {$::utils::blankout_clip == "bov"} {
		set filetypes {{"IFO files" {*.IFO}}}
	} else {
		set filetypes {{"VTS IFO files" {*_0.IFO}}}
	}
	set sourceifofile [tk_getOpenFile -initialdir $::config(importmenuinitialdir) \
		-filetypes $filetypes \
		-title "Select the VTS IFO file with the clip to import"]
	if {$sourceifofile == ""} {return 0}
	if {$::utils::blankout_clip == "bov" && [file tail $sourceifofile] == "VIDEO_TS.IFO"} {
		show_warning -title $utiltitle -icon error -parent . \
				-message "You must select a VTS IFO (VTS_**_*.IFO)"
		return 0
	}
	set sourceifofile [file nativename $sourceifofile]
	set ::config(importmenuinitialdir) [file dirname $sourceifofile]

	# read the VTS
	set sourcevtsnum [readdvdfile $sourceifofile ::imported_sectors ::imported_pgcs ::imported_menus ::imported_ttns]
	if {$sourcevtsnum == -1} {return 0}


	# Process Menu if BOV mode
	if {$::utils::blankout_clip == "bov"} {
		# Find out the first menu PGC with video
		set menupgc 0
		for {set p 1} {$p <= $::imported_menus($sourcevtsnum,1,numpgcs)} {incr p} {
			if {$::imported_menus($sourcevtsnum,1,$p,numcells)>0} {
				set menupgc $p
				break
			}
		}
		if {$menupgc == 0} {
			show_warning -title $utiltitle -icon error -parent . -message "No menu with video found!"
			return 0
		}
		set importedpbtime [string range $::imported_menus($sourcevtsnum,1,$menupgc,cellpbtable) 4 7]
		set importedpbtablepart2 [string range $::imported_menus($sourcevtsnum,1,$menupgc,cellpbtable) 4 23]
		if {$sourcevtsnum == 0} {
			set importedstandard [expr {[get_int ::imported_sectors(0,VMGM_MAT) 0x100 2] & 0x3000}]
			set source_C_ADT_table $::imported_sectors(0,VMGM_C_ADT)
			set source_VOBU_ADMAP_table $::imported_sectors(0,VMGM_VOBU_ADMAP)
			set source_vtsi_table_name "VMGM_MAT"
		} else {
			set importedstandard [expr {[get_int ::imported_sectors($sourcevtsnum,VTSI_MAT) 0x100 2] & 0x3000}]
			set source_C_ADT_table $::imported_sectors($sourcevtsnum,VTSM_C_ADT)
			set source_VOBU_ADMAP_table $::imported_sectors($sourcevtsnum,VTSM_VOBU_ADMAP)
			set source_vtsi_table_name "VTSI_MAT"
		}
		set source_vtsi_table_address [expr {(0x100)}]
	} else {
		# How many TTNs are in the imported VTS?
		set num_new_ttns [get_int ::imported_sectors($sourcevtsnum,VTS_PTT_SRPT) 0 2]
		if {$num_new_ttns == 0 || $num_new_ttns > 1} {
			show_warning -title $utiltitle -icon error -parent . \
					-message "Error: you must select a VTS with one and only one title!"
			# cleanup
			unset ::imported_sectors ::imported_pgcs ::imported_menus
			return 0
		}
		set importedpbtime [string range $::imported_pgcs($sourcevtsnum,1,cellpbtable) 4 7]
		set importedpbtablepart2 [string range $::imported_pgcs($sourcevtsnum,1,cellpbtable) 4 23]
		set importedstandard [expr {[get_int ::imported_sectors($sourcevtsnum,VTSI_MAT) 0x200 2] & 0x3000}]

		set source_C_ADT_table $::imported_sectors($sourcevtsnum,VTS_C_ADT)
		set source_VOBU_ADMAP_table $::imported_sectors($sourcevtsnum,VTS_VOBU_ADMAP)
		set source_vtsi_table_name "VTSI_MAT"
		set source_vtsi_table_address [expr {(0x200)}]
	}
	if {$importedstandard == 0} {	;# if standard of imported clip is NTSC
		set importedstandard "NTSC"
	} else {
		set importedstandard "PAL"
	}

	# copy the TTN VOBs
	show_msg "Copying VOBs..."
	update idletasks
	after 100 {set ::pause false}
	tkwait variable ::pause
	set destvobbasename [file join $::dvddir "VTS_[format %02i $vtsnum]_"]
	if {$::utils::blankout_clip == "bov"} {
		set sourcevobname "[file rootname $sourceifofile].VOB"
	} else {
		set sourcevobbasename [string trimright [file rootname $sourceifofile] "0"]
		set sourcevobname "${sourcevobbasename}1.VOB"
	}
	if {$type == "Menus"} {
		if {$vtsnum == 0} {
			if {$sourcevobname != [file join $::dvddir "VIDEO_TS.VOB"]} {
				# remove old files
				blank_out_remove_vobs $vtsnum $type
				if {[file exists $sourcevobname]} {
					if {[file exists [file join $::dvddir "VIDEO_TS.VOB"]]} {
						deletefile [file join $::dvddir "VIDEO_TS.VOB"] "PgcEdit: Blank Out VOB" .
					}
					copyfile $sourcevobname [file join $::dvddir "VIDEO_TS.VOB"] -force "PgcEdit: Blank Out VOB" .
				} else bell
			}
		} else {
			if {$sourcevobname != "${destvobbasename}0.VOB"} {
				# remove old files
				blank_out_remove_vobs $vtsnum $type
				if {[file exists $sourcevobname]} {
					if {[file exists "${destvobbasename}0.VOB"]} {
						deletefile "${destvobbasename}0.VOB" "PgcEdit: Blank Out VOB" .
					}
					copyfile $sourcevobname "${destvobbasename}0.VOB" -force "PgcEdit: Blank Out VOB" .
				} else bell
			}
		}
	} else {
		if {$::utils::blankout_clip != "bov"} {
			if {"${sourcevobbasename}1.VOB" != "${destvobbasename}1.VOB"} {
				# remove old files
				blank_out_remove_vobs $vtsnum $type
				for {set i 1} {$i < 10} {incr i} {
					if {[file exists "${destvobbasename}$i.VOB"]} {
						deletefile "${destvobbasename}$i.VOB" "PgcEdit: Blank Out VOB" .
					}
					if {[file exists "${sourcevobbasename}$i.VOB"]} {
						copyfile "${sourcevobbasename}$i.VOB" "${destvobbasename}$i.VOB" -force "PgcEdit: Blank Out VOB" .
					}
				}
			}
		} else {
			if {$sourcevobname != "${destvobbasename}1.VOB"} {
				# remove old files
				blank_out_remove_vobs $vtsnum $type
				for {set i 1} {$i < 10} {incr i} {
					if {[file exists "${destvobbasename}$i.VOB"]} {
						deletefile "${destvobbasename}$i.VOB" "PgcEdit: Blank Out VOB" .
					}
				}
				if {[file exists $sourcevobname]} {
					copyfile $sourcevobname "${destvobbasename}1.VOB" -force "PgcEdit: Blank Out VOB" .
				}
			}
		}
	}

	# Fix all cells in all PGCS of current VTS
	set totalpbtime "\000\000\000\000"
	if {$type == "Menus"} {
		for {set lu 1} {$lu <= $::menus($vtsnum,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vtsnum,$lu,numpgcs)} {incr pgc} {
				if {$importedstandard == "NTSC"} {
					set_int totalpbtime    0 192 4
				} else {
					set_int totalpbtime    0 64 4
				}
				for {set cell 1} {$cell <= $::menus($vtsnum,$lu,$pgc,numcells)} {incr cell} {
					set cellidx [expr {($cell-1)*24}]

					;# set cell_type's STC Discontinuity and remove Seamless joint
					set typeflags [get_int ::menus($vtsnum,$lu,$pgc,cellpbtable) $cellidx 1]
					set_int ::menus($vtsnum,$lu,$pgc,cellpbtable) $cellidx [expr {($typeflags & 0xF7) | 2}] 1

					;# clear restricted flag
					set_int ::menus($vtsnum,$lu,$pgc,cellpbtable) [expr {$cellidx+1}] 0 1
					# if first cell or previous cell has cell command, then set the still frame delay,
					# else set still frame delay = 0
					if {$cell == 1 || \
							[get_int ::menus($vtsnum,$lu,$pgc,cellpbtable) [expr {$cellidx+3-24}] 1] > 0} {
						set_int ::menus($vtsnum,$lu,$pgc,cellpbtable) [expr {$cellidx+2}] $cell_still_time 1
					} else {
						set_int ::menus($vtsnum,$lu,$pgc,cellpbtable) [expr {$cellidx+2}] 0 1
					}
					# Set PB time = still PB time in Cell Playback table
					set ::menus($vtsnum,$lu,$pgc,cellpbtable) \
							"[string range $::menus($vtsnum,$lu,$pgc,cellpbtable) 0 [expr {$cellidx+3}] \
							]$importedpbtablepart2[ \
							string range $::menus($vtsnum,$lu,$pgc,cellpbtable) [expr {$cellidx+24}] end]"

					# Use VOB ID 1, Cell ID 1 in Cell Position table
					set idx [expr {($cell-1)*4}]
					set_int ::menus($vtsnum,$lu,$pgc,cellpostable) $idx 1 2
					set_int ::menus($vtsnum,$lu,$pgc,cellpostable) [expr {$idx+3}] 1 1

					# Add cell time to total PGC time, but only if angle 1.
					if {[expr {$typeflags & 0x10}] == 0 || [expr {$typeflags & 0xC0}] == 0x40} {
						set totalpbtime [::pgced::AddBCDtime $totalpbtime $importedpbtime]
					}
				}
				# fix PGC total time
				set ::menus($vtsnum,$lu,$pgc,header) \
						"[string range $::menus($vtsnum,$lu,$pgc,header) 0 3]$totalpbtime[string range $::menus($vtsnum,$lu,$pgc,header) 8 end]"
				# reset PGC Still Time to 0
				set_int ::menus($vtsnum,$lu,$pgc,header) 0xA3 0 1

				# Clear all PUOs
				set_int ::menus($vtsnum,$lu,$pgc,header) 0x08 0 4
				# Add the NextPGCN and PrevPGCN pointers to be able to use the Next and Prev buttons.
				if {[get_int ::menus($vtsnum,$lu,$pgc,header) 0x9C 2] == 0 &&
						[num_type_cmds post ::menus($vtsnum,$lu,$pgc)] > 0} {
					 set_int ::menus($vtsnum,$lu,$pgc,header) 0x9C $pgc 2
				}
				if {[get_int ::menus($vtsnum,$lu,$pgc,header) 0x9E 2] == 0} {
					 set_int ::menus($vtsnum,$lu,$pgc,header) 0x9E $pgc 2
				}
			}
		}
	} else {
		for {set pgc 1} {$pgc <= $::pgcs($vtsnum,numpgcs)} {incr pgc} {
			if {$importedstandard == "NTSC"} {
				set_int totalpbtime    0 192 4
			} else {
				set_int totalpbtime    0 64 4
			}
			for {set cell 1} {$cell <= $::pgcs($vtsnum,$pgc,numcells)} {incr cell} {
				set cellidx [expr {($cell-1)*24}]

				;# set cell_type's STC Discontinuity, and remove Seamless joint
				set typeflags [get_int ::pgcs($vtsnum,$pgc,cellpbtable) $cellidx 1]
				set_int ::pgcs($vtsnum,$pgc,cellpbtable) $cellidx [expr {($typeflags & 0xF7) | 2}] 1
				;# clear restricted flag
				set_int ::pgcs($vtsnum,$pgc,cellpbtable) [expr {$cellidx+1}] 0 1
				# if first cell or previous cell has cell command, then set the still frame delay,
				# else set still frame delay = 0
				if {$cell == 1 || \
						[get_int ::pgcs($vtsnum,$pgc,cellpbtable) [expr {$cellidx+3-24}] 1] > 0} {
					set_int ::pgcs($vtsnum,$pgc,cellpbtable) [expr {$cellidx+2}] $cell_still_time 1
				} else {
					set_int ::pgcs($vtsnum,$pgc,cellpbtable) [expr {$cellidx+2}] 0 1
				}
				# Set PB time = still PB time in Cell Playback table
				set ::pgcs($vtsnum,$pgc,cellpbtable) \
						"[string range $::pgcs($vtsnum,$pgc,cellpbtable) 0 [expr {$cellidx+3}] \
						]$importedpbtablepart2[ \
						string range $::pgcs($vtsnum,$pgc,cellpbtable) [expr {$cellidx+24}] end]"

				# Use VOB ID 1, Cell ID 1 in Cell Position table
				set idx [expr {($cell-1)*4}]
				set_int ::pgcs($vtsnum,$pgc,cellpostable) $idx 1 2
				set_int ::pgcs($vtsnum,$pgc,cellpostable) [expr {$idx+3}] 1 1

				# Add cell time to total PGC time, but only if angle 1.
				if {[expr {$typeflags & 0x10}] == 0 || [expr {$typeflags & 0xC0}] == 0x40} {
					set totalpbtime [::pgced::AddBCDtime $totalpbtime $importedpbtime]
				}
			}
			# fix PGC total time
			set ::pgcs($vtsnum,$pgc,header) \
					"[string range $::pgcs($vtsnum,$pgc,header) 0 3]$totalpbtime[string range $::pgcs($vtsnum,$pgc,header) 8 end]"
			# reset PGC Still Time to 0
			set_int ::pgcs($vtsnum,$pgc,header) 0xA3 0 1
			# Clear all PUOs
			set_int ::pgcs($vtsnum,$pgc,header) 0x08 0 4
			# Add the NextPGCN and PrevPGCN pointers to be able to use the Next and Prev buttons.
			if {[get_int ::pgcs($vtsnum,$pgc,header) 0x9C 2] == 0 &&
					[num_type_cmds post ::pgcs($vtsnum,$pgc)] > 0} {
				 set_int ::pgcs($vtsnum,$pgc,header) 0x9C $pgc 2
			}
			if {[get_int ::pgcs($vtsnum,$pgc,header) 0x9E 2] == 0} {
				 set_int ::pgcs($vtsnum,$pgc,header) 0x9E $pgc 2
			}
		}
	}

	### Go modify the DVD!

	# replace the VTS_C_ADT and VTS_VOBU_ADMAP tables with blank imported ones
	set ::raw_sectors($vtsnum,${tablesubname}_C_ADT) $source_C_ADT_table
	set ::raw_sectors($vtsnum,${tablesubname}_VOBU_ADMAP) $source_VOBU_ADMAP_table

	# copy the Video/Audio/Subpic attributes in VTSI_MAT
	set vattr [get_int ::imported_sectors($sourcevtsnum,$source_vtsi_table_name) $source_vtsi_table_address 2]
	set_int ::raw_sectors($vtsnum,$vtsi_table_name) $vtsi_mat_idx $vattr 2

	# copy audio streams attribs
	set numaudio [get_int ::imported_sectors($sourcevtsnum,$source_vtsi_table_name) [expr {$source_vtsi_table_address + 2}] 2]
	set_int ::raw_sectors($vtsnum,$vtsi_table_name) [expr {$vtsi_mat_idx + 2}] $numaudio 2
	set audioattribs [string range $::imported_sectors($sourcevtsnum,$source_vtsi_table_name) [expr {$source_vtsi_table_address + 4}] [expr {$source_vtsi_table_address + 67}]]
	set ::raw_sectors($vtsnum,$vtsi_table_name) [string range $::raw_sectors($vtsnum,$vtsi_table_name) 0 [expr {$vtsi_mat_idx + 3}]]$audioattribs[string range $::raw_sectors($vtsnum,$vtsi_table_name) [expr {$vtsi_mat_idx + 68}] end]

	# copy subpic streams attribs
	set numsubpics [get_int ::imported_sectors($sourcevtsnum,$source_vtsi_table_name) [expr {$source_vtsi_table_address + 0x54}] 2]
	if {$type == "Menus" && $numsubpics > 1} {set numsubpics 1}
	set_int ::raw_sectors($vtsnum,$vtsi_table_name) [expr {$vtsi_mat_idx + 0x54}] $numsubpics 2
	set subpicattribs [string range $::imported_sectors($sourcevtsnum,$source_vtsi_table_name) [expr {$source_vtsi_table_address + 0x56}] [expr {$source_vtsi_table_address + 0x55 + ($numsubpics*6)}]]
	set ::raw_sectors($vtsnum,$vtsi_table_name) [string range $::raw_sectors($vtsnum,$vtsi_table_name) 0 [expr {$vtsi_mat_idx + 0x55}]]$subpicattribs[string range $::raw_sectors($vtsnum,$vtsi_table_name) [expr {$vtsi_mat_idx + 0x56 + ($numsubpics*6)}] end]

	# cleanup
	unset ::imported_sectors ::imported_pgcs ::imported_menus

	# Reinit GUI
	#refresh_current_pgc
	::main::init_tree_menu_no_move
	::main::dvd_modified true false
	show_msg "VTS $vtsnum $type replaced OK."

	# allow user to reload buttons informations
#	if {$type == "Menus"} {
		::readdvd::reload_buttons_infos $vtsnum
#	}

	return
}


#############################################################################
## New (create or replace) the VMGM_PGCI_UT Title Menu PGC
proc new_title_menu {quiet} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	set utiltitle "New VMGM Title Menu PGC"
	if {!$quiet && !$::config(expert_mode)} {
		set txt "This utility will create the VMGM Title menu PGC."
		set txt "$txt\nNo VOB will be associated with the menu."
		set txt "$txt\n"
		set txt "$txt\nContinue ?"
		set r [tk_messageBox -title $utiltitle -icon question -message $txt \
				-parent . -type yesno -default no]
		if {$r == "no"} return
	}

	### create or replace the VMGM menu
	if {$::menus(0,numlus)>0 && !$::config(expert_mode)} {
		set txt "WARNING: There is already a VMGM Title Menu!"
		set txt "$txt\nIf you continue, it will be REPLACED by the new one!"
		set txt "$txt\n"
		set txt "$txt\nContinue anyway ?"
		set r [tk_messageBox -title $utiltitle -icon warning -message $txt \
				-parent . -type yesno -default no]
		if {$r == "no"} return
	}
	set ::raw_sectors(0,VMGM_PGCI_UT) \
			[readFile $::config(apppath)/lib/data/vmgm_pgci_ut_wo_vob.sec binary]

	parse_PGCI_UT ::raw_sectors(0,VMGM_PGCI_UT) 0 ::menus
	::main::init_tree_menu
	::main::dvd_modified true false
	::main::display_pgc menu 0 1 1

	# if there is an old VOB, delete it or move it to backup
	delete_vob_with_confirm "VIDEO_TS.VOB" true
}

#############################################################################
## New (create or replace) a VTSM_PGCI_UT Root Menu PGC
proc new_root_menu {vtsnum quiet} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}

	set utiltitle "New VTSM Root Menu PGC"
	if {!$quiet && !$::config(expert_mode)} {
		set txt "This utility will create the VTSM $vtsnum Root menu PGC."
		set txt "$txt\nNo VOB will be associated with the menu."
		set txt "$txt\n"
		set txt "$txt\nContinue ?"
		set r [tk_messageBox -title $utiltitle -icon question -message $txt \
				-parent . -type yesno -default no]
		if {$r == "no"} return
	}

	### create or replace the VTS menu
	if {$::menus($vtsnum,numlus)>0 && !$::config(expert_mode)} {
		set txt "WARNING: There is already a VTSM $vtsnum menu!"
		set txt "$txt\nIf you continue, it will be REPLACED by the new one!"
		set txt "$txt\n"
		set txt "$txt\nContinue anyway ?"
		set r [tk_messageBox -title $utiltitle -icon warning -message $txt \
				-parent . -type yesno -default no]
		if {$r == "no"} return
	}

	set ::raw_sectors($vtsnum,VTSM_PGCI_UT) \
			[readFile $::config(apppath)/lib/data/vmgm_pgci_ut_wo_vob.sec binary]
	set_int ::raw_sectors($vtsnum,VTSM_PGCI_UT) 0x18 0x83 1	;# change Title menu to Root menu

	parse_PGCI_UT ::raw_sectors($vtsnum,VTSM_PGCI_UT) $vtsnum ::menus
	::main::init_tree_menu
	::main::dvd_modified true false
	::main::display_pgc menu $vtsnum 1 1

	# if there is an old VOB, delete it or move it to backup
	delete_vob_with_confirm "VTS_[format %02d $vtsnum]_0.VOB" true
}

proc new_menu {quiet} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	set vtsnum [lindex [split [lindex $::editor_cmds_from 1] ","] 0]
	if {$vtsnum == 0} {
		new_title_menu $quiet
	} else {
		new_root_menu $vtsnum $quiet
	}

	foreach a [array names ::buttons::butsinfo menu,$vtsnum,*] {
		unset ::buttons::butsinfo($a)
	}
	set ::buttons::butsinfo(menu,$vtsnum,errorcode) 1
	set ::buttons::butsinfo(menu,$vtsnum,errormsg) "Can't open menu VOB file."
	set ::buttons::butsinfo(menu,$vtsnum,numvobids) 0
}

#############################################################################
## Import a whole Menu
proc import_menu {} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	set utiltitle "Import Menu"
	set vtsnum [lindex [split [lindex $::editor_cmds_from 1] ","] 0]
	if {$vtsnum == 0} {
		set current "VMGM"
	} else {
		set current "VTSM $vtsnum"
	}

	if {$::menus($vtsnum,numlus)>0 && !$::config(expert_mode)} {
		set txt "WARNING: There is already a $current menu!"
		set txt "$txt\nIf you continue, it will be REPLACED by the new one!"
		set txt "$txt\n"
		set txt "$txt\nContinue anyway ?"
		set r [tk_messageBox -title $utiltitle -icon warning -message $txt \
				-parent . -type yesno -default no]
		if {$r == "no"} return
	}

	set sourceifofile [tk_getOpenFile -initialdir $::config(importmenuinitialdir) \
		-filetypes {{"IFO files" {*.IFO}}} \
		-title "Select the IFO file with the menu to import"]
	if {$sourceifofile == ""} {return}
	set sourceifofile [file nativename $sourceifofile]
	set ::config(importmenuinitialdir) [file dirname $sourceifofile]

	set sourcevtsnum [readdvdfile $sourceifofile ::imported_sectors ::imported_pgcs ::imported_menus ::imported_ttns]
	if {$sourcevtsnum == -1} return
	if {$sourcevtsnum == 0} {
		set sourcetype "VMG"
		set sourceMAT "VMGM_MAT"
		set sourcetext "VMGM"
	} else {
		set sourcetype "VTS"
		set sourceMAT "VTSI_MAT"
		set sourcetext "VTSM $sourcevtsnum"
	}

	if {$::imported_sectors($sourcevtsnum,${sourcetype}M_PGCI_UT) == ""} {
		show_warning -title $utiltitle -icon warning -parent . \
				-message "Sorry, there is no menu defined in file\n\"$sourceifofile\""
		return
	}
	set menustreamsdefs [string range $::imported_sectors($sourcevtsnum,$sourceMAT) 0x100 0x1FF]
	if {$vtsnum == 0} {
		set destvtstype "VMG"
		set destvobfilename [file join $::dvddir "VIDEO_TS.VOB"]
		set ::raw_sectors(0,VMGM_MAT) \
				[string range $::raw_sectors(0,VMGM_MAT) 0 0xFF]$menustreamsdefs[string range $::raw_sectors(0,VMGM_MAT) 0x200 end]
	} else {
		set destvtstype "VTS"
		set destvobfilename [file join $::dvddir "VTS_[format %02i $vtsnum]_0.VOB"]
		set ::raw_sectors($vtsnum,VTSI_MAT) \
				[string range $::raw_sectors($vtsnum,VTSI_MAT) 0 0xFF]$menustreamsdefs[string range $::raw_sectors($vtsnum,VTSI_MAT) 0x200 end]
	}
	set ::raw_sectors($vtsnum,${destvtstype}M_PGCI_UT) \
			$::imported_sectors($sourcevtsnum,${sourcetype}M_PGCI_UT)
	set ::raw_sectors($vtsnum,${destvtstype}M_C_ADT) \
			$::imported_sectors($sourcevtsnum,${sourcetype}M_C_ADT)
	set ::raw_sectors($vtsnum,${destvtstype}M_VOBU_ADMAP) \
			$::imported_sectors($sourcevtsnum,${sourcetype}M_VOBU_ADMAP)

	parse_PGCI_UT ::raw_sectors($vtsnum,${destvtstype}M_PGCI_UT) $vtsnum ::menus
	::main::init_tree_menu
	::main::display_pgc menu $vtsnum 1 1
	::main::dvd_modified true false

	set vob_copied false
	if {$::imported_sectors($sourcevtsnum,${sourcetype}M_VOBU_ADMAP) != ""} {
		set sourcevobfilename "[file rootname $sourceifofile].VOB"
		if {$destvobfilename != $sourcevobfilename} {
			if {[file exists $sourcevobfilename]} {
				set txt "Copy the Menu VOB file ?"
				set r [tk_messageBox -title $utiltitle -icon question -message $txt \
						-parent . -type yesno -default yes]
				if {$r == "yes"} {
					# if there is an old VOB, delete it or move it to backup
					delete_vob_with_confirm "VTS_[format %02i $vtsnum]_0.VOB" false

					show_msg "Copying $sourcetext VOB to $current VOB.  Please wait."
					update idletasks
					after 100 {set ::pause false}
					tkwait variable ::pause
					copyfile $sourcevobfilename $destvobfilename -force "PgcEdit: Import Menu" .
					show_msg ""
					set vob_copied true
				}
			}
		}
	} else {
		# if there is an old VOB, delete it or move it to backup
		delete_vob_with_confirm "VTS_[format %02i $vtsnum]_0.VOB" true
	}

	unset ::imported_sectors ::imported_pgcs ::imported_menus

	# allow user to reload buttons informations
	if {$vob_copied} {
		::readdvd::reload_buttons_infos $vtsnum
	}

	# TODO: remove parental managment masks, and if the VMG_PTL_MAIT table exists,
	# prompt the user to remove it (like in Import VTST titles.)
	if {$::raw_sectors(0,VMG_PTL_MAIT) == ""} {
		set remove_pm 1
	} else {
		set remove_pm -1
	}
	if {$vtsnum == 0} {
		set tablename VMGM_PGCI_UT
	} else {
		set tablename VTSM_PGCI_UT
	}
	for {set lu 1} {$lu <= $::menus($vtsnum,numlus)} {incr lu} {
		set luidx [get_int ::raw_sectors($vtsnum,$tablename) [expr ($lu*8)+4] 4]
		for {set pgc 1} {$pgc <= $::menus($vtsnum,$lu,numpgcs)} {incr pgc} {
			set idx [expr {$luidx+2+($pgc*8)}]
			if {[get_int ::raw_sectors($vtsnum,$tablename) $idx 2] != 0} {
				if {$remove_pm == -1} {
					set txt "Warning:"
					set txt "$txt\nA parental managment ID is defined in at least one PGC of the imported menu."
					set txt "$txt\nIf the menu is imported back in the original DVD, you may probably keep his parental managment if you wish."
					set txt "$txt\nBut if the menu comes from another DVD, or if it has been created by an authoring program, or if you add it"
					set txt "$txt\nto an imported titleset, you should remove it, or there is a risk that the menu will not be playable on some"
					set txt "$txt\nstandalone players."
					set txt "$txt\nIf there is no censored material in the menu, it is better and easier to remove the parental IDs anyway."
					set txt "$txt\n(You may also remove the parental managment of the whole DVD with the \"DVD/Remove Parental Managment\"\nfunction.)"
					set txt "$txt\n\nRemove the parental managment IDs from the imported menu (recommended)?"
					set r [tk_messageBox -title $utiltitle -icon question -message $txt \
							-parent . -type yesno -default yes]
					if {$r == "yes"} {
						set remove_pm 1
					} else {
						set remove_pm 0
					}
				}
				if {$remove_pm} {
					set_int ::raw_sectors($vtsnum,$tablename) $idx 0 2
				}
			}
		}
	}
	return
}

#############################################################################
## Remove the whole Menu
proc remove_menu {} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	set utiltitle "Remove Menu"
	set vtsnum [lindex [split [lindex $::editor_cmds_from 1] ","] 0]
	if {$vtsnum == 0} {
		set current "VMGM"
		set vtstype "VMG"
		set vobfilename [file join $::dvddir "VIDEO_TS.VOB"]
	} else {
		set current "VTSM $vtsnum"
		set vtstype "VTS"
		set vobfilename [file join $::dvddir "VTS_[format %02i $vtsnum]_0.VOB"]
	}

	if {$::menus($vtsnum,numlus)>0} {
		if {!$::config(expert_mode)} {
			set txt "This utility will remove the whole $current Menu."
			set txt "$txt\n"
			set txt "$txt\nAre you sure ?"
			set r [tk_messageBox -title $utiltitle -icon warning -message $txt \
					-parent . -type yesno -default no]
			if {$r == "no"} return
		}
	} else {
		show_warning -title $utiltitle -icon error -parent . \
				-message "There is no menu for the $current!"
		return
	}

	set ::raw_sectors($vtsnum,${vtstype}M_PGCI_UT) ""
	set ::raw_sectors($vtsnum,${vtstype}M_C_ADT) ""
	set ::raw_sectors($vtsnum,${vtstype}M_VOBU_ADMAP) ""
	for {set i 1} {$i <= $::menus($vtsnum,numlus)} {incr i} {
		set ::menus($vtsnum,$i,numpgcs) 0
	}
	set ::menus($vtsnum,numlus) 0

	foreach a [array names ::buttons::butsinfo menu,$vtsnum,*] {
		unset ::buttons::butsinfo($a)
	}
	set ::buttons::butsinfo(menu,$vtsnum,errorcode) 1
	set ::buttons::butsinfo(menu,$vtsnum,errormsg) "Can't open menu VOB file."
	set ::buttons::butsinfo(menu,$vtsnum,numvobids) 0

	::main::init_tree_menu
	::main::dvd_modified true false
	if {[lindex $::editor_cmds_from 0] == "menu"} {
		::main::display_pgc pgc 0 0
	}

	# if there is an old VOB, delete it or move it to backup
	delete_vob_with_confirm [file tail $vobfilename] true
}

#############################################################################
## Change the current menu type (Title, Root, Audio...) of the current PGC.
#############################################################################
## Change the current menu type (Title, Root, Audio...) of the current PGC.
proc set_menu_type {vtsnum lunum pgcnum type} {
	set numpgcs $::menus($vtsnum,$lunum,numpgcs)
	set oldmask $::menus($vtsnum,$lunum,$pgcnum,mask)

	switch $type {
		0 {set masktext ""}
		2 {set masktext "TitleM"}
		3 {set masktext "RootM"}
		4 {set masktext "SubPicM"}
		5 {set masktext "AudioM"}
		6 {set masktext "AngleM"}
		7 {set masktext "ChapterM"}
	}
	set newmask [expr {($oldmask & 0XF0) + $type}]
	set ::menus($vtsnum,$lunum,$pgcnum,mask) $newmask
	set ::menus($vtsnum,$lunum,$pgcnum,masktext) $masktext

	if {$type > 0} {
		for {set pgc 1} {$pgc <= $::menus($vtsnum,$lunum,numpgcs)} {incr pgc} {
			if {$pgc == $pgcnum} continue
			if {[expr {$::menus($vtsnum,$lunum,$pgc,mask) & 0X0F}] == $type} {
				set ::menus($vtsnum,$lunum,$pgc,mask) \
						[expr {$::menus($vtsnum,$lunum,$pgc,mask) & 0XF0}]
				set ::menus($vtsnum,$lunum,$pgc,masktext) ""
			}
		}
	}
	::main::init_tree_menu
	::main::dvd_modified true false

	return
}
proc set_menu_type_GUI {} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	set utiltitle "Menu type"
	set pgctype [lindex $::editor_cmds_from 0]
	set vtsnum [lindex [split [lindex $::editor_cmds_from 1] ","] 0]
	if {$pgctype != "menu"} {
		bell
		return
	}
	set lunum  [lindex [split [lindex $::editor_cmds_from 1] ","] 1]
	set pgcnum [lindex [split [lindex $::editor_cmds_from 1] ","] 2]
	set oldmask $::menus($vtsnum,$lunum,$pgcnum,mask)
	set ::utils::menutype_type [expr {$oldmask & 0X0F}]

	if {[winfo exists .menutype]} {destroy .menutype}
	set w .menutype
	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w $utiltitle
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm protocol $w WM_DELETE_WINDOW {set ::utils::menutype_done 0}
	bind $w <Escape> "set ::utils::menutype_done 0"
	bind $w <Return> "set ::utils::menutype_done 1"

	label $w.lab -text "Menu type for PGC\n[$::widget(main_cmd_lab) cget -text]"
	pack  $w.lab
	if {$vtsnum == 0} {
		radiobutton $w.b1 -text "Title menu"   -variable ::utils::menutype_type -value 2 -anchor w
		pack $w.b1 -fill x
	} else {
		radiobutton $w.b1 -text "Root menu"    -variable ::utils::menutype_type -value 3 -anchor w
		pack $w.b1 -fill x
		radiobutton $w.b2 -text "SubPic menu"  -variable ::utils::menutype_type -value 4 -anchor w
		pack $w.b2 -fill x
		radiobutton $w.b3 -text "Audio menu"   -variable ::utils::menutype_type -value 5 -anchor w
		pack $w.b3 -fill x
		radiobutton $w.b4 -text "Angle menu"   -variable ::utils::menutype_type -value 6 -anchor w
		pack $w.b4 -fill x
		radiobutton $w.b5 -text "Chapter menu" -variable ::utils::menutype_type -value 7 -anchor w
		pack $w.b5 -fill x
	}
	radiobutton $w.b0 -text "None" -variable ::utils::menutype_type -value 0 -anchor w
	pack $w.b0 -fill x

	# buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -anchor e
	button $w.fb.ok -text "OK" -width 10 -command {set ::utils::menutype_done 1}
	pack   $w.fb.ok -side left -padx 2
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::utils::menutype_done 0}
	pack   $w.fb.cancel -side left -padx 2

	update
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm maxsize $w [winfo reqwidth $w] [winfo reqheight $w]
	set wx [expr {[winfo rootx .]+(([winfo width  .]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm deiconify $w
	focus $w
	grab set $w

	tkwait variable ::utils::menutype_done
	grab release $w
	destroy $w

	if {$::utils::menutype_done} {
		set first $lunum
		set last  $lunum
		if {$::menus($vtsnum,numlus) > 1} {
			set txt "There are several LUs in this VTSM."
			set txt "$txt\n\nDo you want to modify the PGC type\nin each LU? (recommended)"
			set r [tk_messageBox -title $utiltitle -icon question -message $txt \
					-parent . -type yesno -default yes]
			if {$r == "yes"} {
				set first 1
				set last  $::menus($vtsnum,numlus)
			}
		}

		if {[winfo exists .mf.f2.selector.a.lb]} {
			set selectoroldpos [lindex [.mf.f2.selector.a.lb yview] 0]
		}

		for {set lu $first} {$lu <= $last} {incr lu} {
			if {[catch {set_menu_type $vtsnum $lu $pgcnum $::utils::menutype_type}]} {
				tk_messageBox -title $utiltitle -icon error -parent . -type ok \
						-message "ERROR: Can't find the PGC $pgcnum in LU $lu!"
			}
		}

		::main::display_pgc menu $vtsnum $lunum $pgcnum
		if {[winfo exists .mf.f2.selector.a.lb]} {
			.mf.f2.selector.a.lb yview moveto $selectoroldpos
		}
	}
	return
}

#############################################################################
## Swap two LUs
proc swap_lus {} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	if {[lindex $::editor_cmds_from 0] != "menu"} {
		bell
		return
	}
	set vtsnum [lindex [split [lindex $::editor_cmds_from 1] ","] 0]
	if {$::menus($vtsnum,numlus) < 2} {
		bell
		return
	}

	set ::utils::swaplus_1 [lindex [split [lindex $::editor_cmds_from 1] ","] 1]
	if {$::utils::swaplus_1 == 1} {
		set ::utils::swaplus_2 2
	} else {
		set ::utils::swaplus_2 1
	}

	set w .swaplus
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w "Swap Menu Language Units"
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm protocol $w WM_DELETE_WINDOW {set ::utils::swaplus_done 0}
	bind $w <Escape> "set ::utils::swaplus_done 0"
	bind $w <Return> "set ::utils::swaplus_done 1"

	label $w.lab -text "The first LU is the default on players\nwithout default language options."
	pack  $w.lab

	frame $w.f
	pack  $w.f -fill x -expand 1 -padx 2
	frame $w.f.f1 -bd 2 -relief groove
	pack  $w.f.f1 -side left -fill x -expand 1 -padx 2
	frame $w.f.f2 -bd 2 -relief groove
	pack  $w.f.f2 -side left -fill x -expand 1 -padx 2
	for {set lu 1} {$lu <= $::menus($vtsnum,numlus)} {incr lu} {
		radiobutton $w.f.f1.b$lu -text "LU $lu ($::menus($vtsnum,$lu,lang))" \
				-variable ::utils::swaplus_1 -value $lu -anchor w
		pack $w.f.f1.b$lu -fill x -expand 1
		radiobutton $w.f.f2.b$lu -text "LU $lu ($::menus($vtsnum,$lu,lang))" \
				-variable ::utils::swaplus_2 -value $lu -anchor w
		pack $w.f.f2.b$lu -fill x -expand 1
	}

	# buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -anchor e
	button $w.fb.ok -text "OK" -width 10 -command {set ::utils::swaplus_done 1}
	pack   $w.fb.ok -side left -padx 2
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::utils::swaplus_done 0}
	pack   $w.fb.cancel -side left -padx 2

	update
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm maxsize $w [winfo reqwidth $w] [winfo reqheight $w]
	set wx [expr {[winfo rootx .]+(([winfo width  .]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm deiconify $w
	focus $w
	grab set $w

	set done false
	while {!$done} {
		set ::utils::swaplus_done -1
		tkwait variable ::utils::swaplus_done
		if {!$::utils::swaplus_done} {
			grab release $w
			destroy $w
			return
		}
		if {$::utils::swaplus_1 == $::utils::swaplus_2} {
			bell
		} else {
			set done true
		}
	}
	grab release $w
	destroy $w

	set lu_1 $::utils::swaplus_1
	set lu_2 $::utils::swaplus_2

	show_msg "Swapping LU $lu_1 ($::menus($vtsnum,$lu_1,lang)) with LU $lu_2 ($::menus($vtsnum,$lu_2,lang))..."

	if {$vtsnum == 0} {
		set table "VMGM_PGCI_UT"
	} else {
		set table "VTSM_PGCI_UT"
	}
	set idx1 [expr {$lu_1 * 8}]
	set idx2 [expr {$lu_2 * 8}]
	set luheader_1 [string range $::raw_sectors($vtsnum,$table) $idx1 [expr {$idx1+7}]]
	set luheader_2 [string range $::raw_sectors($vtsnum,$table) $idx2 [expr {$idx2+7}]]
	set ::raw_sectors($vtsnum,$table) [string replace $::raw_sectors($vtsnum,$table) \
			$idx1 [expr {$idx1+7}] $luheader_2]
	set ::raw_sectors($vtsnum,$table) [string replace $::raw_sectors($vtsnum,$table) \
			$idx2 [expr {$idx2+7}] $luheader_1]

	for {set pgc 1} {$pgc <= $::menus($vtsnum,$lu_1,numpgcs)} {incr pgc} {
		set tmp($pgc)				$::menus($vtsnum,$lu_1,$pgc)
		set tmp($pgc,mask)			$::menus($vtsnum,$lu_1,$pgc,mask)
		set tmp($pgc,masktext)		$::menus($vtsnum,$lu_1,$pgc,masktext)
		set tmp($pgc,header)		$::menus($vtsnum,$lu_1,$pgc,header)
		set tmp($pgc,numprograms)	$::menus($vtsnum,$lu_1,$pgc,numprograms)
		set tmp($pgc,programmap)	$::menus($vtsnum,$lu_1,$pgc,programmap)
		set tmp($pgc,numcells)		$::menus($vtsnum,$lu_1,$pgc,numcells)
		set tmp($pgc,cellpbtable)	$::menus($vtsnum,$lu_1,$pgc,cellpbtable)
		set tmp($pgc,cellpostable)	$::menus($vtsnum,$lu_1,$pgc,cellpostable)
	}
	for {set pgc 1} {$pgc <= $::menus($vtsnum,$lu_2,numpgcs)} {incr pgc} {
		set ::menus($vtsnum,$lu_1,$pgc)					$::menus($vtsnum,$lu_2,$pgc)
		set ::menus($vtsnum,$lu_1,$pgc,mask)			$::menus($vtsnum,$lu_2,$pgc,mask)
		set ::menus($vtsnum,$lu_1,$pgc,masktext)		$::menus($vtsnum,$lu_2,$pgc,masktext)
		set ::menus($vtsnum,$lu_1,$pgc,header)			$::menus($vtsnum,$lu_2,$pgc,header)
		set ::menus($vtsnum,$lu_1,$pgc,numprograms)		$::menus($vtsnum,$lu_2,$pgc,numprograms)
		set ::menus($vtsnum,$lu_1,$pgc,programmap)		$::menus($vtsnum,$lu_2,$pgc,programmap)
		set ::menus($vtsnum,$lu_1,$pgc,numcells)		$::menus($vtsnum,$lu_2,$pgc,numcells)
		set ::menus($vtsnum,$lu_1,$pgc,cellpbtable)		$::menus($vtsnum,$lu_2,$pgc,cellpbtable)
		set ::menus($vtsnum,$lu_1,$pgc,cellpostable)	$::menus($vtsnum,$lu_2,$pgc,cellpostable)
	}
	for {set pgc 1} {$pgc <= $::menus($vtsnum,$lu_1,numpgcs)} {incr pgc} {
		set ::menus($vtsnum,$lu_2,$pgc)					$tmp($pgc)
		set ::menus($vtsnum,$lu_2,$pgc,mask)			$tmp($pgc,mask)
		set ::menus($vtsnum,$lu_2,$pgc,masktext)		$tmp($pgc,masktext)
		set ::menus($vtsnum,$lu_2,$pgc,header)			$tmp($pgc,header)
		set ::menus($vtsnum,$lu_2,$pgc,numprograms)		$tmp($pgc,numprograms)
		set ::menus($vtsnum,$lu_2,$pgc,programmap)		$tmp($pgc,programmap)
		set ::menus($vtsnum,$lu_2,$pgc,numcells)		$tmp($pgc,numcells)
		set ::menus($vtsnum,$lu_2,$pgc,cellpbtable)		$tmp($pgc,cellpbtable)
		set ::menus($vtsnum,$lu_2,$pgc,cellpostable)	$tmp($pgc,cellpostable)
	}

	set tmpvar $::menus($vtsnum,$lu_1,lang)
	set ::menus($vtsnum,$lu_1,lang) $::menus($vtsnum,$lu_2,lang)
	set ::menus($vtsnum,$lu_2,lang) $tmpvar

	set tmpvar $::menus($vtsnum,$lu_1,numpgcs)
	set ::menus($vtsnum,$lu_1,numpgcs) $::menus($vtsnum,$lu_2,numpgcs)
	set ::menus($vtsnum,$lu_2,numpgcs) $tmpvar

	# refresh
	::main::init_tree_menu_no_move
	::main::dvd_modified true false
	show_msg ""
	return
}

#############################################################################
## Copy all commands from the current LU to all other LUs in same domain
proc copy_lu_to_lus {} {
	set utiltitle "Copy current LU PGCs commands to all other LUs"
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	if {[lindex $::editor_cmds_from 0] != "menu"} {
		bell
		return
	}
	set vtsnum [lindex [split [lindex $::editor_cmds_from 1] ","] 0]
	if {$vtsnum == 0} {set domain "VMGM"} else {set domain "VTSM $vtsnum"}

	if {$::menus($vtsnum,numlus) < 2} {
		bell
		return
	}

	set source_lu [lindex [split [lindex $::editor_cmds_from 1] ","] 1]
	set numpgcs $::menus($vtsnum,1,numpgcs)

	if {!$::config(expert_mode)} {
		set txt "This utility will copy all commands of all PGCs"
		set txt "$txt\nof the current LU ($domain, LU $source_lu) to ALL other LUs."
		set txt "$txt\n\nThis allow to edit the commands in only one LU"
		set txt "$txt\nand then update the other LUs automatically."
		set txt "$txt\n\nNote that only the PGC commands are copied,"
		set txt "$txt\nnot the PGC parameters nor the PGC menu type."
		set txt "$txt\n\nContinue?"
		set r [tk_messageBox -icon question -title $utiltitle -parent . \
				-type okcancel -default ok -message $txt]
		if {$r == "cancel"} return
	}
	# sanity checks
	if {!$::menus($vtsnum,lus_are_identical)} {
		for {set lu 2} {$lu <= $::menus($vtsnum,numlus)} {incr lu} {
			if {$::menus($vtsnum,$lu,numpgcs) != $numpgcs} {
				set txt "Error: there are not the same number of PGCs"
				set txt "$txt\nin the different LUs of the current domain"
				set txt "$txt\n($domain)."
				set txt "$txt\n\nSorry!"
				tk_messageBox -icon error -title $utiltitle -parent . \
						-type ok -message $txt
				return
			}
		}
		set txt "Warning:\nWhen reading the DVD, PgcEdit has detected"
		set txt "$txt\nthat there were different commands"
		set txt "$txt\nin the various PGCs of the menu LUs,"
		set txt "$txt\nin the current domain ($domain)."
		set txt "$txt\n\nAre you sure you want to continue?"
		set r [tk_messageBox -icon warning -title $utiltitle -parent . \
				-type okcancel -default cancel -message $txt]
		if {$r == "cancel"} return
	}

	for {set lu 1} {$lu <= $::menus($vtsnum,numlus)} {incr lu} {
		if {$lu != $source_lu} {
			show_msg "Copying LU $source_lu ($::menus($vtsnum,$source_lu,lang)) to LU $lu ($::menus($vtsnum,$lu,lang))..."
			update idletasks
			after 10 {set ::pause false}
			tkwait variable ::pause
			for {set pgc 1} {$pgc <= $::menus($vtsnum,$source_lu,numpgcs)} {incr pgc} {
				set ::menus($vtsnum,$lu,$pgc) $::menus($vtsnum,$source_lu,$pgc)
			}
		}
	}
	set ::menus($vtsnum,lus_are_identical) true
	::main::dvd_modified true false
	show_msg "LU $source_lu ($::menus($vtsnum,$source_lu,lang)) commands copied OK."
	return
}


#############################################################################
## Delete the current LU (and mark cells as unused)
proc delete_lu {} {
	set utiltitle "Delete current LU"
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	if {[lindex $::editor_cmds_from 0] != "menu"} {
		bell
		return
	}
	set vtsnum [lindex [split [lindex $::editor_cmds_from 1] ","] 0]
	if {$vtsnum == 0} {
		set domain "VMGM"
		set table "VMGM_PGCI_UT"
	} else {
		set domain "VTSM $vtsnum"
		set table "VTSM_PGCI_UT"
	}

	if {$::menus($vtsnum,numlus) < 2} {
		show_msg "Cannot remove the last LU of $domain!\nUse Remove Menu."
		bell
		return
	}

	set curlu [lindex [split [lindex $::editor_cmds_from 1] ","] 1]
	set curpgc [lindex [split [lindex $::editor_cmds_from 1] ","] 2]

	if {!$::config(expert_mode)} {
		set txt "This utility will remove the current Language Unit."
		set txt "$txt\n\nAll cells will be unreferenced (unless referenced in"
		set txt "$txt\nanother LU), but the VOB files will not be modified."
		set txt "$txt\nProcess the $domain with FixVTS or VobBlanker if you"
		set txt "$txt\nwant to strip out the unused VOB cells."
		set txt "$txt\n\nContinue?"
		set r [tk_messageBox -icon question -title $utiltitle -parent . \
				-type okcancel -default ok -message $txt]
		if {$r == "cancel"} return
	}

	for {set lu [expr $curlu+1]} {$lu <= $::menus($vtsnum,numlus)} {incr lu} {
		foreach v [array names ::menus "$vtsnum,$lu,*"] {
			set tail [string range $v [string length "$vtsnum,$lu,"] end]
			set ::menus($vtsnum,[expr $lu-1],$tail) $::menus($vtsnum,$lu,$tail)
		}
	}
	set luheader [string range $::raw_sectors($vtsnum,$table) 0 7]
	for {set lu 1} {$lu <= $::menus($vtsnum,numlus)} {incr lu} {
		set idx [expr {$lu * 8}]
		if {$lu != $curlu} {
			set luheader $luheader[string range $::raw_sectors($vtsnum,$table) $idx [expr {$idx+7}]]
		}
	}
	set luheader $luheader\000\000\000\000\000\000\000\000[string range $::raw_sectors($vtsnum,$table) [expr {$idx+8}] end]
	incr ::menus($vtsnum,numlus) -1
	set_int luheader 0 $::menus($vtsnum,numlus) 2
	set ::raw_sectors($vtsnum,$table) $luheader

	::main::init_tree_menu
	set newlu $curlu
	if {$newlu > $::menus($vtsnum,numlus)} {incr newlu -1}
	if {$curpgc > $::menus($vtsnum,$newlu,numpgcs)} {set curpgc $::menus($vtsnum,$newlu,numpgcs)}
	::main::display_pgc menu $vtsnum [expr $newlu] $curpgc

	set ::menus($vtsnum,lus_are_identical) true
	::main::dvd_modified true false
	show_msg "LU $curlu removed OK."
	return
}


#############################################################################
## Create a new PGC in the current domain
proc new_pgc {quiet multiple} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	set utiltitle "Create New PGC"

	set pgctype [lindex $::editor_cmds_from 0]
	set vtsnum [lindex [split [lindex $::editor_cmds_from 1] ","] 0]
	# NTSC or PAL 0 length time
	if {[info exists ::raw_sectors(1,VTSI_MAT)]} {
		if {[expr {[get_int ::raw_sectors(1,VTSI_MAT) 0x200 2] & 0x3000}] == 0} {
			set cellpbtime0 192
		} else {
			set cellpbtime0 64
		}
	} else {
		if {[expr {[get_int ::raw_sectors(0,VMGM_MAT) 0x100 2] & 0x3000}] == 0} {
			set cellpbtime0 192
		} else {
			set cellpbtime0 64
		}
	}

	if {$pgctype == "menu"} {
		set lunum [lindex [split [lindex $::editor_cmds_from 1] ","] 1]
		set numpgcs $::menus($vtsnum,$lunum,numpgcs)
		set pgcnum [expr {$numpgcs + 1}]
	} else {
		if {$vtsnum == 0} {
			set txt "Can't add a new PGC to the First Play PGC!"
			set txt "$txt\nPlease select another PGC."
			show_warning -title $utiltitle -icon error -message $txt \
					-parent .
			return false
		}
		set numpgcs $::pgcs($vtsnum,numpgcs)
		set pgcnum [expr {$numpgcs + 1}]
	}

	if {! $quiet} {
		set txt "This utility will create a new, dummy PGC"
		set txt "$txt\nat the end of the current LU, or one PGC"
		set txt "$txt\nat the end of each LU of the current VTS"
		set txt "$txt\nor at the end of the current Title domain."
		set txt "$txt\n"
		set txt "$txt\nShould I create the following PGC?"
		if {$pgctype == "menu"} {
			if {$vtsnum == 0} {
				set txt "$txt\nVMGM"
			} else {
				set txt "$txt\nVTSM $vtsnum"
			}
			set txt "$txt , LU $lunum , $pgcnum"
			set txt "$txt\n\n(If there are several LUs, you will be prompted"
			set txt "$txt\nlater for multiple PGC creation at once)"
		} else {
			set txt "$txt\nVTST $vtsnum , $pgcnum"
			set txt "$txt\n\nThe dummy PGC will be 'attached' to the current"
			set txt "$txt\nTitle (ie. it will inherit the current Title and"
			set txt "$txt\nTTN numbers.)  The current Title will loose its"
			set txt "$txt\none-sequential title status, and therefore, you"
			set txt "$txt\nwill probably not be able to seek to a specific"
			set txt "$txt\ntime in the title any more!  Are you sure?"
		}
		if {!$::config(expert_mode)} {
			set r [tk_messageBox -title $utiltitle -icon question -message $txt \
					-parent . -type yesno -default yes]
			if {$r == "no"} {return false}
		}
	}

	if {$pgctype == "menu"} {
		set first $lunum
		set last  $lunum
		if {$::menus($vtsnum,numlus) > 1} {
			if {! $quiet} {
				set txt "There are several LUs in this VTSM."
				set txt "$txt\n\nDo you want to create a new PGC in each LU?\n(recommended)"
				set r [tk_messageBox -title $utiltitle -icon question -message $txt \
						-parent . -type yesno -default yes]
			} else {
				if {$multiple} {set r "yes"} else {set r "no"}
			}
			if {$r == "yes"} {
				set first 1
				set last  $::menus($vtsnum,numlus)
			}
		}

		for {set lu $first} {$lu <= $last} {incr lu} {
			set pgcn [expr {$::menus($vtsnum,$lu,numpgcs) + 1}]
			set ::menus($vtsnum,$lu,$pgcn,mask) 0
			set ::menus($vtsnum,$lu,$pgcn,masktext) ""
			set ::menus($vtsnum,$lu,$pgcn) {{* pre} {00 00 00 00 00 00 00 00} {* post} {* cell}}
			set ::menus($vtsnum,$lu,$pgcn,header) [string repeat \000 236]
			set_int ::menus($vtsnum,$lu,$pgcn,header) 4 $cellpbtime0 4
			set ::menus($vtsnum,$lu,$pgcn,numprograms) 0
			set ::menus($vtsnum,$lu,$pgcn,programmap) ""
			set ::menus($vtsnum,$lu,$pgcn,numcells) 0
			set ::menus($vtsnum,$lu,$pgcn,cellpbtable) ""
			set ::menus($vtsnum,$lu,$pgcn,cellpostable) ""
			set ::menus($vtsnum,$lu,$pgcn,numbuttons) 0

			# update V**M_PGCI_UT table
			if {$vtsnum == 0} {
				set tablename "VMGM_PGCI_UT"
			} else {
				set tablename "VTSM_PGCI_UT"
			}
			set luidx [get_int ::raw_sectors($vtsnum,$tablename) [expr {($lu*8) + 4}] 4]
			set idx [expr {($::menus($vtsnum,$lu,numpgcs) * 8) + $luidx + 8}]
			set part1 [string range $::raw_sectors($vtsnum,$tablename) 0 [expr {$idx-1}]]
			set part2 [string range $::raw_sectors($vtsnum,$tablename) $idx end]
			set ::raw_sectors($vtsnum,$tablename) $part1[string repeat \000 8]$part2

			incr ::menus($vtsnum,$lu,numpgcs)
		}
		::main::init_tree_menu_no_move
		::main::display_pgc menu $vtsnum $lunum $pgcnum
	} else {
		set ::pgcs($vtsnum,$pgcnum,mask) 0
		set ::pgcs($vtsnum,$pgcnum,masktext) ""
		set ::pgcs($vtsnum,$pgcnum) {{* pre} {00 00 00 00 00 00 00 00} {* post} {* cell}}
		set ::pgcs($vtsnum,$pgcnum,header) [string repeat \000 236]
		set_int ::pgcs($vtsnum,$pgcnum,header) 4 $cellpbtime0 4
		set ::pgcs($vtsnum,$pgcnum,numprograms) 0
		set ::pgcs($vtsnum,$pgcnum,programmap) ""
		set ::pgcs($vtsnum,$pgcnum,numcells) 0
		set ::pgcs($vtsnum,$pgcnum,cellpbtable) ""
		set ::pgcs($vtsnum,$pgcnum,cellpostable) ""
		set ::pgcs($vtsnum,$pgcnum,pttlist) ""
		set ::pgcs($vtsnum,$pgcnum,numbuttons) 0

		if {[info exists ::pgcs([lindex $::editor_cmds_from 1],TTN)]} {
			set ttnnum $::pgcs([lindex $::editor_cmds_from 1],TTN)
		} else {
			set ttnnum 1
			set ::ttns($vtsnum,1) $pgcnum		;# bug fixed
		}
		set ::pgcs($vtsnum,$pgcnum,TTN) $ttnnum	;# bug fixed

		# update VTS_PGCI table
		set idx [expr {($::pgcs($vtsnum,numpgcs) * 8) + 8}]
		set part1 [string range $::raw_sectors($vtsnum,VTS_PGCI) 0 [expr {$idx-1}]]
		set part2 [string range $::raw_sectors($vtsnum,VTS_PGCI) $idx end]
		set ::raw_sectors($vtsnum,VTS_PGCI) $part1[string repeat \000 8]$part2
		set_int ::raw_sectors($vtsnum,VTS_PGCI) $idx $ttnnum 1

		incr ::pgcs($vtsnum,numpgcs)

		# Add blank Time Map table in last position in VTS_TMAPT
		if {$::raw_sectors($vtsnum,VTS_TMAPTI) != ""} {
			set numentries [get_int ::raw_sectors($vtsnum,VTS_TMAPTI) 0 2]
			set table [string range $::raw_sectors($vtsnum,VTS_TMAPTI) 0 7][string repeat \000 [expr 4*$::pgcs($vtsnum,numpgcs)]]
			for {set i 1} {$i <= $numentries} {incr i} {
				set idx1 [get_int ::raw_sectors($vtsnum,VTS_TMAPTI) [expr 4+(4*$i)] 4]
				set idx2 [expr $idx1+3+([get_int ::raw_sectors($vtsnum,VTS_TMAPTI) [expr $idx1+2] 2]*4)]
				set tmap($i) [string range $::raw_sectors($vtsnum,VTS_TMAPTI) $idx1 $idx2]
			}
			while {$i <= $::pgcs($vtsnum,numpgcs)} {
				set tmap($i) \001\000\000\000
				incr i
			}
			set_int table 0 $::pgcs($vtsnum,numpgcs) 2	;# number of PGCs in time map
			for {set i 1} {$i <= $::pgcs($vtsnum,numpgcs)} {incr i} {
				set_int table [expr 4+(4*$i)] [string length $table] 4
				set table $table$tmap($i)
			}
			set ::raw_sectors($vtsnum,VTS_TMAPTI) $table
			unset table
		}
		
		# Change the title status to non-sequential in VMG_TT_SRPT
		set table $::raw_sectors(0,VMG_TT_SRPT)
		set idx 8
		while {$idx < [string length $table]} {
			if {[get_int table [expr $idx+6] 1] == $vtsnum && \
				[get_int table [expr $idx+7] 1] == $ttnnum} {
				set_int table $idx [expr [get_int table $idx 1] | 0x40] 1
				set ::raw_sectors(0,VMG_TT_SRPT) $table
				break
			}
			incr idx 12
		}

		::main::init_tree_menu_no_move
		::main::display_pgc pgc $vtsnum $pgcnum
	}
	::main::dvd_modified true false
	return true
}

proc clone_pgc {} {
	if {![info exists ::dvddir]} {bell; return}

	set cur_type  [lindex $::editor_cmds_from 0]
	set cur_index [lindex $::editor_cmds_from 1]
	set cur_vts   [lindex [split $cur_index ","] 0]

	if {$cur_type == "pgc" && $cur_vts == 0} {bell; return}

	if {! $::config(expert_mode)} {
		show_msg ""
		set txt "This utility will clone the current PGC.\n\n"
		if {$cur_type == "pgc"} {
			if {$::pgcs($cur_index,numprograms) > 0} {
				set txt "${txt}A new TTN number will be assigned to the new PGC.\n"
				set txt "${txt}Optionally, a new Title will also be assigned.\n"
			} else {
				set txt "${txt}As the current PGC is a dummy PGC, the new PGC\n"
				set txt "${txt}will inherit the current TTN number.\n"
			}
		}
		set txt "${txt}\nContinue?"
		set r [tk_messageBox -title "Clone PGC" -icon question \
				-message $txt -parent . -type okcancel -default ok]
		if {$r == "cancel"} return
	}


	if {$cur_type == "menu"} {
		set cur_lu [lindex [split $cur_index ,] 1]
		set cur_numpgcs $::menus($cur_vts,$cur_lu,numpgcs)
		set cur_pgcnum [lindex [split $cur_index ,] 2]
		set new_pgcnum [expr {$cur_numpgcs + 1}]
		set new_index $cur_vts,$cur_lu,$new_pgcnum

		if {$::menus($cur_vts,numlus) > 1} {
			set txt "There are several LUs in this VTSM."
			set txt "$txt\n\nDo you want to clone the PGC in each LU?\n(recommended)"
			set r [tk_messageBox -title "Clone PGC" -icon question -message $txt \
					-parent . -type yesnocancel -default yes]
			if {$r == "yes"} {
				set first 1
				set last  $::menus($cur_vts,numlus)
			} else {
				set ::menus($cur_vts,lus_are_identical) false
				set first $cur_lu
				set last  $cur_lu
			}
			if {$r == "cancel"} {
				return
			}
		} else {
			set first $cur_lu
			set last  $cur_lu
		}
	} else {
		set cur_numpgcs $::pgcs($cur_vts,numpgcs)
		set cur_pgcnum [lindex [split $cur_index ,] 1]
		set new_pgcnum [expr {$cur_numpgcs + 1}]
		set new_index $cur_vts,$new_pgcnum
	}

	show_msg "Creating New PGC $new_pgcnum..."

	# copy PGC
	if {$cur_type == "menu"} {
		for {set lu $first} {$lu <= $last} {incr lu} {
			if {! [info exists ::menus($cur_vts,$lu,$cur_pgcnum)]} {
				tk_messageBox -title "Clone PGC" -icon warning -parent . -type ok \
						-message "WARNING!\nPGC $cur_pgcnum do not exist in LU $lu.\nIt will not be cloned!"
				set ::menus($cur_vts,lus_are_identical) false
				break
			}
			incr ::menus($cur_vts,$lu,numpgcs)
			set newpgc $::menus($cur_vts,$lu,numpgcs)
	
			set ::menus($cur_vts,$lu,$newpgc) \
					$::menus($cur_vts,$lu,$cur_pgcnum)
			set ::menus($cur_vts,$lu,$newpgc,header) \
					$::menus($cur_vts,$lu,$cur_pgcnum,header)
			set ::menus($cur_vts,$lu,$newpgc,numprograms) \
					$::menus($cur_vts,$lu,$cur_pgcnum,numprograms)
			set ::menus($cur_vts,$lu,$newpgc,numcells) \
					$::menus($cur_vts,$lu,$cur_pgcnum,numcells)
			set ::menus($cur_vts,$lu,$newpgc,cellpbtable) \
					$::menus($cur_vts,$lu,$cur_pgcnum,cellpbtable)
			set ::menus($cur_vts,$lu,$newpgc,cellpostable) \
					$::menus($cur_vts,$lu,$cur_pgcnum,cellpostable)
			set ::menus($cur_vts,$lu,$newpgc,programmap) \
					$::menus($cur_vts,$lu,$cur_pgcnum,programmap)
	
			set ::menus($cur_vts,$lu,$newpgc,mask)			0
			set ::menus($cur_vts,$lu,$newpgc,masktext)		""
			if {[info exists ::menus($cur_vts,$lu,$cur_pgcnum,numbuttons)]} {
				set ::menus($cur_vts,$lu,$newpgc,numbuttons) \
						$::menus($cur_vts,$lu,$cur_pgcnum,numbuttons)
			} else {
				set ::menus($cur_vts,$lu,$newpgc,numbuttons) 0
			}
	
			# update V**M_PGCI_UT table
			if {$cur_vts == 0} {
				set tablename "VMGM_PGCI_UT"
			} else {
				set tablename "VTSM_PGCI_UT"
			}
			set luidx [get_int ::raw_sectors($cur_vts,$tablename) [expr {($cur_lu*8) + 4}] 4]
			set idx [expr {($::menus($cur_vts,$cur_lu,numpgcs) * 8) + $luidx + 8}]
			set origentry [string range $::raw_sectors($cur_vts,$tablename) $idx [expr {$idx+7}]]
			set_int origentry 0 0 1	;# clear meny type
	
			set luidx [get_int ::raw_sectors($cur_vts,$tablename) [expr {($cur_lu*8) + 4}] 4]
			set idx [expr {($::menus($cur_vts,$lu,numpgcs) * 8) + $luidx + 8}]
			set part1 [string range $::raw_sectors($cur_vts,$tablename) 0 [expr {$idx-1}]]
			set part2 [string range $::raw_sectors($cur_vts,$tablename) $idx end]
			set ::raw_sectors($cur_vts,$tablename) $part1$origentry$part2
		}
	} else {
		if {$::pgcs($cur_index,numprograms) > 0} {
			set dummypgc false
			set donewtitle [tk_messageBox -title "Clone PGC" -icon question \
					-message "Create a new Title number?\n(This will add an entry in the VMG_TT_SRPT table)" \
					-parent . -type yesnocancel -default yes]
			if {$donewtitle == "cancel"} {
				show_msg "Creation of New PGC $new_pgcnum cancelled."
				return
			}
		} else {
			set dummypgc true
		}

		set ::pgcs($cur_vts,numpgcs) $new_pgcnum
		set ::pgcs($new_index)				$::pgcs($cur_index)
		set ::pgcs($new_index,header)		$::pgcs($cur_index,header)
		set ::pgcs($new_index,numprograms)	$::pgcs($cur_index,numprograms)
		set ::pgcs($new_index,numcells)		$::pgcs($cur_index,numcells)
		set ::pgcs($new_index,cellpbtable)	$::pgcs($cur_index,cellpbtable)
		set ::pgcs($new_index,cellpostable) $::pgcs($cur_index,cellpostable)
		set ::pgcs($new_index,programmap)	$::pgcs($cur_index,programmap)
		if {[info exists ::pgcs($cur_index,numbuttons)]} {
			set ::pgcs($new_index,numbuttons) $::pgcs($cur_index,numbuttons)
		}

		if {$dummypgc} {
			set ttn $::pgcs($cur_index,TTN)
		} else {
			set ttn 0
			for {set i 1} {$i < $new_pgcnum} {incr i} {
				if {[info exists ::pgcs($cur_vts,$i,TTN)] && $::pgcs($cur_vts,$i,TTN) > $ttn} {
					set ttn $::pgcs($cur_vts,$i,TTN)
				}
			}
			incr ttn
		}
		set ::pgcs($new_index,TTN) $ttn
		set ::ttns($cur_vts,$ttn) $new_pgcnum

		set ::pgcs($new_index,pttlist) ""
		foreach ttu $::pgcs($cur_index,pttlist) {
			lset ttu 2 $ttn
			lappend ::pgcs($new_index,pttlist) $ttu
		}

		# update VTS_PGCI table
		set idx [expr {($new_pgcnum * 8)}]
		set part1 [string range $::raw_sectors($cur_vts,VTS_PGCI) 0 [expr {$idx-1}]]
		set part2 [string range $::raw_sectors($cur_vts,VTS_PGCI) $idx end]
		set ::raw_sectors($cur_vts,VTS_PGCI) $part1[string repeat \000 8]$part2
		set_int ::raw_sectors($cur_vts,VTS_PGCI) $idx [expr {$ttn+128}] 1

		### build chapter table
		if {! $dummypgc} {
			set ptttable $::raw_sectors($cur_vts,VTS_PTT_SRPT)
			set pttnumtitles [get_int ptttable 0 2]
			set pttheader [string range $ptttable 0 [expr {($pttnumtitles*4) + 7}]]
			set pttrest [string range $ptttable [expr {($pttnumtitles*4) + 8}] end]
	
			# get start and end index in the PTT table for the old PTTs.
			set startidx [get_int pttheader [expr {($::pgcs($cur_index,TTN) * 4)+4}] 4]
			if {[expr {($::pgcs($cur_index,TTN) * 4)+8}] >= [string length $pttheader]} {
				set endidx end
			} else {
				set endidx [expr {[get_int pttheader [expr {($::pgcs($cur_index,TTN) * 4)+8}] 4] -1}]
			}
			set newptts [string range $ptttable $startidx $endidx]
	
			# add entry to the PTT table header
			set idx [string length $pttheader]
			set pttheader $pttheader\000\000\000\000
			set_int pttheader $idx [expr {[string length $pttheader]+[string length $pttrest]}] 4
			for {set i 0} {$i < $pttnumtitles} {incr i} {
				set idx [expr {($i * 4)+8}]
				set_int pttheader $idx [expr {[get_int pttheader $idx 4] +4}] 4
			}
			set ptttable $pttheader$pttrest$newptts
			set_int ptttable 0 $::pgcs($new_index,TTN) 2
			set_int ptttable 4 [expr {[string length $ptttable]-1}] 4
			set ::raw_sectors($cur_vts,VTS_PTT_SRPT) $ptttable
	
			if {$donewtitle == "yes"} {
				# Add the new title to the VMG global table of titles
				set data $::raw_sectors(0,VMG_TT_SRPT)
				set srptnumtitles [get_int data 0 2]
				set data [string range $data 0 [expr {($srptnumtitles*12)+7}]]
				incr srptnumtitles
				set_int data 0 $srptnumtitles 2
				set oldtitle ""
				if {[info exists ::pgcs($cur_index,Title)] && $::pgcs($cur_index,Title) != ""} {
					set oldtitle $::pgcs($cur_index,Title)
				}
				if {$oldtitle == ""} {
					for {set p 1} {$p <= $::pgcs($cur_vts,numpgcs)} {incr p} {
						if {$::pgcs($cur_vts,$p,TTN) == $::pgcs($cur_index,TTN) &&
								[info exists ::pgcs($cur_vts,$p,Title)] && $::pgcs($cur_vts,$p,Title) != ""} {
							set oldtitle $::pgcs($cur_vts,$p,Title)
							break
						}
					}
				}
				if {$oldtitle != ""} {
					set newentry [string range $::raw_sectors(0,VMG_TT_SRPT) \
							[expr {($oldtitle*12)-4}] [expr {($oldtitle*12)+8}]]
				} else {
					set newentry \024\001\000\000\000\000\000\000\000\000\000\000
					set_int newentry 6 $cur_vts 1
				}
				set_int newentry 2 [llength $::pgcs($new_index,pttlist)] 2
				set_int newentry 7 $::pgcs($new_index,TTN) 1
				set data $data$newentry
				set_int data 4 [expr {[string length $data]-1}] 4
				set ::raw_sectors(0,VMG_TT_SRPT) $data
				set ::pgcs($new_index,Title) $srptnumtitles
				set ::pgcs(Title,$srptnumtitles) [list $cur_vts $new_pgcnum]
				set ::pgcs(Title,NumTitles) $srptnumtitles
			}
		}

		# Copy PGC's Time Map in last position in VTS_TMAPT
		if {$::raw_sectors($cur_vts,VTS_TMAPTI) != ""} {
			set numentries [get_int ::raw_sectors($cur_vts,VTS_TMAPTI) 0 2]
			set table [string range $::raw_sectors($cur_vts,VTS_TMAPTI) 0 7][string repeat \000 [expr 4*$new_pgcnum]]
			for {set i 1} {$i <= $numentries} {incr i} {
				set idx1 [get_int ::raw_sectors($cur_vts,VTS_TMAPTI) [expr 4+(4*$i)] 4]
				set idx2 [expr $idx1+3+([get_int ::raw_sectors($cur_vts,VTS_TMAPTI) [expr $idx1+2] 2]*4)]
				set tmap($i) [string range $::raw_sectors($cur_vts,VTS_TMAPTI) $idx1 $idx2]
			}
			while {$i <= $cur_numpgcs} {
				set tmap($i) \001\000\000\000
				incr i
			}
			set tmap($new_pgcnum) $tmap($cur_pgcnum)
			set_int table 0 $new_pgcnum 2	;# number of PGCs in time map
			for {set i 1} {$i <= $new_pgcnum} {incr i} {
				set_int table [expr 4+(4*$i)] [string length $table] 4
				set table $table$tmap($i)
			}
			set ::raw_sectors($cur_vts,VTS_TMAPTI) $table
			unset table
		}
	}
	# refresh GUI
	::main::dvd_modified true false
	::main::init_tree_menu

	if {$cur_type == "menu"} {
		::main::display_pgc menu $cur_vts $cur_lu $new_pgcnum
	} else {
		::main::display_pgc pgc $cur_vts $new_pgcnum
	}
	show_msg "PGC cloned OK."
	return
}

# Create new PGC with all cells of the PGCs of the same domain.
proc create_playall_pgc {} {
	if {![info exists ::dvddir]} {bell; return}

	set cur_type  [lindex $::editor_cmds_from 0]
	if {$cur_type != "pgc"} {bell; return}
	set cur_index [lindex $::editor_cmds_from 1]
	set cur_vts   [lindex [split $cur_index ","] 0]
	if {$cur_vts == 0} {bell; return}
	set cur_pgc   [lindex [split $cur_index ","] 1]

	if {! $::config(expert_mode)} {
		show_msg ""
		set txt "This utility will create a new Title (Title [expr {[get_int ::raw_sectors(0,VMG_TT_SRPT) 0 2]+1}]) with all the\n"
		set txt "${txt}VOB/Cell IDs of the current VTS (VTS $cur_vts).\n\n"
		set txt "${txt}The streams status and the color lookup table are\n"
		set txt "${txt}copied from the current PGC (PGC $cur_pgc).\n\n"
		set txt "${txt}A chapter will be created for each VOB/Cell ID.\n\n"
		set txt "${txt}This new Title will have the pre-commands of the\n"
		set txt "${txt}first PGC and the post-commands of the last PGC\n"
		set txt "${txt}of the current title domain.\n\n"
		set txt "${txt}Continue?"
		set r [tk_messageBox -title "New 'Play All' Title" -icon question \
				-message $txt -parent . -type okcancel -default ok]
		if {$r == "cancel"} return
	}

	show_msg "Creating New PGC $::pgcs($cur_vts,numpgcs)..."

	# Search for all VOB IDs in the current VTS, and for all Cell IDs in each VOB IDs
	# and build the list of vobids, and the lists of cellids for all vobids.
	# Also, for each pair VOB/Cell ID, keep track of the belonging PGC's cell.
	set totcells 0
	set all_vobids ""
	for {set pgc 1} {$pgc <= $::pgcs($cur_vts,numpgcs)} {incr pgc} {
		set cellpostable $::pgcs($cur_vts,$pgc,cellpostable)
		for {set cell 1} {$cell <= $::pgcs($cur_vts,$pgc,numcells)} {incr cell} {
			incr totcells
			set cellidx [expr {($cell - 1) * 4}]
			set vobid  [get_int cellpostable $cellidx 2]
			set cellid [get_int cellpostable [expr {$cellidx + 3}] 1]
			if {[lsearch $all_vobids $vobid] == -1} {
				lappend all_vobids $vobid
				set all_cellids($vobid) ""
			}
			if {[lsearch $all_cellids($vobid) $cellid] == -1} {
				lappend all_cellids($vobid) $cellid
			}
			set src_pgc_cell($vobid,$cellid) [list $pgc $cell]
		}
	}
	if {$totcells > 255} {
		set txt "You have $totcells cells in this VTS.\n"
		set txt "${txt}This is too much for one PGC: the DVD\n"
		set txt "${txt}standard limit the number of cells per\n"
		set txt "${txt}PGC to 255.\n\n"
		set txt "${txt}Do you want ot create a title with only\n"
		set txt "${txt}the first 255 cells?"
		set r [tk_messageBox -title "New 'Play All' Title" -icon error \
				-message $txt -parent . -type okcancel -default cancel]
		if {$r == "cancel"} return
	}
	set all_vobids [lsort -integer $all_vobids]
	foreach vobid $all_vobids {
		set all_cellids($vobid) [lsort -integer $all_cellids($vobid)]
	}

	# Begin the job!
	set numcells    0
	set header $::pgcs($cur_vts,[lindex [split $cur_index ","] 1],header)
	set programmap ""
	set cellpbtable ""
	set cellpostable ""
	set old_vobid 0
	set old_cellid 0
	set totalpbtime "\000\000\000\000"
	set numbuttons 0
	foreach vobid $all_vobids {
		if {$numcells == 256} break
		foreach cellid $all_cellids($vobid) {
			incr numcells
			if {$numcells == 256} break
			set thispgc  [lindex $src_pgc_cell($vobid,$cellid) 0]
			set thiscell [lindex $src_pgc_cell($vobid,$cellid) 1]
			set thiscellpbtable $::pgcs($cur_vts,$thispgc,cellpbtable)
			set idx [expr {($thiscell-1) * 0x18}]
			set thiscellpbentry [string range $thiscellpbtable $idx [expr {$idx+0x17}]]
			# fix playback mode
			if {$cellpostable == ""} {
				set mode 2
			} else {
				if {$old_vobid == $vobid && [expr {$old_cellid+1}] == $cellid} {
					set mode 8
				} else {
					set mode 10
				}
			}
			set oldmode [expr {[get_int thiscellpbentry 0 1] & 0xF5}]
			set_int thiscellpbentry 0 [expr {$oldmode | $mode}] 1
			# set cell still time to a max of 3 seconds
			if {[get_int thiscellpbentry 2 1] > 3} {
				set_int thiscellpbentry 2 3 1
			}
			# clear cell command
			set_int thiscellpbentry 3 0 1

			# Add cell time to total PGC time, but only if angle 1.
			set type [get_int thiscellpbtable [expr {($thiscell - 1) * 24}] 1]
			if {[expr {$type & 0x10}] == 0 || [expr {$type & 0xC0}] == 0x40} {
				set totalpbtime [::pgced::AddBCDtime [string range $thiscellpbentry 4 7] $totalpbtime]
			}

			# add this cell infos to the new cell playback table
			set cellpbtable $cellpbtable$thiscellpbentry

			# add this cell position to the new cell position table
			set idx [string length $cellpostable]
			set cellpostable $cellpostable\000\000\000\000
			set_int cellpostable $idx $vobid 2
			incr idx 3
			set_int cellpostable $idx $cellid 1

			# create a new program for each cell
			set tmp \000
			set_int tmp 0 $numcells 1
			set programmap $programmap$tmp

			# compute number of buttons
			if {[info exists ::buttons::butsinfo(pgc,$cur_vts,scanned)] && \
					$::buttons::butsinfo(pgc,$cur_vts,scanned) == 1} {
				incr numbuttons $::buttons::butsinfo(pgc,$cur_vts,$vobid,$cellid,numbuts)
			}

			set old_cellid $cellid
			set old_vobid $vobid
		}
	}
	if {[expr {[string length $programmap] % 2}] == 1} {
		set programmap $programmap\000
	}
	if {$numcells > 255} {set numcells 255}

	set newpgcn [expr {$::pgcs($cur_vts,numpgcs)+1}]

	# fix PGC header attributes
	set_int header 2 $numcells 1	;# number of programs = total number of cells
	set_int header 3 $numcells 1	;# number of cells = total number of cells
	set_int header 4 [get_int totalpbtime 0 4] 4	;# total Playback time
	set_int header 8 0 4	;# No PUOs
	set_int header 0x9C $newpgcn 2	;# NextPGCN
	set_int header 0x9E $newpgcn 2	;# PrevPGCN
	set_int header 0xA0 0 2	;# GoUpPGCN
	set_int header 0xA3 0 1	;# PGC Still Time = 0
	set_int header 0xA2 0 1	;# PB Mode = sequential

	# take track of the new changes
	set precmds  [get_type_cmds pre  ::pgcs($cur_vts,1)]
	set postcmds [get_type_cmds post ::pgcs($cur_vts,$::pgcs($cur_vts,numpgcs))]
	incr ::pgcs($cur_vts,numpgcs)
	set ::pgcs($cur_vts,$newpgcn) \
			[concat {{* pre}} $precmds {{* post}} $postcmds {{* cell}}]
	set ::pgcs($cur_vts,$newpgcn,header) $header
	set ::pgcs($cur_vts,$newpgcn,numprograms) $numcells
	set ::pgcs($cur_vts,$newpgcn,numcells) $numcells
	set ::pgcs($cur_vts,$newpgcn,cellpbtable) $cellpbtable
	set ::pgcs($cur_vts,$newpgcn,cellpostable) $cellpostable
	set ::pgcs($cur_vts,$newpgcn,programmap) $programmap
	set ttn 0
	for {set i 1} {$i < $newpgcn} {incr i} {
		if {[info exists ::pgcs($cur_vts,$i,TTN)] && $::pgcs($cur_vts,$i,TTN) > $ttn} {
			set ttn $::pgcs($cur_vts,$i,TTN)
		}
	}
	incr ttn
	set ::pgcs($cur_vts,$newpgcn,TTN) $ttn
	set ::ttns($cur_vts,$ttn) $newpgcn
	if {[info exists ::buttons::butsinfo(pgc,$cur_vts,scanned)] && \
			$::buttons::butsinfo(pgc,$cur_vts,scanned) == 1} {
		set ::pgcs($cur_vts,$newpgcn,numbuttons) $numbuttons
	}
	
	# update VTS_PGCI table
	set idx [expr {($newpgcn * 8)}]
	set part1 [string range $::raw_sectors($cur_vts,VTS_PGCI) 0 [expr {$idx-1}]]
	set part2 [string range $::raw_sectors($cur_vts,VTS_PGCI) $idx end]
	set ::raw_sectors($cur_vts,VTS_PGCI) $part1[string repeat \000 8]$part2
	set_int ::raw_sectors($cur_vts,VTS_PGCI) $idx [expr {$ttn+128}] 1

	### build chapter table
	set ptttable $::raw_sectors($cur_vts,VTS_PTT_SRPT)
	set pttnumtitles [get_int ptttable 0 2]
	set newtitle [expr {$pttnumtitles+1}]
	set pttheader [string range $ptttable 0 [expr {($pttnumtitles*4) + 7}]]
	set pttrest [string range $ptttable [expr {($pttnumtitles*4) + 8}] end]
	# add entry to the PTT table header
	set idx [string length $pttheader]
	set pttheader $pttheader\000\000\000\000
	set_int pttheader $idx [expr {[string length $pttheader]+[string length $pttrest]}] 4
	for {set i 0} {$i < $pttnumtitles} {incr i} {
		set idx [expr {($i * 4)+8}]
		set_int pttheader $idx [expr {[get_int pttheader $idx 4] +4}] 4
	}
	# add a chapter for each cell
	set pttlist ""
	for {set ptt 1} {$ptt <= $numcells} {incr ptt} {
		set tmp \000\000\000\000
		set_int tmp 0 $newpgcn 2
		set_int tmp 2 $ptt 2
		set pttrest $pttrest$tmp
		lappend pttlist "$ptt $ptt $newtitle"
	}
	set ::pgcs($cur_vts,$newpgcn,pttlist) $pttlist
	set ::pgcs($cur_vts,numttus) $newtitle
	set ptttable $pttheader$pttrest
	set_int ptttable 0 [expr {$pttnumtitles+1}] 2
	set_int ptttable 4 [expr {[string length $ptttable]-1}] 4
	set ::raw_sectors($cur_vts,VTS_PTT_SRPT) $ptttable

	# Add the new title to the VMG global table of titles
	set data $::raw_sectors(0,VMG_TT_SRPT)
	set srptnumtitles [get_int data 0 2]
	set data [string range $data 0 [expr {($srptnumtitles*12)+7}]]
	incr srptnumtitles
	set_int data 0 $srptnumtitles 2
	set newentry \024\001\000\000\000\000\000\000\000\000\000\000
	set_int newentry 2 $numcells 2
	set_int newentry 6 $cur_vts 1
	set_int newentry 7 $::pgcs($cur_vts,$newpgcn,TTN) 1
	set data $data$newentry
	set_int data 4 [expr {[string length $data]-1}] 4
	set ::raw_sectors(0,VMG_TT_SRPT) $data
	set ::pgcs($cur_vts,$newpgcn,Title) $srptnumtitles
	set ::pgcs(Title,$srptnumtitles) [list $cur_vts $newpgcn]
	set ::pgcs(Title,NumTitles) $srptnumtitles

	# refresh GUI
	::main::dvd_modified true false
	::main::init_tree_menu
	::main::display_pgc pgc $cur_vts $newpgcn
	::tmap::need_new_tmap $cur_vts $newpgcn

	if {$numcells > 99} {
		set msg "New PGC $newpgcn created OK, with $numcells cells!"
		if {$numcells > 127} {
			set msg "$msg\n\nThere are more than 127 cells in the new PGC!"
			set msg "$msg\nThe maximum number of programs is 127, and the maximum"
			set msg "$msg\nnumber of chapters is 99."
			set msg "$msg\n\nYou MUST manually remove some programs and chapters!"
			set msg "$msg\n\nYou should remove some cells with the \"Remove Cells\" button"
			set msg "$msg\nin the PGC Editor, or use the \"Prog.\" buttons in the cells list"
			set msg "$msg\nand the \"Edit chapter (PTT) table\" button to remove some"
			set msg "$msg\nprograms and chapters, or the DVD will hang on many players!"
		} else {
			set msg "$msg\n\nThere are more than 99 cells in the new PGC!"
			set msg "$msg\nThe maximum number of chapters is 99."
			set msg "$msg\n\nYou MUST manually remove some programs or chapters!"
			set msg "$msg\n\nUse the \"Prog.\" buttons in the cells list to reorganize"
			set msg "$msg\nthe programs, and the \"Edit chapter (PTT) table\" button"
			set msg "$msg\nin the PGC Editor to remove the surplus chapters, or the"
			set msg "$msg\nDVD will hang on many players!"
		}
		tk_messageBox -message $msg -title "New 'Play All' Title" -icon warning
		::main::action_Edit_PGC
	} else {
		show_msg "New PGC $newpgcn created OK, with $numcells cells."
	}
	return
}


proc swap_or_copy_pgc_from {} {
	if {![info exists ::dvddir]} {bell; return}
	if {[lindex $::editor_cmds_from 0] == "pgc"} {bell;return}
	set cur_index [lindex $::editor_cmds_from 1]
	set cur_vts   [lindex [split $cur_index ","] 0]
	set cur_lu    [lindex [split $cur_index ","] 1]
	set cur_pgc   [lindex [split $cur_index ","] 2]
	if {$::menus($cur_vts,numlus) == 1 && $::menus($cur_vts,$cur_lu,numpgcs) <= 1} {bell;return}

	if {! [info exists ::utils::swap_pgc_mode]} {
		set ::utils::swap_pgc_mode 1
		set ::utils::copy_pgc_copy_commands 1
		set ::utils::copy_pgc_copy_header 1
		set ::utils::copy_pgc_src_lu 0
		set ::utils::copy_pgc_src_pgc 0
		set ::utils::copy_pgc_src "..."
	}

	if {[winfo exists .copypgc]} {destroy .copypgc}
	set w .copypgc
	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w "Swap/Copy PGCs"
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm protocol $w WM_DELETE_WINDOW {set ::utils::copypgc_done 0}
	bind $w <Escape> "set ::utils::copypgc_done 0"
	bind $w <Return> "set ::utils::copypgc_done 1"

	set txt "This utility will swap two PGCs, or copy a PGC over the current PGC."
	set txt "$txt\n\nWARNING: The references to the PGCs will NOT be modified!"
	set txt "$txt\nIn Copy mode, the original cells will be lost (unless reused elsewhere) !"
	set txt "$txt\nBe sure you know what you are doing!"
	label $w.warn -text $txt -anchor w -justify left
	pack  $w.warn -padx 2 -pady 4 -fill x -anchor w

	frame $w.f -bd 2 -relief groove
	pack  $w.f -padx 4 -pady 3 -fill x

	radiobutton $w.f.sp -text "Swap source and current PGCs" -variable ::utils::swap_pgc_mode -value 1 -anchor w
	pack  $w.f.sp -fill x
	radiobutton $w.f.cp -text "Copy source PGC over current PGC" -variable ::utils::swap_pgc_mode -value 0 -anchor w
	pack  $w.f.cp -fill x

	label $w.f.lab -text "Source PGC:"
	pack  $w.f.lab -anchor w -padx 2

	menubutton $w.f.source -textvariable ::utils::copy_pgc_src \
			-width 46 -menu $w.f.source.mnu -anchor w
	pack $w.f.source -fill x -padx 4

	menu $w.f.source.mnu -tearoff 0

	checkbutton $w.f.ch -text "Swap/Copy PGC parameters (streams, color table, cell table...)" \
			-variable ::utils::copy_pgc_copy_header -anchor w
	pack  $w.f.ch -fill x
	checkbutton $w.f.cc -text "Swap/Copy PGC VM commands (pre, post and cell commands)" \
			-variable ::utils::copy_pgc_copy_commands -anchor w
	pack  $w.f.cc -fill x

	# buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -anchor e
	button $w.fb.ok -text "OK" -width 10 -command {set ::utils::copypgc_done 1} -state disabled
	pack   $w.fb.ok -side left -padx 2
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::utils::copypgc_done 0}
	pack   $w.fb.cancel -side left -padx 2

	update
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm maxsize $w [winfo reqwidth $w] [winfo reqheight $w]
	set wx [expr {[winfo rootx .]+(([winfo width  .]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm deiconify $w
	focus $w
	grab set $w

	set radioindex 1
	for {set lu 1} {$lu <= $::menus($cur_vts,numlus)} {incr lu} {
		if {$::menus($cur_vts,numlus) > 1} {
			$w.f.source.mnu add cascade -label "LU $lu ($::menus($cur_vts,$lu,lang))" \
					-menu $w.f.source.mnu.lu$lu
			menu $w.f.source.mnu.lu$lu -tearoff 0
			set m $w.f.source.mnu.lu$lu
		} else {
			set m $w.f.source.mnu
		}
		for {set pgc 1} {$pgc <= $::menus($cur_vts,$lu,numpgcs)} {incr pgc} {
			set state normal
			if {$lu == $cur_lu && $pgc == $cur_pgc} {set state disabled}
			set label [::main::pgc_label $::config(global_font) menu $cur_vts $lu $pgc]
			$m add radiobutton -variable ::utils::copy_pgc_src_item \
					-value $radioindex -label $label -state $state -command \
					"set ::utils::copy_pgc_src \"     $label\"; set ::utils::copy_pgc_src_lu $lu; set ::utils::copy_pgc_src_pgc $pgc; $w.fb.ok configure -state normal"
			if {$radioindex == $::utils::copy_pgc_src_item} {
				if {$state == "disabled"} {
					set ::utils::copy_pgc_src "..."
					#set ::utils::copy_pgc_src_item 0
				} else {
					set ::utils::copy_pgc_src $label
					$w.fb.ok configure -state normal
				}
			}
			incr radioindex
		}
	}

	tkwait variable ::utils::copypgc_done
	destroy $w
	if {! $::utils::copypgc_done} {
		return
	}

	set src_lu  $::utils::copy_pgc_src_lu
	set src_pgc $::utils::copy_pgc_src_pgc
	set src_index $cur_vts,$src_lu,$src_pgc

	# copy PGC
	if {$::utils::swap_pgc_mode} {
		set tmp(commands)     $::menus($cur_vts,$cur_lu,$cur_pgc)
		set tmp(header)       $::menus($cur_vts,$cur_lu,$cur_pgc,header)
		set tmp(numprograms)  $::menus($cur_vts,$cur_lu,$cur_pgc,numprograms)
		set tmp(numcells)     $::menus($cur_vts,$cur_lu,$cur_pgc,numcells)
		set tmp(cellpbtable)  $::menus($cur_vts,$cur_lu,$cur_pgc,cellpbtable)
		set tmp(cellpostable) $::menus($cur_vts,$cur_lu,$cur_pgc,cellpostable)
		set tmp(programmap)   $::menus($cur_vts,$cur_lu,$cur_pgc,programmap)
		set tmp(numbuttons)   $::menus($cur_vts,$cur_lu,$cur_pgc,numbuttons)
	}

	if {$::utils::copy_pgc_copy_commands} {
		set ::menus($cur_vts,$cur_lu,$cur_pgc) $::menus($cur_vts,$src_lu,$src_pgc)
	}
	if {$::utils::copy_pgc_copy_header} {
		set ::menus($cur_vts,$cur_lu,$cur_pgc,header)       $::menus($cur_vts,$src_lu,$src_pgc,header)
		set ::menus($cur_vts,$cur_lu,$cur_pgc,numprograms)  $::menus($cur_vts,$src_lu,$src_pgc,numprograms)
		set ::menus($cur_vts,$cur_lu,$cur_pgc,numcells)     $::menus($cur_vts,$src_lu,$src_pgc,numcells)
		set ::menus($cur_vts,$cur_lu,$cur_pgc,cellpbtable)  $::menus($cur_vts,$src_lu,$src_pgc,cellpbtable)
		set ::menus($cur_vts,$cur_lu,$cur_pgc,cellpostable) $::menus($cur_vts,$src_lu,$src_pgc,cellpostable)
		set ::menus($cur_vts,$cur_lu,$cur_pgc,programmap)   $::menus($cur_vts,$src_lu,$src_pgc,programmap)
		set ::menus($cur_vts,$cur_lu,$cur_pgc,numbuttons)   $::menus($cur_vts,$src_lu,$src_pgc,numbuttons)
	}

	if {$::utils::swap_pgc_mode} {
		if {$::utils::copy_pgc_copy_commands} {
			set ::menus($cur_vts,$src_lu,$src_pgc)     			$tmp(commands)
		}
		if {$::utils::copy_pgc_copy_header} {
			set ::menus($cur_vts,$src_lu,$src_pgc,header)       $tmp(header)
			set ::menus($cur_vts,$src_lu,$src_pgc,numprograms)  $tmp(numprograms)
			set ::menus($cur_vts,$src_lu,$src_pgc,numcells)     $tmp(numcells)
			set ::menus($cur_vts,$src_lu,$src_pgc,cellpbtable)  $tmp(cellpbtable)
			set ::menus($cur_vts,$src_lu,$src_pgc,cellpostable) $tmp(cellpostable)
			set ::menus($cur_vts,$src_lu,$src_pgc,programmap)   $tmp(programmap)
			set ::menus($cur_vts,$src_lu,$src_pgc,numbuttons)   $tmp(numbuttons)
		}
	}

	# refresh GUI
	::main::dvd_modified true false
	::main::init_tree_menu_no_move
	::main::display_pgc menu $cur_vts $cur_lu $cur_pgc
	return
}

#############################################################################
## Procedure:  new_DVD
## Create a new DVD wuth an empty VMG
proc {new_DVD} {} {
	set utiltitle "New DVD"

	if {!$::config(expert_mode)} {
		set txt "This utility will create a new VIDEO_TS.IFO file"
		set txt "$txt\nin the specified folder."
		set txt "$txt\nThe new DVD will have no VTST Titles, nor menus."
		set txt "$txt\nYou must therefore import titles to the new DVD,"
		set txt "$txt\nwith 'Import VTST Titles', and modify the PGCs"
		set txt "$txt\ncommands to suit your needs."
		set txt "$txt\n\nContinue?"
		set r [tk_messageBox -title $utiltitle -icon question -message $txt \
				-parent . -type yesno -default yes]
		if {$r == "no"} {return -1}
	}

	if {$::dvd_modified} {
		set r [tk_messageBox -message "The DVD has been modified.\n\nSave it before?" \
				-icon question -title "PgcEdit" -type yesnocancel -default yes]
		if {$r == "cancel"} {return 0}
		if {$r == "yes"} {
			set rc [::main::Write_DVD false]
			if {$rc != 0} {return 0}
		}
	}

	set destdir [file nativename [file normalize \
			[tk_chooseNewDirectory -initialdir $::config(initialdir) \
			-title "$utiltitle: Select or create the EMPTY target folder..."]]]
	if {$destdir == ""} {return -1}
	if {[file exists [file join $destdir "VIDEO_TS.IFO"]]} {
		set r [tk_messageBox -title $utiltitle -icon question \
				-message "File [file join $destdir "VIDEO_TS.IFO"] already exists!\nOverwrite ?" \
				-parent . -type yesno -default no]
		if {$r == "no"} {return -1}
	}

	# copy the new VIDEO_TS.IFO to the destination folder
	set tempdata [readFile [file join $::config(apppath) "lib/data/new_dvd_video_ts.ifo"] binary]
	set err [saveFile [file join $destdir "VIDEO_TS.IFO"] binary $tempdata]
	if {$err != 0} {return $err}
	set err [saveFile [file join $destdir "VIDEO_TS.BUP"] binary $tempdata]
	if {$err != 0} {return $err}

	set ::dvd_modified false
	return [::main::Read_New_DVD $destdir]
}
proc copy_from_data_dir {source destination} {
	set data [readFile $::config(apppath)/lib/data/$source binary]
	saveFile $destination binary $data
}

#############################################################################
## Procedure:  set_provider_ID
## Show/Change the provider ID
proc {set_provider_ID} {} {
	if {[info exists ::raw_sectors(0,VMGM_MAT)]} {
		# Get old Provider ID (if any)
		set oldid [get_provider_ID]
		# get new ID
		set newid [::stringreq::stringreq -window .getid -def $oldid -ok "Set ID" \
				-callback ::utils::set_provider_ID_callback -width 32 \
				-text "Enter the DVD Provider ID:" -title "PgcEdit" -geometry w \
				-cancelreturn "***CANCELLED***"]
		if {$newid != "***CANCELLED***"} {
			set newid [string trim $newid " "]
			set newid "$newid[string repeat \000 [expr {32-[string length $newid]}]]"
			set ::raw_sectors(0,VMGM_MAT) \
					[string replace $::raw_sectors(0,VMGM_MAT) 0X40 0X5F $newid]
			::main::set_window_title
			::main::dvd_modified true false
		}
	} else {
		bell
	}
}
proc {set_provider_ID_callback} {str} {
	if {[string length $str] > 32} {
		bell
		return 0
	} else {
		return 1
	}
}
proc {get_provider_ID} {} {
	if {[info exists ::raw_sectors(0,VMGM_MAT)]} {
		set id [string range $::raw_sectors(0,VMGM_MAT) 0X40 0X5F]
		set stopchar [string first \000 $id]
		if {$stopchar != -1} {
			set id [string range $id 0 [expr {$stopchar - 1}]]
		}
		return [string trim $id " "]
	} else {
		return ""
	}
}

#############################################################################
## Procedure:  set_General_Name
## Show/Set/Change the DVD-Text General Name
proc {set_General_Name_GUI} {} {
	set utiltitle "Set DVD-Text General Name and Movie Name"
	if {![info exists ::dvddir]} {
		bell
		return
	}
	if {$::raw_sectors(0,VMG_TXTDT_MG) != ""} {
		set newsec false
		set newmsg "VMG_TXTDT_MG (DVD-TEXT data sector) already exists."
		set oldid [get_General_Name]
		set maxlen [get_General_Name_max_length]
		if {$maxlen < 1} {
			set text "Can't find the DVD-TEXT General Name or Movie Name.\n\n"
			set text "${text}If you really want to proceed, you need to remove\n"
			set text "${text}the current DVD-TEXT data sector and retry..."
			tk_messageBox -title $utiltitle -icon warning -message $text
			return
		}
	} else {
		set newsec true
		set newmsg "The VMG_TXTDT_MG (DVD-TEXT data sector) will be created."
		set ::raw_sectors(0,VMG_TXTDT_MG) \
				[readFile $::config(apppath)/lib/data/vmg_txtdt_mg_default.sec binary]
		set oldid ""
		set maxlen 30
	}

	# get new ID
	set newid [::stringreq::stringreq -window .getid -def $oldid -ok "Set Name" \
			-callback "::utils::set_general_name_callback $maxlen" -width $maxlen \
			-text "$newmsg\n\nEnter the DVD-TEXT General Name\n(Max $maxlen characters)" \
			-title $utiltitle -geometry w \
			-cancelreturn "***CANCELLED***"]
	if {$newid != "***CANCELLED***"} {
		set_General_Name $newid $maxlen
		::main::set_window_title
		::main::dvd_modified true false
	} elseif {$newsec} {
		set ::raw_sectors(0,VMG_TXTDT_MG) ""
	}
}
proc {set_general_name_callback} {maxlen str} {
	if {[string length $str] > $maxlen} {
		bell
		return 0
	} else {
		return 1
	}
}
# return one of the Volume General Name, Volume Movie Name,
# Title 1 General Name or Title 1 Movie Name
proc {get_General_Name} {} {
	if {![info exists ::dvddir]} {return ""}
	if {$::raw_sectors(0,VMG_TXTDT_MG) == ""} {return ""}

	set name ""
	set numentries [get_int ::raw_sectors(0,VMG_TXTDT_MG) 0xE8 2]
	set type 0
	set volume_name_idx  0
	set volume_movie_idx 0
	set title_name_idx   0
	set title_movie_idx  0
	for {set i 0} {$i < $numentries} {incr i} {
		set idx [expr {(0xEC + ($i * 4))}]
		set entry [get_int ::raw_sectors(0,VMG_TXTDT_MG) $idx 1]
		if {$entry < 48} {set type $entry; continue}
		incr idx 2
		if {$type == 1 && $entry == 48} {
			set volume_name_idx [get_int ::raw_sectors(0,VMG_TXTDT_MG) $idx 2]
			continue
		}
		if {$type == 1 && $entry == 57} {
			set volume_movie_idx [get_int ::raw_sectors(0,VMG_TXTDT_MG) $idx 2]
			continue
		}
		if {$type == 2 && $entry == 48 && $title_name_idx == 0} {
			set title_name_idx [get_int ::raw_sectors(0,VMG_TXTDT_MG) $idx 2]
			continue
		}
		if {$type == 2 && $entry == 57 && $title_movie_idx == 0} {
			set title_movie_idx [get_int ::raw_sectors(0,VMG_TXTDT_MG) $idx 2]
			continue
		}
	}
	set nameidx 0
	if {$title_movie_idx > 0}  {set nameidx $title_movie_idx}
	if {$title_name_idx > 0}   {set nameidx $title_name_idx}
	if {$volume_movie_idx > 0} {set nameidx $volume_movie_idx}
	if {$volume_name_idx > 0}  {set nameidx $volume_name_idx}
	if {$nameidx == 0} {return ""}

	incr nameidx 0xE8
	set name [string range $::raw_sectors(0,VMG_TXTDT_MG) $nameidx [expr {$nameidx + 32}]]
	set stopchar [string first \000 $name]
	if {$stopchar != -1} {
		set name [string range $name 0 [expr {$stopchar - 1}]]
	}
	set stopchar [string first \011 $name]
	if {$stopchar != -1} {
		set name [string range $name 0 [expr {$stopchar - 1}]]
	}
	set name [string trim $name " "]
	return $name
}

proc {get_General_Name_max_length} {} {
	if {![info exists ::dvddir]} {return 0}

	set numentries [get_int ::raw_sectors(0,VMG_TXTDT_MG) 0xE8 2]
	set type 0
	set volume_name_idx  0
	set volume_movie_idx 0
	for {set i 0} {$i < $numentries} {incr i} {
		set idx [expr {(0xEC + ($i * 4))}]
		set entry [get_int ::raw_sectors(0,VMG_TXTDT_MG) $idx 1]
		if {$entry < 48} {set type $entry; continue}
		incr idx 2
		if {$type == 1 && $entry == 48} {
			set volume_name_idx [get_int ::raw_sectors(0,VMG_TXTDT_MG) $idx 2]
			continue
		}
		if {$type == 1 && $entry == 57} {
			set volume_movie_idx [get_int ::raw_sectors(0,VMG_TXTDT_MG) $idx 2]
			continue
		}
	}
	if {$volume_movie_idx == 0 && $volume_name_idx == 0} {bell; return 0}
	set max1 30
	set max2 30
	if {$volume_movie_idx > 0} {
		set max1 [get_General_Name_max_length_sub [expr {$volume_movie_idx + 0xE8}]]
	}
	if {$volume_name_idx > 0}  {
		set max2 [get_General_Name_max_length_sub [expr {$volume_name_idx + 0xE8}]]
	}

	if {$max1 <= $max2} {
		return $max1
	} else {
		return $max2
	}
}
proc get_General_Name_max_length_sub {nameidx} {
	set name [string range $::raw_sectors(0,VMG_TXTDT_MG) $nameidx [expr {$nameidx + 30}]]
	set stopchar1 [string first \000 $name]
	if {$stopchar1 == -1} {set stopchar1 30}
	set stopchar2 [string first \011 $name]
	if {$stopchar2 == -1} {set stopchar2 30}
	if {$stopchar1 < $stopchar2} {
		set stopchar $stopchar1
	} else {
		set stopchar $stopchar2
	}
	for {set i [expr {$stopchar+1}]} {$i < 30} {incr i} {
		if {[string range ${name} $i $i] != "\000"} {
			return [expr {$i-1}]
		}
	}
	return 30
}

proc {set_General_Name} {name maxlen} {
	if {![info exists ::dvddir]} {return 0}

	set numentries [get_int ::raw_sectors(0,VMG_TXTDT_MG) 0xE8 2]
	set type 0
	set volume_name_idx  0
	set volume_movie_idx 0
	for {set i 0} {$i < $numentries} {incr i} {
		set idx [expr {(0xEC + ($i * 4))}]
		set entry [get_int ::raw_sectors(0,VMG_TXTDT_MG) $idx 1]
		if {$entry < 48} {set type $entry; continue}
		incr idx 2
		if {$type == 1 && $entry == 48} {
			set volume_name_idx [get_int ::raw_sectors(0,VMG_TXTDT_MG) $idx 2]
			continue
		}
		if {$type == 1 && $entry == 57} {
			set volume_movie_idx [get_int ::raw_sectors(0,VMG_TXTDT_MG) $idx 2]
			continue
		}
	}
	if {$volume_movie_idx == 0 && $volume_name_idx == 0} {bell; return 0}
	if {$volume_movie_idx > 0} {
		set_general_name_at_address $name [expr {$volume_movie_idx + 0xE8}] $maxlen
	}
	if {$volume_name_idx > 0}  {
		set_general_name_at_address $name [expr {$volume_name_idx + 0xE8}] $maxlen
	}
	return 1
}
proc set_general_name_at_address {name address maxlen} {
	set name $name\011
	incr maxlen
	while {[string length $name] < $maxlen} {set name $name\000}

	set data1 [string range $::raw_sectors(0,VMG_TXTDT_MG) 0 [expr {$address-1}]]
	set data2 [string range $::raw_sectors(0,VMG_TXTDT_MG) [expr {$address+$maxlen}] end]
	set ::raw_sectors(0,VMG_TXTDT_MG) $data1$name$data2
}

#############################################################################
## Remove the DVD-TEXT datas (the whole sector is erased)
proc {remove_dvd_text_data} {} {
	if {![info exists ::dvddir]} {
		bell
		return
	}
	if {$::raw_sectors(0,VMG_TXTDT_MG) != ""} {
		if {!$::config(expert_mode)} {
			set msg "This will remove the whole VMG_TXTDT_MG\n(DVD-TEXT data sector), not only the\nGeneral Name!\n\nAre you sure ?"
			set r [tk_messageBox -title "Remove DVD-TEXT data" -icon question -message $msg \
					-parent . -type okcancel]
		} else {
			set r "ok"
		}
		if {$r == "ok"} {
			set ::raw_sectors(0,VMG_TXTDT_MG) ""
			::main::set_window_title
			::main::dvd_modified true false
		}
	} else {
		show_warning -title "Remove DVD-TEXT data" -icon error -parent . \
				-message "There is no VMG_TXTDT_MG (DVD-TEXT\ndata sector) in this DVD."
	}
}

#############################################################################
## Delete unused VOB file or move to backup or (optionally) leave it.
## Args: the VOB filename, without path.
##       leave_ok boolean: Add the "Leave it there" button for no action.
proc delete_vob_with_confirm {filename leave_ok} {
	set filename [file tail $filename]
	set vobfile [file join $::dvddir $filename]

	if {[file exists $vobfile]} {
		set txt "Warning: there is an old $filename"
		set txt "$txt\nin your DVD folder."
		set txt "$txt\n\nWhat do you want to do?"
		if {$leave_ok} {
			set r [tk_dialog .confirm "PgcEdit" $txt "warning" 0 \
					" Backup file " " Delete file " "Leave it there"]
		} else {
			set r [tk_dialog .confirm "PgcEdit" $txt "warning" 0 \
					" Backup file " " Delete file "]
		}
		switch $r {
			0 {
				if {! [file exists $::backupdir]} {makedir $::backupdir "PgcEdit: Create Backup"}
				if {[file exists $::backupdir]} {
					set doit true
					if {[file exists [file join $::backupdir $filename]]} {
						set txt "Warning: There is already a $filename"
						set txt "$txt\nin the backup folder.\n\nOverwrite?"
						set txt "$txt\n\nIf you answer No, the current $filename"
						set txt "$txt\nwill be erased without backup."
						set r [tk_messageBox -title "PgcEdit" -icon warning \
								-message $txt -parent . -type yesno -default no]
						if {$r == "no"} {
							set doit false
						}
					}
					if {$doit} {
						movefile $vobfile $::backupdir -force "PgcEdit: Delete VOB" .
					} else {
						deletefile $vobfile "PgcEdit: Delete VOB" .
					}
				}
			}
			1 {
				deletefile $vobfile "PgcEdit: Delete VOB" .
			}
		}
	}
}

#########################################################
# find_results
proc find_results {searchstring case regexpr {buttons_only false}} {
	if {! [info exists ::dvddir]} {
		bell
		return -1
	}

	# check expression sybtax
	if {$searchstring == ""} {
		show_msg "Type a search string first!"
		return -1
	}
	if {$::main::searchregexp} {
		set err [catch {set found [regexp -- $searchstring "blah blah"]} errstr]
		if {$err != 0} {
			bell
			show_msg [string map {": " ":\n"} $errstr]
			return -1
		}
	}
	if {$buttons_only} {set in "in buttons"} else {set in "in DVD"}
	show_msg "Searching for \"$searchstring\" $in..."

	set w .findresults
	if {![winfo exists $w]} {
		set already_waiting false
		toplevel $w -class Dialog
		wm withdraw $w
		if {$::config(tk) != "aqua"} {wm transient $w .}
		if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
		wm protocol $w WM_DELETE_WINDOW {set ::utils::findresults_done 0}
		bind $w <Escape> "set ::utils::findresults_done 0"
	
		frame  $w.f -borderwidth 2 -relief groove
		pack   $w.f -expand 1 -fill both -padx 4 -pady 3
		listbox $w.f.c -listvariable ::utils::find_results_display -yscrollcommand "$w.f.vs set" \
				-height 40 -width 100 -font $::config(global_font)
		pack   $w.f.c -side left -expand 1 -fill both
		bind   $w.f.c <ButtonRelease-1> {::utils::gotoref}
		scrollbar $w.f.vs -command "$w.f.c yview" -orient vert
		pack   $w.f.vs -side left -fill y

		frame  $w.bf
		pack   $w.bf -fill x -pady 3
		button $w.bf.help -text "Help" -width 10 -command "::utils::find_results_help $w"
		pack   $w.bf.help -side left -padx 4
		button $w.bf.cancel -text "Close" -width 10 -command {set ::utils::findresults_done 0}
		pack   $w.bf.cancel -side right -padx 4
		button $w.bf.refresh -text "Refresh" -width 10
		pack   $w.bf.refresh -side right
		if {$::config(platform) == "mac"} {
			pack [frame $w.fb.macpad] -side left -padx 6
		}
	} else {
		set already_waiting true
		set ::config(findresults_win_geometry) [wm geometry $w]
	}
	$w.bf.refresh configure -command "::utils::find_results $searchstring $case $regexpr $buttons_only"
	bind $w <Return> "::utils::find_results $searchstring $case $regexpr $buttons_only"
	wm title $w "Searching for \"$searchstring\" $in."

	set ::utils::find_results_display {{Please wait...}}
	catch {$w.f.c itemconfigure 0 -background {}}
	after 100 {set ::tmp 1}
	update idletasks
	tkwait variable ::tmp
	unset ::tmp

	set progress_max [expr ($::pgcs(numvts)+1)*2]
	set refs 0
	set ::utils::find_results_display {}
	set ::utils::find_results_commands {}
	if {! $buttons_only} {
		show_msg "Searching for \"$searchstring\" $in: FP-PGC" 1 100
		set refs [add_matches $refs pgc 0 -1 0 $searchstring $case $regexpr]
	}
	for {set vts 0} {$vts <= $::pgcs(numvts)} {incr vts} {
		show_msg "Searching for \"$searchstring\" $in: VTSM $vts" [expr ($vts*2)+1] $progress_max
		for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
				if {! $buttons_only} {
					set refs [add_matches $refs menu $vts $lu $pgc $searchstring $case $regexpr]
				}
				if {[info exists ::menus($vts,$lu,$pgc,numbuttons)] && $::menus($vts,$lu,$pgc,numbuttons) > 0} {
					set refs [add_buttons_matches $refs menu $vts $lu $pgc $searchstring $case $regexpr]
				}
			}
		}
		if {$vts > 0} {
			show_msg "Searching for \"$searchstring\" $in: VTST $vts" [expr ($vts*2)+2] $progress_max
			for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
				if {! $buttons_only} {
					set refs [add_matches $refs pgc $vts -1 $pgc $searchstring $case $regexpr]
				}
				if {[info exists ::pgcs($vts,$pgc,numbuttons)] && $::pgcs($vts,$pgc,numbuttons) > 0} {
					set refs [add_buttons_matches $refs pgc $vts -1 $pgc $searchstring $case $regexpr]
				}
			}
		}
	}
	show_msg "Found \"$searchstring\" $refs times $in." 0
	set i [expr [llength $::utils::find_results_display] -1]
	set ::utils::find_results_display [lreplace $::utils::find_results_display $i $i]

	set lbh [llength $::utils::find_results_display]
	if {$lbh < 10} {
		set lbh 10
	} elseif {$lbh > 30} {
		set lbh 30
	}
	set lbw 0
	foreach i $::utils::find_results_display {
		set linesize [font measure $::config(global_font) $i]
		if {$linesize > $lbw} {
			set lbw $linesize
		}
	}
	set tmp "00000000000000000000"
	while {[font measure $::config(global_font) $tmp] <= $lbw} {
		set tmp "${tmp}0"
	}
	set lbw [string length $tmp]
	$w.f.c configure -width $lbw -height $lbh

	update
	set width [winfo reqwidth $w]
	set height [winfo reqheight $w]
	wm minsize $w 280 100
	wm maxsize $w $width 2000
	set wx [expr {[winfo rootx .]+(([winfo width  .]-$width)/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	$w.f.c configure -width 10 -height 4
	if {[info exists ::config(findresults_win_geometry)]} {
		wm geometry $w $::config(findresults_win_geometry)
	} else {
		wm geometry $w ${width}x$height+$wx+$wy
	}
	wm geometry $w ${width}x$height
	wm deiconify $w
	focus $w
	show_msg ""
	wm title $w "Found \"$searchstring\" $refs times $in."
	after 50 {set ::tmp 1}
	update idletasks
	tkwait variable ::tmp
	unset ::tmp

	if {$already_waiting} {return $refs}
	tkwait variable ::utils::findresults_done
	destroy $w
	unset -nocomplain ::utils::find_results_display ::utils::find_results_commands
	return $refs
}

proc add_matches {matchnum type vts lu pgc searchstring case regexpr} {
	if {$type == "menu"} {
		set cmds $::menus($vts,$lu,$pgc)
		set label [::main::pgc_label $::config(global_font) menu $vts $lu $pgc]
	} else {
		set cmds $::pgcs($vts,$pgc)
		set label [::main::pgc_label $::config(global_font) pgc $vts $pgc]
	}
	foreach section {pre post cell} {
		set scmds [get_type_cmds $section cmds]
		set scmds [linsert $scmds 0 "* $section"]
		set linenum [::main::find_in_cmds $searchstring $scmds -forwards $case $regexpr 1]
		while {$linenum > -1} {
			set targetline $linenum
			lappend ::utils::find_results_display " $label  -  $section command $targetline :"
			lappend ::utils::find_results_display "                [::disasm::disasm [lindex $scmds $linenum]]"
			lappend ::utils::find_results_display ""
			lappend ::utils::find_results_commands "::utils::gotoxref \"${type}s $vts $lu $pgc $section $targetline\""
			.findresults.f.c itemconfigure [expr {$matchnum*3}] -background [button_color ${type}s]
			.findresults.f.c itemconfigure [expr {($matchnum*3)+1}] -background [button_color ${type}s]
			.findresults.f.c itemconfigure [expr {($matchnum*3)+2}] -selectbackground $::config(system_window_color)
			incr matchnum
			set linenum [::main::find_in_cmds $searchstring $scmds -forwards \
					$case $regexpr [expr $linenum + 1]]
		}
	}
	return $matchnum
}


proc add_buttons_matches {matchnum type vts lu pgc searchstring case regexpr} {
	if {$type == "menu"} {
		set numcells $::menus($vts,$lu,$pgc,numcells)
		if {$numcells == 0} {return $matchnum}
		set cellpostable $::menus($vts,$lu,$pgc,cellpostable)
		set label [::main::pgc_label $::config(global_font) menu $vts $lu $pgc]
		::menubuttons::get_menu_buttons_infos $vts menu
		set reftype "menu_button"
	} else {
		set numcells $::pgcs($vts,$pgc,numcells)
		if {$numcells == 0} {return $matchnum}
		set cellpostable $::pgcs($vts,$pgc,cellpostable)
		set label [::main::pgc_label $::config(global_font) pgc $vts $pgc]
		::menubuttons::get_menu_buttons_infos $vts pgc
		set reftype "BOV"
	}
	for {set cell 1} {$cell <= $numcells} {incr cell} {
		set idx [expr {($cell-1) * 4}]
		set vobid [get_int cellpostable $idx 2]
		incr idx 3
		set cellid [get_int cellpostable $idx 1]
		if {[info exists ::buttons::curbutsinfo($vobid,$cellid,numbuts)]} {
			set numbuts $::buttons::curbutsinfo($vobid,$cellid,numbuts)
			if {$numbuts == 0} {continue}
			set numgroups $::buttons::curbutsinfo($vobid,$cellid,numgroups)
			if {$numgroups == 0} {continue}
			for {set butnum 1} {$butnum <= $numbuts} {incr butnum} {
				for {set group 1} {$group <= $numgroups} {incr group} {
					set hex [string toupper $::buttons::curbutsinfo($vobid,$cellid,$group,$butnum,command)]
					set linenum [::main::find_in_cmds $searchstring "\{$hex\}" -forwards $case $regexpr 0]
					if {$linenum > -1} {
						lappend ::utils::find_results_display " $label  -  cell $cell, group $group, button $butnum :"
						lappend ::utils::find_results_display "                [::disasm::disasm $hex]"
						lappend ::utils::find_results_display ""
						lappend ::utils::find_results_commands "::utils::gotoxref \"${reftype}s $vts $lu $pgc $cell $group $butnum\""
						.findresults.f.c itemconfigure [expr {$matchnum*3}] -background [button_color ${reftype}s]
						.findresults.f.c itemconfigure [expr {($matchnum*3)+1}] -background [button_color ${reftype}s]
						.findresults.f.c itemconfigure [expr {($matchnum*3)+2}] -selectbackground $::config(system_window_color)
						incr matchnum
					}
				}
			}
		}
	}
	return $matchnum
}

proc gotoref {} {
	set w .findresults.f.c
	set idx [$w index @[expr {[winfo pointerx .]-[winfo rootx $w]}],[expr {[winfo pointery .]-[winfo rooty $w]}]]
	if {[expr {$idx % 3}] == 2} return
	set idx [expr $idx / 3]
	if {$idx <= [llength $::utils::find_results_commands]} {
		eval [lindex $::utils::find_results_commands $idx]
	} else {
		bell
	}
}

#########################################################
# Show Jumps from VMGM to current VTS
proc vts_refs {} {
	if {! [info exists ::dvddir]} {
		bell
		return 0
	}
	set vts [lindex [split [lindex $::editor_cmds_from 1] ,] 0]
	if {$vts == 0} {
		bell
		return 0
	}

	unset -nocomplain ::crossrefs_calculated
	unset -nocomplain ::crossrefs
	build_crossrefs $vts

	set w .vtsxrefs
	if {[winfo exists $w]} {
		destroy $w
	}
	toplevel $w -class Dialog
	wm withdraw $w
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm protocol $w WM_DELETE_WINDOW {set ::utils::vtsxrefs_done 0}
	bind $w <Escape> "set ::utils::vtsxrefs_done 0"

	frame  $w.f -borderwidth 2 -relief groove
	pack   $w.f -expand 1 -fill both -padx 4 -pady 3
	canvas $w.f.c -yscrollcommand "$w.f.vs set"
	pack   $w.f.c -side left -expand 1 -fill both
	scrollbar $w.f.vs -command "$w.f.c yview" -orient vert
	pack   $w.f.vs -side left -fill y
	frame $w.f.c.f1

	wm title $w "Jumps from FP-PGC and VMGM to VTS $vts"

	set found 0
	if {$::menus($vts,numlus) > 0} {
		for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
				if {$::crossrefs(menus,$vts,$lu,$pgc,numrefs) > 0} {
					set curtarget [::main::pgc_label "" menu $vts $lu $pgc]
					for {set ref 1} {$ref <= $::crossrefs(menus,$vts,$lu,$pgc,numrefs)} {incr ref} {
						if {[vts_of $::crossrefs(menus,$vts,$lu,$pgc,$ref)] != $vts} {
							incr found
							set txt [string trim "[display_ref $::crossrefs(menus,$vts,$lu,$pgc,$ref)]\nto: $curtarget" \n]
							pack [button $w.f.c.f1.but$found -text $txt \
								-command "::utils::gotoxref \"$::crossrefs(menus,$vts,$lu,$pgc,$ref)\"" \
								-justify left -anchor w -pady 0  -background [button_color \
								$::crossrefs(menus,$vts,$lu,$pgc,$ref)]] -anchor w -fill x -ipadx 2
						}
					}
				}
			}
		}
	}
	for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
		if {$::crossrefs(pgcs,$vts,$pgc,numrefs) > 0} {
			set curtarget [::main::pgc_label "" pgc $vts $pgc]
			for {set ref 1} {$ref <= $::crossrefs(pgcs,$vts,$pgc,numrefs)} {incr ref} {
				if {[vts_of $::crossrefs(pgcs,$vts,$pgc,$ref)] != $vts} {
					incr found
					set txt [string trim "[display_ref $::crossrefs(pgcs,$vts,$pgc,$ref)]\nto: $curtarget" \n]
					pack [button $w.f.c.f1.but$found -text $txt \
						-command "::utils::gotoxref \"$::crossrefs(pgcs,$vts,$pgc,$ref)\"" \
						-justify left -anchor w -pady 0  -background [button_color \
						$::crossrefs(pgcs,$vts,$pgc,$ref)]] -anchor w -fill x -ipadx 2
				}
			}
		}
	}
	set txt "Found $found references from FP-PGC or VMGM to VTS $vts"
	pack [label $w.f.c.f1.fl -text $txt] -anchor w

	pack  $w.f.c.f1 -anchor nw -fill both -expand 1
	$w.f.c create window 0 0 -window $w.f.c.f1 -anchor nw

	# inform scrollbar of the height of the canvas's internal window
	update
	$w.f.c configure -scrollregion "0 0 0 [winfo height $w.f.c.f1]" \
			-width [winfo width $w.f.c.f1]

	# buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 3 -fill x
	button $w.fb.help -text "?" -command "::utils::vts_refs_help $w"
	pack   $w.fb.help -side left -padx 2
	button $w.fb.cancel -text "Close" -width 10 -command {set ::utils::vtsxrefs_done 0}
	pack   $w.fb.cancel -side right -padx 2
	if {$::config(platform) == "mac"} {
		pack [frame $w.fb.macpad] -side left -padx 6
	}

	update
	set width [winfo reqwidth $w]
	if {$width < 300} {set width 300}
	wm minsize $w $width [winfo reqheight $w]
	wm maxsize $w $width 2000
	if {[info exists ::config(vtsxrefs_win_geometry)]} {
		wm geometry $w $::config(vtsxrefs_win_geometry)
	} else {
		set wx [expr {[winfo rootx .]+(([winfo width  .]-$width)/2)}]
		set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
		if {$wx < 10} {set wx 10}
		if {$wy < 22} {set wy 22}
		wm geometry $w +$wx+$wy
	}
	wm deiconify $w
	focus $w
	#grab set $w

	update_progress 0

	tkwait variable ::utils::vtsxrefs_done
	set ::config(vtsxrefs_win_geometry) [wm geometry $w] 
	destroy $w
	return $found
}
proc vts_of {ref} {
	set type [lindex $ref 0]
	switch $type {
		"menus" -
		"pgcs" -
		"menu_buttons" -
		"BOVs" {set idx 1}
		default {set idx 2}
	}
	return [lindex $ref $idx]
}

#########################################################
# Build Calls cross references table
proc build_crossrefs {only_vts} {
	show_msg "Building cross references..."
	if {![info exists ::crossrefs(pgcs,0,0,numrefs)]} {
		for {set vts 0} {$vts <= $::pgcs(numvts)} {incr vts} {
			for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
				for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
					set ::crossrefs(menus,$vts,$lu,$pgc,numrefs) 0
				}
			}
			for {set pgc 0} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
				set ::crossrefs(pgcs,$vts,$pgc,numrefs) 0
			}
		}
	}
	set progress_max [expr ($::pgcs(numvts)*2)+2]
	set progress_cur 0
	for {set vts 0} {$vts <= $::pgcs(numvts)} {incr vts} {
		if {$only_vts != -1 && $vts != 0 && $vts != $only_vts} continue
		if {[info exists ::crossrefs_calculated($vts)]} continue
		if {$vts == 0} {set name "VMGM"} else {set name "VTSM $vts"}
		incr progress_cur
		show_msg "Building cross references: $name..." $progress_cur $progress_max
		for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
				add_link_refs "menu" $vts $lu $pgc

				set numcmds [llength $::menus($vts,$lu,$pgc)]
				set linenum 0
				set section "pre"
				for {set c 1} {$c < $numcmds} {incr c} {
					set cmd [lindex $::menus($vts,$lu,$pgc) $c]
					if {[lindex $cmd 0] == "*"} {
						set linenum 0
						set section [lindex $cmd 1]
					} else {
						incr linenum
						set r [::main::highlight_jump_cmd menu $vts,$lu,$pgc [string toupper \[$cmd\]] [expr {$c+1}] false false]
						#set debugout "$debugout\nmenus $vts $lu $pgc $section $linenum \[$cmd\] : $r"
						add_crossref $r [list "menus" $vts $lu $pgc $section $linenum $cmd]
					}
				}

				set numcells $::menus($vts,$lu,$pgc,numcells)
				set cellpostable $::menus($vts,$lu,$pgc,cellpostable)
				for {set cell 0} {$cell < $numcells} {incr cell} {
					set idx [expr {$cell * 4}]
					set vobid [get_int cellpostable $idx 2]
					incr idx 3
					set cellid [get_int cellpostable $idx 1]
					if {[info exists ::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)]} {
						set numbuts $::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)
						if {$numbuts > 0} {
							set numgroups $::buttons::butsinfo(menu,$vts,$vobid,$cellid,numgroups)
							for {set butnum 1} {$butnum <= $numbuts} {incr butnum} {
								for {set group 1} {$group <= $numgroups} {incr group} {
									set cmd $::buttons::butsinfo(menu,$vts,$vobid,$cellid,$group,$butnum,command)
									set r [::main::highlight_jump_cmd menu $vts,$lu,$pgc \[$cmd\] [expr {$c+1}] false false]
									add_crossref $r [list "menu_buttons" $vts $lu $pgc [expr {$cell+1}] $group $butnum $cmd]
								}
							}
						}
					}
				}
			}
		}
		update_progress 0
		if {$vts == 0} {
			incr progress_cur
			show_msg "Building cross references: FP-PGC..." $progress_cur $progress_max
			set numcmds [llength $::pgcs($vts,0)]
			set linenum 0
			set section "pre"
			for {set c 1} {$c < $numcmds} {incr c} {
				set cmd [lindex $::pgcs($vts,0) $c]
				if {[lindex $cmd 0] == "*"} {
					set linenum 0
					set section [lindex $cmd 1]
				} else {
					incr linenum
					set r [::main::highlight_jump_cmd pgc $vts,0 [string toupper \[$cmd\]] [expr {$c+1}] false false]
					add_crossref $r [list "pgcs" $vts 0 0 $section $linenum $cmd]
				}
			}
		} else {
			incr progress_cur
			show_msg "Building cross references: VTST $vts..." $progress_cur $progress_max
			for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
				add_link_refs "pgc" $vts 0 $pgc

				set numcmds [llength $::pgcs($vts,$pgc)]
				set linenum 0
				set section "pre"
				for {set c 1} {$c < $numcmds} {incr c} {
					set cmd [lindex $::pgcs($vts,$pgc) $c]
					if {[lindex $cmd 0] == "*"} {
						set linenum 0
						set section [lindex $cmd 1]
					} else {
						incr linenum
						set r [::main::highlight_jump_cmd vts $vts,$pgc [string toupper \[$cmd\]] [expr {$c+1}] false false]
						add_crossref $r [list "pgcs" $vts 0 $pgc $section $linenum $cmd]
					}
				}

				set numcells $::pgcs($vts,$pgc,numcells)
				set cellpostable $::pgcs($vts,$pgc,cellpostable)
				for {set cell 0} {$cell < $numcells} {incr cell} {
					set idx [expr {$cell * 4}]
					set vobid [get_int cellpostable $idx 2]
					incr idx 3
					set cellid [get_int cellpostable $idx 1]
					if {[info exists ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)]} {
						set numbuts $::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)
						if {$numbuts > 0} {
							set numgroups $::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numgroups)
							for {set butnum 1} {$butnum <= $numbuts} {incr butnum} {
								for {set group 1} {$group <= $numgroups} {incr group} {
									set cmd $::buttons::butsinfo(pgc,$vts,$vobid,$cellid,$group,$butnum,command)
									set r [::main::highlight_jump_cmd pgc $vts,$pgc \[$cmd\] [expr {$c+1}] false false]
									add_crossref $r [list "BOVs" $vts 0 $pgc [expr {$cell+1}] $group $butnum $cmd]
								}
							}
						}
					}
				}
			}
		}
		set ::crossrefs_calculated($vts) true
	}
	show_msg ""
}

proc add_crossref {targetlist refslist} {
	if {$targetlist == ""} return

	set targettype		[lindex $targetlist 0]
	set targetvts		[lindex $targetlist 1]
	set targetlu		[lindex $targetlist 2]
	set targetpgc		[lindex $targetlist 3]
	set targetvalid     [lindex $targetlist 4]
	if {!$targetvalid} {
		lappend ::crossrefs(invalidrefs) $refslist
		return
	}

	# remove the links to same PGC
	set reftype [lindex $refslist 0]
	if {$reftype == "pgcs"} {
		set reftype pgc
	} else {
		set reftype menu
	}
	if {	$targettype	== $reftype
			&& $targetvts == [lindex $refslist 1]
			&& $targetpgc == [lindex $refslist 3]} {
		return
	}

	if {$targettype == "menu"} {
		if {$targetlu == "all"} {
			for {set tlu 1} {$tlu <= $::menus($targetvts,numlus)} {incr tlu} {
				catch {incr ::crossrefs(menus,$targetvts,$tlu,$targetpgc,numrefs)}
				set ::crossrefs(menus,$targetvts,$tlu,$targetpgc,$::crossrefs(menus,$targetvts,$tlu,$targetpgc,numrefs)) $refslist
			}
		} else {
			catch {incr ::crossrefs(menus,$targetvts,$targetlu,$targetpgc,numrefs)}
			set ::crossrefs(menus,$targetvts,$targetlu,$targetpgc,$::crossrefs(menus,$targetvts,$targetlu,$targetpgc,numrefs)) $refslist
		}
	} else {
		catch {incr ::crossrefs(pgcs,$targetvts,$targetpgc,numrefs)}
		set ::crossrefs(pgcs,$targetvts,$targetpgc,$::crossrefs(pgcs,$targetvts,$targetpgc,numrefs)) $refslist
	}
}
proc add_link_refs {type vts lu pgc} {
	if {$type == "menu"} {
		set header $::menus($vts,$lu,$pgc,header)
	} else {
		set header $::pgcs($vts,$pgc,header)
	}

	set nextpgcn [get_int header 0x9C 2]
	if {$nextpgcn > 0 && $nextpgcn != $pgc} {
		if {$type == "menu"} {
			if {[catch {incr ::crossrefs(menus,$vts,$lu,$nextpgcn,numrefs)}]} {
				set ::crossrefs(menus,$vts,$lu,$nextpgcn,numrefs) 1
			}
			set ::crossrefs(menus,$vts,$lu,$nextpgcn,$::crossrefs(menus,$vts,$lu,$nextpgcn,numrefs)) [list "nextpgcn" menu $vts $lu $pgc $nextpgcn]
			if {$nextpgcn > $::menus($vts,$lu,numpgcs)} {
				lappend ::crossrefs(invalidrefs) [list "nextpgcn" menu $vts $lu $pgc $nextpgcn]
			}
		} else {
			if {[catch {incr ::crossrefs(pgcs,$vts,$nextpgcn,numrefs)}]} {
				set ::crossrefs(pgcs,$vts,$nextpgcn,numrefs) 1
			}
			set ::crossrefs(pgcs,$vts,$nextpgcn,$::crossrefs(pgcs,$vts,$nextpgcn,numrefs)) [list "nextpgcn" pgc $vts $lu $pgc $nextpgcn]
			if {$nextpgcn > $::pgcs($vts,numpgcs)} {
				lappend ::crossrefs(invalidrefs) [list "nextpgcn" pgc $vts $lu $pgc $nextpgcn]
			}
		}
	}

	set prevpgcn [get_int header 0x9E 2]
	if {$prevpgcn > 0 && $prevpgcn != $pgc} {
		if {$type == "menu"} {
			if {[catch {incr ::crossrefs(menus,$vts,$lu,$prevpgcn,numrefs)}]} {
				set ::crossrefs(menus,$vts,$lu,$prevpgcn,numrefs) 1
			}
			set ::crossrefs(menus,$vts,$lu,$prevpgcn,$::crossrefs(menus,$vts,$lu,$prevpgcn,numrefs)) [list "prevpgcn" menu $vts $lu $pgc $prevpgcn]
			if {$prevpgcn > $::menus($vts,$lu,numpgcs)} {
				lappend ::crossrefs(invalidrefs) [list "prevpgcn" menu $vts $lu $pgc $prevpgcn]
			}
		} else {
			if {[catch {incr ::crossrefs(pgcs,$vts,$prevpgcn,numrefs)}]} {
				set ::crossrefs(pgcs,$vts,$prevpgcn,numrefs) 1
			}
			set ::crossrefs(pgcs,$vts,$prevpgcn,$::crossrefs(pgcs,$vts,$prevpgcn,numrefs)) [list "prevpgcn" pgc $vts $lu $pgc $prevpgcn]
			if {$prevpgcn > $::pgcs($vts,numpgcs)} {
				lappend ::crossrefs(invalidrefs) [list "prevpgcn" pgc $vts $lu $pgc $prevpgcn]
			}
		}
	}

	set gouppgcn [get_int header 0xA0 2]
	if {$gouppgcn > 0 && $gouppgcn != $pgc && $gouppgcn != 65535} {
		if {$type == "menu"} {
			if {[catch {incr ::crossrefs(menus,$vts,$lu,$gouppgcn,numrefs)}]} {
				set ::crossrefs(menus,$vts,$lu,$gouppgcn,numrefs) 1
			}
			set ::crossrefs(menus,$vts,$lu,$gouppgcn,$::crossrefs(menus,$vts,$lu,$gouppgcn,numrefs)) [list "gouppgcn" menu $vts $lu $pgc $gouppgcn]
			if {$gouppgcn > $::menus($vts,$lu,numpgcs) && $gouppgcn != 255} {
				lappend ::crossrefs(invalidrefs) [list "gouppgcn" menu $vts $lu $pgc $gouppgcn]
			}
		} else {
			if {[catch {incr ::crossrefs(pgcs,$vts,$gouppgcn,numrefs)}]} {
				set ::crossrefs(pgcs,$vts,$gouppgcn,numrefs) 1
			}
			set ::crossrefs(pgcs,$vts,$gouppgcn,$::crossrefs(pgcs,$vts,$gouppgcn,numrefs)) [list "gouppgcn" pgc $vts $lu $pgc $gouppgcn]
			if {$gouppgcn > $::pgcs($vts,numpgcs)} {
				lappend ::crossrefs(invalidrefs) [list "gouppgcn" pgc $vts $lu $pgc $gouppgcn]
			}
		}
	}
}
#########################################################
# Show Calls cross references table
proc show_crossrefs {} {
	if {! [info exists ::dvddir]} {
		bell
		return ""
	}

	set msg "Do you want to delete the VM commands"
	set msg "$msg\nand clear the Prev/Next/GoUpPGC links"
	set msg "$msg\nin the unreferenced PGCs?"
	set msg "$msg\n(Useful to find more unreferenced PGCs"
	set msg "$msg\nin subsequent usages of this function.)\n"
	set msg "$msg\nCAUTION:  Backup your IFOs first!\n"
	set clearcmds [tk_messageBox -title "Calls Cross References" -icon question \
			-message $msg -parent . -type yesnocancel -default yes]
	if {$clearcmds == "cancel"} {return ""}
	if {$clearcmds == "yes"} {
		set clearcmds true
	} else {
		set clearcmds false
	}
	set convert_to_dummies -1
	set num_unreferred_pgcs 0
	set num_dummies 0
	set pinklines {}

	unset -nocomplain ::crossrefs_calculated
	unset -nocomplain ::crossrefs
	set ::crossrefs(invalidrefs) {}
	build_crossrefs -1

	::main::init_tree_menu_no_move	;# clear old highlights

	set unreferenced ""
	set txt "Calls cross references"
	set txt "$txt\nDVD-TEXT General Name: \"[::utils::get_General_Name]\""
	set txt "$txt\nProvider ID: \"[::utils::get_provider_ID]\""
	set txt "$txt\nNumber of VTS: $::pgcs(numvts)\n"

	show_msg "Dumping cross references: FP-PGC" 100
	set txt "$txt\n################################ First-Play PGC ################################"
	set txt "$txt\n[::main::pgc_label "" pgc 0 0]:"
	set txt "$txt\n    Entry PGC."
	for {set ref 1} {$ref <= $::crossrefs(pgcs,0,0,numrefs)} {incr ref} {
		set txt $txt[display_ref $::crossrefs(pgcs,0,0,$ref)]
	}
	set txt "$txt\n"

	for {set vts 0} {$vts <= $::pgcs(numvts)} {incr vts} {
		if {$vts == 0} {set name "VMGM"} else {set name "VTSM $vts"}
		show_msg "Dumping cross references: $name..." 100
		set label "[string repeat "#" [expr {(78 - [string length $name]) / 2}]] $name "
		set label $label[string repeat "#" [expr {80 - [string length $label]}]]

		if {$vts == 0} {
			set no_referenced_titles false
		} else {
			set no_referenced_titles true
			for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
				if {$::crossrefs(pgcs,$vts,$pgc,numrefs) > 0} {
					set no_referenced_titles false
					break
				}
			}
		}

		if {$::menus($vts,numlus) > 0} {
			set txt "$txt\n$label"
			for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
				for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
					set found 0
					set txt "$txt\n[::main::pgc_label "" menu $vts $lu $pgc]:"
					if {$::menus($vts,$lu,$pgc,masktext) != ""} {
						if {!$no_referenced_titles} {
							set callable [menuIsCallableWithRemote $vts $::menus($vts,$lu,$pgc,mask)]
							if {$callable} {
								set txt "$txt\n    Callable with remote control $::menus($vts,$lu,$pgc,masktext)enu button."
								set found 1
							} else {
								set txt "$txt\n    NOT callable with remote control $::menus($vts,$lu,$pgc,masktext)enu button due to PUOs!"
							}
						}
					}
					if {$::crossrefs(menus,$vts,$lu,$pgc,numrefs) > 0} {
						for {set ref 1} {$ref <= $::crossrefs(menus,$vts,$lu,$pgc,numrefs)} {incr ref} {
							set txt $txt[display_ref $::crossrefs(menus,$vts,$lu,$pgc,$ref)]
						}
						set found 1
					}
					if {$found} {
						::utils::remove_from_pgc_comment menu $vts $lu $pgc "<unref>"
					} else {
						incr num_unreferred_pgcs
						set txt "$txt\n    *** UNREFERENCED! ***"
						set unreferenced "$unreferenced[::main::pgc_label $::config(fixed_width_font) menu $vts $lu $pgc]\n"
						set lab [::main::pgc_label $::config(global_font) menu $vts $lu $pgc]
						set item [lsearch $::main::labelslist $lab]
						lappend pinklines $item
						if {$clearcmds} {
							if {$::menus($vts,$lu,$pgc,numcells) > 0} {
								if {$convert_to_dummies == -1} {
									::main::display_pgc menu $vts $lu $pgc
									after 100 {set ::pause 1}
									tkwait variable ::pause
									set msg "This menu PGC with associated video is unreferenced:"
									set msg "$msg\n[::main::pgc_label "" menu $vts $lu $pgc]."
									set msg "$msg\n\nDo you want to convert it to a dummy PGC?"
									set msg "$msg\n(Useful to easily blank it with FixVTS or\nVobBlanker.)\n"
									set msg "$msg\nCAUTION: Subsequent unreferenced menu PGCs,"
									set msg "$msg\neven with menu buttons, will be converted to"
									set msg "$msg\ndummies as well, without further confirmation!"
									set r [tk_messageBox -title "Calls Cross References" -icon question \
											-message $msg -parent . -type yesno -default yes]
									if {$r == "yes"} {
										set convert_to_dummies 1
									} else {
										set convert_to_dummies 0
									}
								}
								if {$convert_to_dummies} {
									incr num_dummies
									set ::menus($vts,$lu,$pgc,numprograms) 0
									set ::menus($vts,$lu,$pgc,programmap) ""
									set ::menus($vts,$lu,$pgc,numcells) 0
									set ::menus($vts,$lu,$pgc,cellpbtable) ""
									set ::menus($vts,$lu,$pgc,cellpostable) ""
								}
							}
							set ::menus($vts,$lu,$pgc) {{* pre} {* post} {* cell}}
							set_int ::menus($vts,$lu,$pgc,header) 0x9C 0 2
							set_int ::menus($vts,$lu,$pgc,header) 0x9E 0 2
							set_int ::menus($vts,$lu,$pgc,header) 0xA0 0 2
							::main::dvd_modified true false
						}
						::utils::add_to_pgc_comment menu $vts $lu $pgc "<unref>" false
					}
					set txt "$txt\n"
				}
			}
		}
		if {$vts > 0} {
			show_msg "Dumping cross references: VTST $vts..." 100
			set label "[string repeat "#" [expr {(78 - [string length "VTST $vts"]) / 2}]] VTST $vts "
			set label $label[string repeat "#" [expr {80 - [string length $label]}]]
			set txt "$txt\n$label"
			for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
				set txt "$txt\n[::main::pgc_label "" pgc $vts $pgc]:"
				if {$::crossrefs(pgcs,$vts,$pgc,numrefs) > 0} {
					for {set ref 1} {$ref <= $::crossrefs(pgcs,$vts,$pgc,numrefs)} {incr ref} {
						set txt $txt[display_ref $::crossrefs(pgcs,$vts,$pgc,$ref)]
					}
					::utils::remove_from_pgc_comment pgc $vts 0 $pgc "<unref>"
				} else {
					incr num_unreferred_pgcs
					set txt "$txt\n    *** UNREFERENCED! ***"
					set unreferenced "$unreferenced[::main::pgc_label $::config(fixed_width_font) pgc $vts $pgc]\n"
					set lab [::main::pgc_label $::config(global_font) pgc $vts $pgc]
					set item [lsearch $::main::labelslist $lab]
					lappend pinklines $item
					if {$clearcmds} {
						set ::pgcs($vts,$pgc) {{* pre} {* post} {* cell}}
						set_int ::pgcs($vts,$pgc,header) 0x9C 0 2
						set_int ::pgcs($vts,$pgc,header) 0x9E 0 2
						set_int ::pgcs($vts,$pgc,header) 0xA0 0 2
						::main::dvd_modified true false
					}
					::utils::add_to_pgc_comment pgc $vts 0 $pgc "<unref>" false
				}
				set txt "$txt\n"
			}
		}
	}
	show_msg "Dumping cross references: Unreferenced PGCs..." 100
	set txt "$txt\n############################### Unreferenced PGCs ##############################"
	if {$unreferenced == ""} {
		set txt "${txt}\nNone."
	} else {
		set txt "$txt\n$unreferenced"
		if {$clearcmds} {
			set txt "$txt\nCleared $num_unreferred_pgcs unreferenced PGCs (including $num_dummies menu PGCs converted to dummy)."
		} else {
			set txt "$txt\nFound $num_unreferred_pgcs unreferenced PGCs."
		}
	}
	show_msg "Dumping cross references: Illegal references..." 100
	set txt "$txt\n\n############################### Jumps to nowhere! ##############################"
	if {$::crossrefs(invalidrefs) == ""} {
		set txt "${txt}\nNone."
	} else {
		bell
		set txt "${txt}\nJumps, Calls and Links to non-existing PGCs or Titles:"
		foreach ref $::crossrefs(invalidrefs) {
			set txt "$txt[display_ref $ref]"
		}
	}

	if {$clearcmds} {
		set type  [lindex $::editor_cmds_from 0]
		set indexes [split [lindex $::editor_cmds_from 1] ","]
		set vts [lindex $indexes 0]
		if {$type == "menu"} {
			::main::display_pgc $type $vts [lindex $indexes 1] [lindex $indexes 2]
		} else {
			::main::display_pgc $type $vts [lindex $indexes 1]
		}
		show_msg "Cleared $num_unreferred_pgcs unreferenced PGCs (including $num_dummies menu PGCs converted to dummy)"
	} else {
		show_msg "Found $num_unreferred_pgcs unreferenced PGCs"
	}
	::main::init_tree_menu_no_move
	foreach item $pinklines {
		.mf.f2.selector.a.lb itemconfigure $item -background #FFE0E0
	}

	::main::showtext .crossrefs "Calls cross references" $txt
	return
}
proc display_ref {ref} {
	switch [lindex $ref 0] {
		"menus" {
			set txt "\n    [::main::pgc_label "" menu [lindex $ref 1] [lindex $ref 2] [lindex $ref 3]], [lindex $ref 4] command [lindex $ref 5]:"
			set txt "$txt\n	[::disasm::disasm [lindex $ref 6]]"
		}
		"pgcs" {
			set txt "\n    [::main::pgc_label "" pgc [lindex $ref 1] [lindex $ref 3]], [lindex $ref 4] command [lindex $ref 5]:"
			set txt "$txt\n	[::disasm::disasm [lindex $ref 6]]"
		}
		"menu_buttons" {
			set txt "\n    [::main::pgc_label "" menu [lindex $ref 1] [lindex $ref 2] [lindex $ref 3]], cell [lindex $ref 4], group [lindex $ref 5], button [lindex $ref 6]:"
			set txt "$txt\n	[::disasm::disasm [lindex $ref 7]]"
		}
		"BOVs" {
			set txt "\n    [::main::pgc_label "" pgc [lindex $ref 1] [lindex $ref 3]], cell [lindex $ref 4], group [lindex $ref 5], BOV [lindex $ref 6]:"
			set txt "$txt\n	[::disasm::disasm [lindex $ref 7]]"
		}
		"nextpgcn" {
			if {[lindex $ref 1] == "pgc"} {
				set txt "\n    [::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 4]]:  NextPGC link to PGC [lindex $ref 5]"
			} else {
				set txt "\n    [::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 3] [lindex $ref 4]]:  NextPGC link to PGC [lindex $ref 5]"
			}
		}
		"prevpgcn" {
			if {[lindex $ref 1] == "pgc"} {
				set txt "\n    [::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 4]]:  PrevPGC link to PGC [lindex $ref 5]"
			} else {
				set txt "\n    [::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 3] [lindex $ref 4]]:  PrevPGC link to PGC [lindex $ref 5]"
			}
		}
		"gouppgcn" {
			if {[lindex $ref 1] == "pgc"} {
				set txt "\n    [::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 4]]:  GoUpPGC link to PGC [lindex $ref 5]"
			} else {
				set txt "\n    [::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 3] [lindex $ref 4]]:  GoUpPGC link to PGC [lindex $ref 5]"
			}
		}
	}
	return $txt
}

#########################################################
# Jump to crossref
proc pgc_crossrefs {{illegalrefs false}} {
	if {! [info exists ::dvddir]} {
		bell
		return ""
	}

	if {!$illegalrefs} {
		set type [lindex $::editor_cmds_from 0]
		set indexes [split [lindex $::editor_cmds_from 1] ","]
		set vts [lindex $indexes 0]
		if {$type == "menu"} {
			set lu [lindex $indexes 1]
			set pgc [lindex $indexes 2]
			set returncmd "::main::display_pgc menu $vts $lu $pgc"
		} else {
			set lu 0
			set pgc [lindex $indexes 1]
			set returncmd "::main::display_pgc pgc $vts $pgc"
		}
	} else {
		set returncmd ""
	}

	set w .pgcxrefs
	if {![winfo exists $w]} {
		unset -nocomplain ::crossrefs_calculated
		unset -nocomplain ::crossrefs
		set ::crossrefs(invalidrefs) {}

		toplevel $w -class Dialog
		wm withdraw $w
		if {$::config(tk) != "aqua"} {wm transient $w .}
		if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
		wm protocol $w WM_DELETE_WINDOW {set ::utils::pgcxrefs_done 0}
		bind $w <Escape> "set ::utils::pgcxrefs_done 0"
	
		frame  $w.f -borderwidth 2 -relief groove
		pack   $w.f -expand 1 -fill both -padx 4 -pady 3
		canvas $w.f.c -yscrollcommand "$w.f.vs set"
		pack   $w.f.c -side left -expand 1 -fill both
		scrollbar $w.f.vs -command "$w.f.c yview" -orient vert
		pack   $w.f.vs -side left -fill y
	} else {
		set ::config(pgcxrefs_win_geometry) [wm geometry $w] 
		destroy $w.f.c.f1
		label  $w.f.c.tmp -text "Building cross references.\nPlease wait..."
		pack   $w.f.c.tmp
		update idletasks
	}

	frame $w.f.c.f1

	if {$illegalrefs} {
		wm title $w "Jumps, Calls and Links to nowhere!"
		build_crossrefs -1

		pack [label $w.f.c.f1.lab1 -text "Jumps, Calls and Links to non-existing PGCs or Titles :"]
		if {$::crossrefs(invalidrefs) == ""} {
			pack [label $w.f.c.f1.lab2 -text "None." -anchor w] -fill x
		} else {
			set nref 1
			set old [pgc_ref [lindex $::crossrefs(invalidrefs) 0]]
			set old [string range $old 0 [string first , $old]]
			foreach ref $::crossrefs(invalidrefs) {
				set txt [pgc_ref $ref]
				if {[string range $txt 0 [string first , $txt]]!=$old} {
					pack [frame $w.f.c.f1.pad$ref -height 6]
					set old [string range $txt 0 [string first , $txt]]
				}
				pack [button $w.f.c.f1.but$nref -text [pgc_ref $ref] \
					-command "::utils::gotoxref \"$ref\"" \
					-justify left -anchor w -pady 0  -background [button_color $ref]] \
					-anchor w -fill x -ipadx 2
				incr nref
			}
		}


	} else {
		if {$type=="pgc"} {
			wm title $w [::main::pgc_label {} $type $vts $pgc]
		} else {
			wm title $w [::main::pgc_label {} $type $vts $lu $pgc]
		}
		if {$vts == 0} {
			build_crossrefs -1
		} else {
			build_crossrefs $vts
		}
		if {$type=="pgc"} {
			if {$vts==0} {
				pack [label $w.f.c.f1.lab1 -text "Entry PGC."] -anchor w -padx 2
			}
			if {$::crossrefs(pgcs,$vts,$pgc,numrefs)>0} {
				set old [pgc_ref $::crossrefs(pgcs,$vts,$pgc,1)]
				set old [string range $old 0 [string first , $old]]
				for {set ref 1} {$ref <= $::crossrefs(pgcs,$vts,$pgc,numrefs)} {incr ref} {
					set txt [pgc_ref $::crossrefs(pgcs,$vts,$pgc,$ref)]
					if {[string range $txt 0 [string first , $txt]]!=$old} {
						pack [frame $w.f.c.f1.pad$ref -height 6]
						set old [string range $txt 0 [string first , $txt]]
					}
					pack [button $w.f.c.f1.but$ref -text $txt \
						-command "::utils::gotoxref \"$::crossrefs(pgcs,$vts,$pgc,$ref)\"" \
						-justify left -anchor w -pady 0  -background [button_color \
						$::crossrefs(pgcs,$vts,$pgc,$ref)]] -anchor w -fill x -ipadx 2
				}
			} elseif {! [winfo exists $w.f.c.f1.lab1]} {
				pack [label $w.f.c.f1.lab1 -text "*** UNREFERENCED! ***"]
			}
		} else {
			if {$vts == 0} {
				set no_referenced_titles false
			} else {
				set no_referenced_titles true
				for {set p 1} {$p <= $::pgcs($vts,numpgcs)} {incr p} {
					if {$::crossrefs(pgcs,$vts,$p,numrefs) > 0} {
						set no_referenced_titles false
						break
					}
				}
			}
			if {$::menus($vts,$lu,$pgc,masktext) != ""} {
				set callable [menuIsCallableWithRemote $vts $::menus($vts,$lu,$pgc,mask)]
				if {$no_referenced_titles} {
					set txt "Normally callable with remote control $::menus($vts,$lu,$pgc,masktext)enu button.\n"
					set txt "${txt}But since no titles in VTS $vts are referenced, this menu\ncannot actually be called with the remote."
				} else {
					if {$callable} {
						set txt "Callable with remote control $::menus($vts,$lu,$pgc,masktext)enu button."
					} else {
						set txt "NOT callable with remote control $::menus($vts,$lu,$pgc,masktext)enu button due to PUOs!"
					}
				}
				pack [label $w.f.c.f1.lab1 -text $txt -justify left] -anchor w -padx 2
			}
			if {$::crossrefs(menus,$vts,$lu,$pgc,numrefs)>0} {
				set old [pgc_ref $::crossrefs(menus,$vts,$lu,$pgc,1)]
				set old [string range $old 0 [string first , $old]]
				for {set ref 1} {$ref <= $::crossrefs(menus,$vts,$lu,$pgc,numrefs)} {incr ref} {
					set txt [pgc_ref $::crossrefs(menus,$vts,$lu,$pgc,$ref)]
					if {[string range $txt 0 [string first , $txt]]!=$old} {
						pack [frame $w.f.c.f1.pad$ref -height 6]
						set old [string range $txt 0 [string first , $txt]]
					}
					pack [button $w.f.c.f1.but$ref -text $txt \
						-command "::utils::gotoxref \"$::crossrefs(menus,$vts,$lu,$pgc,$ref)\"" \
						-justify left -anchor w -pady 0 -background [button_color \
						$::crossrefs(menus,$vts,$lu,$pgc,$ref)]] -anchor w -fill x -ipadx 2
				}
			} elseif {! [winfo exists $w.f.c.f1.lab1]} {
				pack [label $w.f.c.f1.lab1 -text "*** UNREFERENCED! ***"] -padx 2
			}
		}
	}



	pack  $w.f.c.f1 -anchor nw -fill both -expand 1
	$w.f.c create window 0 0 -window $w.f.c.f1 -anchor nw

	# inform scrollbar of the height of the canvas's internal window
	update
	$w.f.c configure -scrollregion "0 0 0 [winfo height $w.f.c.f1]" \
			-width [winfo width $w.f.c.f1]

	# buttons
	if {! [winfo exists $w.fb]} {
		frame  $w.fb
		pack   $w.fb -padx 2 -pady 3 -fill x
		button $w.fb.help -text "?" -command "::utils::pgc_crossrefs_help $w"
		pack   $w.fb.help -side left -padx 2
		button $w.fb.return -text "Return to target"
		pack   $w.fb.return -side left -padx 2
		button $w.fb.new -text "New Target" -width 10 -command {
				set ::utils::pgcxrefs_done -1
				after 100 {set ::utils::pause 1}
				tkwait variable ::utils::pause
				::utils::pgc_crossrefs
			}
		pack   $w.fb.new -side left -padx 2
		button $w.fb.refresh -text "Refresh" -width 10 -command {
				set ::utils::pgcxrefs_done -1
				after 100 {set ::utils::pause 1}
				tkwait variable ::utils::pause
				unset -nocomplain ::crossrefs_calculated
				unset -nocomplain ::crossrefs
				::utils::pgc_crossrefs $illegalrefs
			}
		pack   $w.fb.refresh -side left -padx 2
		button $w.fb.cancel -text "Close" -width 10 -command {set ::utils::pgcxrefs_done 0}
		pack   $w.fb.cancel -side right -padx 2
		if {$::config(platform) == "mac"} {
			pack [frame $w.fb.macpad] -side left -padx 6
		}
	} else {
		catch {destroy $w.f.c.tmp}
	}
	if {$illegalrefs} {
		$w.fb.return configure -state disabled -command bell
	} else {
		$w.fb.return configure -state normal -command $returncmd
	}

	update
	set width [winfo reqwidth $w]
	if {$width < 300} {set width 300}
	wm minsize $w $width [winfo reqheight $w]
	wm maxsize $w $width 2000
	if {[info exists ::config(pgcxrefs_win_geometry)]} {
		wm geometry $w $::config(pgcxrefs_win_geometry)
	} else {
		set wx [expr {[winfo rootx .]+(([winfo width  .]-$width)/2)}]
		set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
		if {$wx < 10} {set wx 10}
		if {$wy < 22} {set wy 22}
		wm geometry $w +$wx+$wy
	}
	wm deiconify $w
	focus $w
	#grab set $w

	set ok false
	while {!$ok} {
		tkwait variable ::utils::pgcxrefs_done
		if {$::utils::pgcxrefs_done != -1} {
			set ok true
		}
	}
	if {[winfo exists $w]} {
		set ::config(pgcxrefs_win_geometry) [wm geometry $w] 
		destroy $w
	}
	return
}

proc button_color {ref} {
	switch [lindex $ref 0] {
		menus {return #ddeedd}
		pgcs  {return #ddddee}
		menu_buttons {return #eedddd}
		BOVs {return #eeddee}
		default {return #eeeeee}
	}
}

proc pgc_ref {ref} {
	switch [lindex $ref 0] {
		"menus" {
			set txt "[::main::pgc_label "" menu [lindex $ref 1] [lindex $ref 2] [lindex $ref 3]], [lindex $ref 4] command [lindex $ref 5]"
			set txt "$txt\n[::disasm::disasm [lindex $ref 6]]"
		}
		"pgcs" {
			set txt "[::main::pgc_label "" pgc [lindex $ref 1] [lindex $ref 3]], [lindex $ref 4] command [lindex $ref 5]"
			set txt "$txt\n[::disasm::disasm [lindex $ref 6]]"
		}
		"menu_buttons" {
			set txt "[::main::pgc_label "" menu [lindex $ref 1] [lindex $ref 2] [lindex $ref 3]], cell [lindex $ref 4], group [lindex $ref 5], button [lindex $ref 6]"
			set txt "$txt\n[::disasm::disasm [lindex $ref 7]]"
		}
		"BOVs" {
			set txt "[::main::pgc_label "" pgc [lindex $ref 1] [lindex $ref 3]], cell [lindex $ref 4], group [lindex $ref 5], BOV [lindex $ref 6]"
			set txt "$txt\n[::disasm::disasm [lindex $ref 7]]"
		}
		"nextpgcn" {
			if {[lindex $ref 1] == "pgc"} {
				set txt "[::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 4]],  NextPGC link."
			} else {
				set txt "[::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 3] [lindex $ref 4]],  NextPGC link."
			}
		}
		"prevpgcn" {
			if {[lindex $ref 1] == "pgc"} {
				set txt "[::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 4]],  PrevPGC link."
			} else {
				set txt "[::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 3] [lindex $ref 4]],  PrevPGC link."
			}
		}
		"gouppgcn" {
			if {[lindex $ref 1] == "pgc"} {
				set txt "[::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 4]],  GoUpPGC link."
			} else {
				set txt "[::main::pgc_label "" [lindex $ref 1] [lindex $ref 2] [lindex $ref 3] [lindex $ref 4]],  GoUpPGC link."
			}
		}
	}
	return $txt
}

proc gotoxref {ref} {
	#destroy .pgcxrefs
	#set ::utils::pgcxrefs_done 1
	switch [lindex $ref 0] {
		"menus" {
			::main::display_pgc menu [lindex $ref 1] [lindex $ref 2] [lindex $ref 3]
			switch [lindex $ref 4] {
				pre {
					set linenum [lindex $ref 5]
				}
				post {
					set cmds $::menus([lindex $ref 1],[lindex $ref 2],[lindex $ref 3])
					set linenum [expr {[num_type_cmds pre cmds] + [lindex $ref 5] +1}]
				}
				cell {
					set cmds $::menus([lindex $ref 1],[lindex $ref 2],[lindex $ref 3])
					set linenum [expr {[num_type_cmds pre cmds] + [num_type_cmds post cmds] + [lindex $ref 5] +2}]
				}
			}
			::main::set_listbox_selection $linenum
		}
		"pgcs" {
			::main::display_pgc pgc [lindex $ref 1] [lindex $ref 3]
			switch [lindex $ref 4] {
				pre {
					set linenum [lindex $ref 5]
				}
				post {
					set cmds $::pgcs([lindex $ref 1],[lindex $ref 3])
					set linenum [expr {[num_type_cmds pre cmds] + [lindex $ref 5] +1}]
				}
				cell {
					set cmds $::pgcs([lindex $ref 1],[lindex $ref 3])
					set linenum [expr {[num_type_cmds pre cmds] + [num_type_cmds post cmds] + [lindex $ref 5] +2}]
				}
			}
			::main::set_listbox_selection $linenum
		}
		"menu_buttons" {
			unset -nocomplain ::buttons::groupnum
			::main::display_pgc menu [lindex $ref 1] [lindex $ref 2] [lindex $ref 3]
			::menubuttons::display_menu_buttons [lindex $ref 1] [lindex $ref 2] [lindex $ref 3] [expr {[lindex $ref 6]*-1024}] info [lindex $ref 4] [lindex $ref 5]
		}
		"BOVs" {
			unset -nocomplain ::buttons::groupnum
			::main::display_pgc pgc [lindex $ref 1] [lindex $ref 3]
			::menubuttons::display_BOVs [lindex $ref 1] [lindex $ref 3] [expr {[lindex $ref 6]*-1024}] info [lindex $ref 4] [lindex $ref 5]
		}
		"nextpgcn" -
		"prevpgcn" -
		"gouppgcn" {
			if {[lindex $ref 1] == "pgc"} {
				::main::display_pgc pgc [lindex $ref 2] [lindex $ref 4]
			} else {
				::main::display_pgc menu [lindex $ref 2] [lindex $ref 3] [lindex $ref 4]
			}
			::main::action_Edit_PGC
		}
	}
}
proc find_results_help {w} {
	set txt "This window lists all occurences of your search string,"
	set txt "$txt\nincluding in menu buttons and BOVs."
	set txt "$txt\nClicking on a coloured line will jump to that occurence."
	set txt "$txt\nYou may continue working when this window is opened."
	set txt "$txt\nClick on the Refresh button to rescan the DVD."
	set txt "$txt\n\nThe colors are used to identify the provenance of the\ncommand:"
	set txt "$txt\nBlue:  PGC command in a Title domain or FP-PGC."
	set txt "$txt\nGreen:  PGC command in a menu domain."
	set txt "$txt\nPink:  Menu button command."
	set txt "$txt\nPurple:  BOV command (menu button in title domain)."
	set txt "$txt\n\nNote: launching the search function by clicking on the"
	set txt "$txt\n\"All\" button with the right mouse button will search in"
	set txt "$txt\nthe menu button and BOV commands only."
	tk_messageBox -title "Find all occurences in DVD" -message $txt -icon info -parent $w
}
proc pgc_crossrefs_help {w} {
	set txt "This window lists all commands or links jumping to the"
	set txt "$txt\ntarget PGC, including in menu buttons and BOVs."
	set txt "$txt\nClick on a coloured button to jump to that command."
	set txt "$txt\nClick on Return to Target to reselect the original PGC."
	set txt "$txt\nYou may continue working when this window is opened."
	set txt "$txt\nClick on the Refresh button to rescan the DVD."
	set txt "$txt\nClick on New Target to search the jumps to the currently\nselected PGC."
	set txt "$txt\n\nThe colors are used to identify the provenance of the\ncommand:"
	set txt "$txt\nBlue:  PGC command in a Title domain or FP-PGC."
	set txt "$txt\nGreen:  PGC command in a menu domain."
	set txt "$txt\nPink:  Menu button command."
	set txt "$txt\nPurple:  BOV command (menu button in title domain)."
	set txt "$txt\nGray:  PrevPGC, NextPGC or GoUpPGC link, in any PGC."
	tk_messageBox -title "Go to Calling Command" -message $txt -icon info -parent $w
}
proc vts_refs_help {w} {
	set txt "This window lists all commands or links jumping from the"
	set txt "$txt\nFirst Play PGC or the VMGM menu PGCs to any PGC of"
	set txt "$txt\nthe current titleset (menu and title domains)."
	set txt "$txt\nThe jumps from the VTS to itself are not displayed."
	set txt "$txt\nYou can use it to check if the VTS is unreferenced,"
	set txt "$txt\nor to remove all jumps to it to make it unreferenced."
	set txt "$txt\n\nWhen a VTS is unreferenced, you can safely delete it."
	set txt "$txt\nTo delete a VTS, it must be at the end of the DVD."
	set txt "$txt\nUse DVD -> Remap Titlesets to move the VTS at the end."
	set txt "$txt\nThe Title numbers in the VTS must also be the last ones"
	set txt "$txt\nof the DVD.  Use Title -> Remap Titles Numbers."
	set txt "$txt\n\nThe colors are used to identify the provenance of the\ncommand:"
	set txt "$txt\nBlue:  PGC command in the First-Play PGC."
	set txt "$txt\nGreen:  PGC command in the VMGM menu domain."
	set txt "$txt\nPink:  Menu button command."
	tk_messageBox -title "Jumps from VMG to current VTS" -message $txt -icon info -parent $w
}

proc menuIsCallableWithRemote {vts menumask} {
	set menumask [expr $menumask & 7]
	if {$vts == 0} {
		set firstvts 1
		set lastvts  $::pgcs(numvts)
	} else {
		set firstvts $vts
		set lastvts  $vts
	}
	for {set v $firstvts} {$v <= $lastvts} {incr v} {
		for {set p 1} {$p <= $::pgcs($v,numpgcs)} {incr p} {
			set PUOs [get_int ::pgcs($v,$p,header) 8 4]
			switch $menumask {
				2 {set puo [expr $PUOs & 1024]}
				3 {set puo [expr $PUOs & 2048]}
				4 {set puo [expr $PUOs & 4096]}
				5 {set puo [expr $PUOs & 8192]}
				6 {set puo [expr $PUOs & 16384]}
				7 {set puo [expr $PUOs & 32768]}
				default {set puo 1}
			}
			if {$puo == 0} {
				return true
			}
		}
		if {$menumask != 3} {
			for {set l 1} {$l <= $::menus($v,numlus)} {incr l} {
				for {set p 1} {$p <= $::menus($v,$l,numpgcs)} {incr p} {
					if {$::menus($v,$l,$p,numcells) > 0} {
						set PUOs [get_int ::menus($v,$l,$p,header) 8 4]
						switch $menumask {
							2 {set puo [expr $PUOs & 1024]}
							3 {set puo [expr $PUOs & 2048]}
							4 {set puo [expr $PUOs & 4096]}
							5 {set puo [expr $PUOs & 8192]}
							6 {set puo [expr $PUOs & 16384]}
							7 {set puo [expr $PUOs & 32768]}
							default {set puo 1}
						}
						if {$puo == 0} {
							return true
						}
					}
				}
			}
		}
	}
	return false
}

proc delete_last_pgc {} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	set type [lindex $::editor_cmds_from 0]
	set indexes [split [lindex $::editor_cmds_from 1] ","]
	set vts [lindex $indexes 0]
	if {$type == "menu"} {
		set lu [lindex $indexes 1]
		set pgc $::menus($vts,$lu,numpgcs)
		::main::display_pgc $type $vts $lu $pgc
		if {$pgc == 1} {
			show_msg "Cannot remove the first PGC of a LU."
			bell
			return
		}
	} else {
		if {$vts==0} {
			show_msg "Cannot remove the First-Play PGC."
			bell
			return
		}
		set lu 0
		set pgc $::pgcs($vts,numpgcs)
		::main::display_pgc $type $vts $pgc
		if {$pgc == 1} {
			show_msg "Cannot remove the first PGC in a title domain."
			bell
			return
		}
		if {[info exists ::pgcs($vts,$pgc,Title)]} {
			set title $::pgcs($vts,$pgc,Title)
			if {$title > 0 && $title < [get_int ::raw_sectors(0,VMG_TT_SRPT) 0 2]} {
				show_msg "Can only remove the last Title of the DVD."
				bell
				return
			}
			if {$title < 2} {
				show_msg "Cannot remove the first title of the DVD."
				bell
				return
			}
		} else {
			set title 0
		}
		set numttus [get_int ::raw_sectors($vts,VTS_PTT_SRPT) 0 2]
		if {$::pgcs($vts,$pgc,TTN) != $numttus} {
			set numttnpgcs 0
			for {set p 1} {$p <= $::pgcs($vts,numpgcs)} {incr p} {
				if {$::pgcs($vts,$p,TTN) == $::pgcs($vts,$pgc,TTN)} {
					incr numttnpgcs
				}
			}
			if {$numttnpgcs <= 1} {
				show_msg "Can only remove the last TTN in the titleset."
				bell
				return
			}
		}
	}

	if {! [info exists ::warning_delete_last_pgc]} {
		set ::warning_delete_last_pgc true
		set txt "This utility will remove the LAST PGC of the current domain."
		set txt "$txt\n\nWARNING: If there are references to the PGC elsewhere in the DVD,"
		set txt "$txt\nthe playback may hang when the player will try to play the PGC!"
		set txt "$txt\n(Use the Info -> Go to Calling Command or Calls Cross References"
		set txt "$txt\nto see if there are references to this PGC)"
		set txt "$txt\n\n(Note: this warning will be displayed only once.)"
		set txt "$txt\n\nContinue anyway?"
		set r [tk_messageBox -title "Delete last PGC in domain" -icon warning -message $txt \
				-parent . -type yesno -default no]
		if {$r == "no"} return
	}

	incr pgc -1
	if {$type == "menu"} {
		set firstlu $lu
		set lastlu $lu
		if {$::menus($vts,numlus) > 1} {
			set samenumpgcs true
			set numpgcs $::menus($vts,1,numpgcs)
			for {set l 2} {$l <= $::menus($vts,numlus)} {incr l} {
				if {$::menus($vts,$lu,numpgcs) != $numpgcs} {
					set samenumpgcs false
					break
				}
			}
			if {$samenumpgcs} {
				set txt "There are several LUs in this menu domain."
				set txt "$txt\nDo you want to remove the last PGC in all LUs (recomended)?"
				set r [tk_messageBox -title "Delete last PGC in domain" -icon question -message $txt \
						-parent . -type yesnocancel -default yes]
				if {$r == "cancel"} return
				if {$r == "yes"} {
					set firstlu 1
					set lastlu $::menus($vts,numlus)
				} else {
					set ::menus($vts,lus_are_identical) false
				}
			}
		}
		for {set l $firstlu} {$l <= $lastlu} {incr l} {		
			set ::menus($vts,$l,numpgcs) $pgc
		}
		::main::init_tree_menu
		::main::display_pgc $type $vts $lu $pgc
	} else {
		set ::pgcs($vts,numpgcs) $pgc
		if {$title > 0} {
			unset -nocomplain titles($vts,$title)
			incr title -1
			set_int ::raw_sectors(0,VMG_TT_SRPT) 0 $title 2
			set ::pgcs(Title,NumTitles) $title
			unset -nocomplain ::pgcs($vts,[expr $pgc+1],Title)
			set ::raw_sectors(0,VMG_TT_SRPT) [string range $::raw_sectors(0,VMG_TT_SRPT) 0 end-12]
		} else {
			# if a dummy PGC attached to a non-sequential title is removed
			# and there is opnly one PGC left in the title,
			# and the PGC is played sequentially,
			# then mark the title as sequential in VMG_TT_SRPT.
			# TODO: should also test for non-zero Prev/Next/GoUpPGCN links
			set numttnsintitle 0
			set seqmode 0
			for {set p 1} {$p <= $::pgcs($vts,numpgcs)} {incr p} {
				if {$::pgcs($vts,$p,TTN) == $::pgcs($vts,$pgc,TTN)} {
					incr numttnsintitle
					set titlepgc $p
					set seqmode [expr $seqmode | [get_int ::pgcs($vts,$p,header) 0xA2 1]]
				}
			}
			if {$numttnsintitle == 1 && $seqmode == 0} {
				# Change the title status to non-sequential in VMG_TT_SRPT
				set table $::raw_sectors(0,VMG_TT_SRPT)
				set idx 8
				while {$idx < [string length $table]} {
					if {[get_int table [expr $idx+6] 1] == $vts && \
						[get_int table [expr $idx+7] 1] == $::pgcs($vts,$pgc,TTN)} {
						set_int table $idx [expr [get_int table $idx 1] & 0xBF] 1
						set ::raw_sectors(0,VMG_TT_SRPT) $table
						::tmap::need_new_tmap $vts $titlepgc
						break
					}
					incr idx 12
				}
			}
		}

		#incr numttus -1
		set numttus 0
		for {set p 1} {$p <= $::pgcs($vts,numpgcs)} {incr p} {
			if {$::pgcs($vts,$p,TTN) > $numttus} {
				set numttus $::pgcs($vts,$p,TTN)
			}
		}
		set_int ::raw_sectors($vts,VTS_PTT_SRPT) 0 $numttus 2

		# Remove last Time Map in VTS_TMAPT
		if {$::raw_sectors($vts,VTS_TMAPTI) != ""} {
			set numentries [get_int ::raw_sectors($vts,VTS_TMAPTI) 0 2]
			if {$numentries > $pgc} {
				set table [string range $::raw_sectors($vts,VTS_TMAPTI) 0 7][string repeat \000 [expr 4*$pgc]]
				for {set i 1} {$i <= $numentries} {incr i} {
					set idx1 [get_int ::raw_sectors($vts,VTS_TMAPTI) [expr 4+(4*$i)] 4]
					set idx2 [expr $idx1+3+([get_int ::raw_sectors($vts,VTS_TMAPTI) [expr $idx1+2] 2]*4)]
					set data($i) [string range $::raw_sectors($vts,VTS_TMAPTI) $idx1 $idx2]
				}
				while {$i <= $pgc} {
					set tmap($i) \001\000\000\000
					incr i
				}
				set_int table 0 $pgc 2	;# number of PGCs in time map
				for {set i 1} {$i <= $pgc} {incr i} {
					set_int table [expr 4+(4*$i)] [string length $table] 4
					set table $table$data($i)
				}
				set ::raw_sectors($vts,VTS_TMAPTI) $table
				unset table
			}
		}
		::main::init_tree_menu
		::main::display_pgc $type $vts $pgc
	}
	::main::dvd_modified true false
}

proc delete_last_vts {} {
	if {! [info exists ::dvddir]} {
		bell
		return
	}
	if {! $::config(expert_mode)} {
		set txt "This utility will remove the LAST TitleSet (VTS) of the DVD."
		set txt "$txt\n\nWARNING: If there are references to the VTS elsewhere in the DVD,"
		set txt "$txt\nthe playback may hang when the player will try to play a PGC in the VTS!"
		set txt "$txt\n(Use the Info -> Go to Calling Command or Calls Cross References to see"
		set txt "$txt\nif there are references to this VTS)"
		set txt "$txt\n\nThe IFO and VOB files of the last VTS will be saved to the backup folder,"
		set txt "$txt\nif these files are not already present in the backup folder."
		set txt "$txt\n\nIMPORTANT:  Don't forget to Save the DVD after the operation!"
		set txt "$txt\n\nContinue?"
		set r [tk_messageBox -title "Delete last VTS in DVD" -icon warning -message $txt \
				-parent . -type yesno -default no]
		if {$r == "no"} return
	}
	
	set lastvts $::pgcs(numvts)
	if {$lastvts <= 1} {
		set txt "Error: Cannot delete VTS 1!"
		set txt "$txt\n\nIf you want to start over without any VTS,"
		set txt "$txt\nyou should use the File -> New DVD function,"
		set txt "$txt\nand import the VMGM menu if needed."
		tk_messageBox -title "Delete last VTS in DVD" -icon error -message $txt \
				-parent . -type ok
		return
	}
	
	# shorten global table of titles
	set numtitles [get_int ::raw_sectors(0,VMG_TT_SRPT) 0 2]
	for {set i 1} {$i <= $numtitles} {incr i} {
		set title($i) 1
	}
	
	set mintitle 9999
	set lastvtshastitles 0
	for {set pgc 1} {$pgc <= $::pgcs($lastvts,numpgcs)} {incr pgc} {
		if {[info exists ::pgcs($lastvts,$pgc,Title)]} {
			set lastvtshastitles 1
			set title($::pgcs($lastvts,$pgc,Title)) 0
			if {$::pgcs($lastvts,$pgc,Title) < $mintitle} {
				set mintitle $::pgcs($lastvts,$pgc,Title)
			}
		}
	}
	if {$lastvtshastitles} {
		for {set t $mintitle} {$t <= $numtitles} {incr t} {
			if {$title($t) == 1} {
				set txt "Error: Title number $t (defined in the VMG table of title)"
				set txt "$txt\nis not in the last VTS but is higher than the lowest title"
				set txt "$txt\nnumber present in the last VTS (Title $mintitle)."
				set txt "$txt\nBefore using this function, you should remap the Title"
				set txt "$txt\nnumbers so that the last titles are all in the last VTS."
				set txt "$txt\n\nPgcEdit will now launch the Remap Title Numbers function."
				set txt "$txt\nMulti-select the titles of the last VTS, and click on the"
				set txt "$txt\n\"To Bottom\" button.  Then relaunch this function."
				tk_messageBox -title "Delete last VTS in DVD" -icon error -message $txt \
						-parent . -type ok
				after 100 {::srptinfo::remap_titles_numbers}
				return
			}
		}
		set ::raw_sectors(0,VMG_TT_SRPT) [string range $::raw_sectors(0,VMG_TT_SRPT) \
				0 [expr {7+(($mintitle-1)*12)}]]
		set_int ::raw_sectors(0,VMG_TT_SRPT) 0 [expr $mintitle-1] 2
		for {set t $mintitle} {$t <= $numtitles} {incr t} {
			unset -nocomplain ::pgcs(Title,$t)
		}
		set ::pgcs(Title,NumTitles) [expr $mintitle-1]
	}
	
	set ::pgcs(numvts)  [expr $lastvts -1]
	set ::menus(numvts) [expr $lastvts -1]

	# backup old files
	if {! [file exists $::backupdir]} {
		makedir $::backupdir "PgcEdit: Create Backup" .
	}
	set filename "VTS_[format %02d $lastvts]_0.IFO"
	if {![file exists [file join $::backupdir $filename]]} {
		movefile [file join $::dvddir $filename] [file join $::backupdir $filename]
	} else {
		file delete -force [file join $::dvddir $filename]
	}
	file delete -force [file join $::dvddir "VTS_[format %02d $lastvts]_0.BUP"]
	for {set i 0} {$i < 10} {incr i} {
		set filename "VTS_[format %02d $lastvts]_$i.VOB"
		if {[file exists [file join $::dvddir $filename]]} {
			if {![file exists [file join $::backupdir $filename]]} {
				movefile [file join $::dvddir $filename] [file join $::backupdir $filename]
			} else {
				file delete -force [file join $::dvddir $filename]
			}
		}
	}

	# remove the useless entries in the VMG_PTL_MAIT table
	fix_vts_parental_management

	# refresh
	if {[lindex [split [lindex $::editor_cmds_from 1] ","] 0] < $lastvts} {
		::main::init_tree_menu_no_move
	} else {
		::main::init_tree_menu
		::main::display_pgc pgc 0 0
	}
	show_msg "VTS $lastvts successfully removed.  Save the DVD now, and verify the Jumps to Nowhere!"
	::main::dvd_modified true false
}


proc delete_all_domain_commands {} {
	if {! [info exists ::dvddir] || $::scratchbook_active} {
		bell
		return
	}
	set type [lindex $::editor_cmds_from 0]
	set indexes [split [lindex $::editor_cmds_from 1] ","]
	set vts [lindex $indexes 0]

	if {!$::config(expert_mode)} {
		set txt "This utility will delete all VM commands"
		set txt "$txt\nin ALL PGCs of the current domain!"
		set txt "$txt\n\nAre you sure?"
		set r [tk_messageBox -title "Delete All Commands in Domain" -icon question -message $txt \
				-parent . -type yesno -default yes]
		if {$r == "no"} return
	}

	if {$type == "pgc"} {
		if {$vts == 0} {
			set ::pgcs(0,0) {{* pre} {* post} {* cell}}
		} else {
			for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
				set ::pgcs($vts,$pgc) {{* pre} {* post} {* cell}}
			}
		}
	} else {
		for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
				set ::menus($vts,$lu,$pgc) {{* pre} {* post} {* cell}}
			}
		}
	}
	set ::editor_cmds_list {{* pre} {* post} {* cell}}
	::main::refresh_listbox
	::main::set_listbox_selection 0
	::main::listbox_to_cmds
	::main::dvd_modified true false
}

proc delete_all_DVD_commands {} {
	if {! [info exists ::dvddir] || $::scratchbook_active} {
		bell
		return
	}
	set txt "This utility will delete all VM commands"
	set txt "$txt\nin ALL PGCs of the whole DVD!!!"
	set txt "$txt\n\nAre you really sure?"
	set r [tk_messageBox -title "Delete All Commands in DVD" -icon warning -message $txt \
			-parent . -type yesno -default no]
	if {$r == "no"} return

	set ::pgcs(0,0) {{* pre} {* post} {* cell}}
	for {set vts 0} {$vts <= $::pgcs(numvts)} {incr vts} {
		if {$vts > 0} {
			for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
				set ::pgcs($vts,$pgc) {{* pre} {* post} {* cell}}
			}
		}
		for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
				set ::menus($vts,$lu,$pgc) {{* pre} {* post} {* cell}}
			}
		}
	}
	set ::editor_cmds_list {{* pre} {* post} {* cell}}
	::main::refresh_listbox
	::main::set_listbox_selection 0
	::main::listbox_to_cmds
	::main::dvd_modified true false
}

proc blank_out_all_DVD_titles {} {
	if {! [info exists ::dvddir] || $::scratchbook_active} {
		bell
		return
	}
	for {set vts 1} {$vts <= $::pgcs(numvts)} {incr vts} {
		::main::display_pgc pgc $vts $::pgcs($vts,numpgcs)
		::main::display_pgc pgc $vts 1
		blank_out_PGCS_GUI
	}
}

#########################
## PGC Label functions ##
#########################

proc pgc_comment {} {
	if {$::scratchbook_active || ! [info exists ::dvddir]} {
		bell
		return
	}
	set type  [lindex $::editor_cmds_from 0]
	set index [lindex $::editor_cmds_from 1]

	set oldcomment ""
	if {$type == "pgc"} {
		if {[info exists ::pgcs_comments($index)]} {
			set oldcomment $::pgcs_comments($index)
		}
	} else {
		if {[info exists ::menus_comments($index)]} {
			set oldcomment $::menus_comments($index)
		}
	}

	set comment [::stringreq::stringreq -window .pgccomment -title "PGC comment" \
				-def $oldcomment -width 60 -geometry c -cancelreturn "-*CANCELLED*-" \
				-callback ::utils::pgc_comment_callback]
	if {$comment == "-*CANCELLED*-"} return
	if {$comment != $oldcomment} {
		set ::pgc_comments_modified true
		if {$type == "pgc"} {
			if {$comment == ""} {
				unset -nocomplain ::pgcs_comments($index)
			} else {
				set ::pgcs_comments($index) $comment
			}
		} else {
			if {$comment == ""} {
				unset -nocomplain ::menus_comments($index)
			} else {
				set ::menus_comments($index) $comment
			}
		}
		::main::init_tree_menu_no_move
	}
}
proc pgc_comment_callback {str} {
	if {[string map {\{ "" \} "" \[ "" \] "" \" ""} $str] != $str || [string length $str] > 60} {
		bell
		return false
	}
	return true
}
proc add_to_pgc_comment {type vts lu pgc comment {onlyifroom true}} {
	if {! $::config(allow_auto_comments)} return

	set oldcomment ""
	if {$type == "pgc"} {
		if {[info exists ::pgcs_comments($vts,$pgc)]} {
			set oldcomment $::pgcs_comments($vts,$pgc)
		}
	} else {
		if {[info exists ::menus_comments($vts,$lu,$pgc)]} {
			set oldcomment $::menus_comments($vts,$lu,$pgc)
		}
	}
	if {! $onlyifroom || [expr [string length $oldcomment] + [string length $comment]] < 60} {
		if {[string first $comment $oldcomment] == -1} {
			if {$oldcomment != ""} {
				set comment "$comment $oldcomment"
			}
			if {[string length $comment] > 60} {
				set comment [string range $comment 0 59]
			}
			if {$type == "pgc"} {
				set ::pgcs_comments($vts,$pgc) $comment
			} else {
				set ::menus_comments($vts,$lu,$pgc) $comment
			}
			set ::pgc_comments_modified true
		}
	}
}
proc remove_from_pgc_comment {type vts lu pgc comment} {
	if {! $::config(allow_auto_comments)} return

	set oldcomment ""
	if {$type == "pgc"} {
		if {[info exists ::pgcs_comments($vts,$pgc)]} {
			set oldcomment $::pgcs_comments($vts,$pgc)
		}
	} else {
		if {[info exists ::menus_comments($vts,$lu,$pgc)]} {
			set oldcomment $::menus_comments($vts,$lu,$pgc)
		}
	}
	set pos [string first $comment $oldcomment]
	if {$pos > -1} {
		set comment [string trim [string map {"  " " "} [string map [list $comment ""] $oldcomment]] " "]
		if {$comment == ""} {
			if {$type == "pgc"} {
				unset ::pgcs_comments($vts,$pgc)
			} else {
				unset ::menus_comments($vts,$lu,$pgc)
			}
		} else {
			if {$type == "pgc"} {
				set ::pgcs_comments($vts,$pgc) $comment
			} else {
				set ::menus_comments($vts,$lu,$pgc) $comment
			}
		}
		set ::pgc_comments_modified true
	}
}
proc clear_all_pgc_comments {} {
	unset -nocomplain ::menus_comments
	unset -nocomplain ::pgcs_comments
	::main::init_tree_menu_no_move
	set ::pgc_comments_modified false
}
proc export_pgc_comments {{auto false}} {

	if {![info exists ::dvddir]} {
		bell
		return
	}
	set out ""
	if {[info exists ::pgcs_comments(0,0)]} {
		set out "$out\nset ::pgcs_comments(0,0) [list $::pgcs_comments(0,0)]"
	}
	for {set vts 0} {$vts <= $::pgcs(numvts)} {incr vts} {
		for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
				if {[info exists ::menus_comments($vts,$lu,$pgc)]} {
					set out "$out\nset ::menus_comments($vts,$lu,$pgc) [list $::menus_comments($vts,$lu,$pgc)]"
				}
			}
		}
		if {$vts > 0} {
			for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
				if {[info exists ::pgcs_comments($vts,$pgc)]} {
					set out "$out\nset ::pgcs_comments($vts,$pgc) [list $::pgcs_comments($vts,$pgc)]"
				}
			}
		}
	}
	if {$out == ""} {
		if {! $auto} {
			tk_messageBox -title "Export user's PGC comments" -icon warning -parent . \
					-message "No user's PGC comments defined yet.\nNothing to save!"
		}
		return
	}

	set head "# User's PGC comments saved automatically by PgcEdit.\n"
	set head "${head}# Edit at your own risk!\n"
	set head "${head}#\n"
	set head "${head}# Source DVD: \"$::dvddir\"\n"
	set head "${head}# DVD-TEXT General Name: \"[get_General_Name]\"\n"
	set head "${head}# Provider ID: \"[get_provider_ID]\"\n"
	set head "${head}# Number of VTS: $::pgcs(numvts)\n"
	set out "$head$out"

	if {$auto && $::config(auto_backup_comments) && [file exists $::backupdir]} {
		set filename [file join $::backupdir "User comments.upc"]
	} else {
		set deffile [get_General_Name]
		if {$deffile == ""} {
			set deffile "User comments.upc"
		}
		set filename [tk_getSaveFile \
			-filetypes {{"User's PGC comments" {*.upc}} {"all files" {*.*}}} \
			-parent . -initialdir $::config(import_initialdir) \
			-initialfile $deffile -title "PgcEdit: Export user's PGC comments"]
		if {$filename == ""} {
			return
		}
		set ::config(import_initialdir) [file dirname $filename]
	}
	if {[string toupper [file extension $filename]] != ".UPC"} {
		set filename "${filename}.upc"
	}
	if {[saveFile $filename auto $out] == 0 && $auto} {
		set ::pgc_comments_modified false
	}
}
proc import_pgc_comments {{auto false}} {
	if {![info exists ::dvddir]} {
		bell
		return
	}
	if {$auto && $::config(auto_backup_comments)} {
		set filename [file join $::backupdir "User comments.upc"]
	} else {
		set deffile [get_General_Name]
		if {$deffile == ""} {
			set deffile "User comments.upc"
		} else {
			set deffile $deffile.upc
		}
		set filename [tk_getOpenFile \
			-filetypes {{"User's PGC comments" {*.upc}} {"all files" {*.*}}} \
			-parent . -initialdir $::config(import_initialdir) \
			-initialfile $deffile -title "PgcEdit: Import user's PGC comments"]
		if {$filename == ""} {
			return
		}
		if {[string toupper [file extension $filename]] != ".UPC"} {
			set filename "${filename}.upc"
		}
		set ::config(import_initialdir) [file dirname $filename]
	}

	if {[file exists $filename]} {
		unset -nocomplain ::menus_comments
		unset -nocomplain ::pgcs_comments
		set rc [catch {source $filename} errstr]
		if {$rc} {
			tk_messageBox -title "User's PGC comments" -icon error -parent . \
					-message "Error sourcing [file tail $filename]:\n$errstr"
		} else {
			set ::pgc_comments_modified false
		}
		::main::init_tree_menu_no_move
	}
}

# Removes or adds entries in the parental management table VMG_PTL_MAIT
# so that the number of VTS in the table matches the real number of VTS.
# When an entry is added, the value of the first entry of the same level
# is copied, therefore the new VTS will inherit the mask of the VMGM.
proc fix_vts_parental_management {} {
	set table $::raw_sectors(0,VMG_PTL_MAIT)
	if {$table == ""} return
	set numcountries [get_int table 0 2]
	if {$numcountries == 0} return
	set numvts [get_int table 2 2]
	if {$numvts == 0 || $numvts == $::pgcs(numvts)} return

	for {set country 1} {$country <= $numcountries} {incr country} {
		set idx [expr $country*8]
		set countrycode($country) [get_int table $idx 2]
		set countryidx  [get_int table [expr $idx+4] 2]
		for {set lev 0} {$lev < 8} {incr lev} {
			set levidx [expr $countryidx+($lev*(($numvts+1)*2))]
			if {$numvts >= $::pgcs(numvts)} {
				set out($country,$lev) [string range $table $levidx \
						[expr $levidx+(($::pgcs(numvts)+1)*2)-1]]
			} else {
				set out($country,$lev) [string range $table $levidx \
						[expr $levidx+(($numvts+1)*2)-1]]
				set new \000\000
				set_int new 0 [get_int out($country,$lev) 0 2] 2
				for {set i $numvts} {$i < $::pgcs(numvts)} {incr i} {
					set out($country,$lev) $out($country,$lev)$new
				}
			}
		}
	}
	set newtable \000\000\000\000\000\000\000\000
	set_int newtable 0 $numcountries 2
	set_int newtable 2 $::pgcs(numvts) 2
	for {set country 1} {$country <= $numcountries} {incr country} {
		set tmp \000\000\000\000\000\000\000\000
		set_int tmp 0 $countrycode($country) 2
		set countryoffsetidx($country) [expr [string length $newtable]+4]
		set newtable $newtable$tmp
	}
	for {set country 1} {$country <= $numcountries} {incr country} {
		set_int newtable $countryoffsetidx($country) [string length $newtable] 2
		for {set lev 0} {$lev < 8} {incr lev} {
			set newtable $newtable$out($country,$lev)
		}
	}
	set_int newtable 4 [expr [string length $newtable]-1] 4
	set ::raw_sectors(0,VMG_PTL_MAIT) $newtable
	return
}


}	;# end of namespace ::utils
